@@ -25,6 +25,8 @@ import Agda.TypeChecking.Monad hiding (withCurrentModule)
 -- import Agda.TypeChecking.Monad.Context
 -- import Agda.TypeChecking.Monad.Signature
 import Agda.TypeChecking.Substitute
+import Agda.TypeChecking.Telescope
+
 import qualified Agda.Syntax.Abstract as A
 import Agda.Syntax.Abstract.Pretty (prettyA)
 import qualified Text.PrettyPrint as PP
@@ -370,12 +372,12 @@ auto ii rng argstr = do
             case cls' of
              Left{} -> dispmsg "No solution found"
              Right cls' -> do
-              cls'' <- forM cls' $ \ (I.Clause _ tel perm ps body t) -> do
+              cls'' <- forM cls' $ \ (I.Clause _ tel ps body t catchall) -> do
                 withCurrentModule (AN.qnameModule def) $ do
                  -- Normalise the dot patterns
                  ps <- addCtxTel tel $ normalise ps
                  body <- etaContractBody body
-                 liftM modifyAbstractClause $ inContext [] $ reify $ AN.QNamed def $ I.Clause noRange tel perm ps body t
+                 liftM modifyAbstractClause $ inContext [] $ reify $ AN.QNamed def $ I.Clause noRange tel ps body t catchall
               pcs <- withInteractionId ii $ mapM prettyA cls''
               ticks <- liftIO $ readIORef ticks
 
@@ -393,7 +395,7 @@ auto ii rng argstr = do
           minfo = getMetaInfo mv
       targettyp <- withMetaInfo minfo $ do
        vs <- getContextArgs
-       let targettype = tt `piApply` permute (takeP (length vs) $ mvPermutation mv) vs
+       targettype <- tt `piApplyM` permute (takeP (length vs) $ mvPermutation mv) vs
        normalise targettype
       let tctx = length $ envContext $ clEnv minfo
 
