@@ -12,7 +12,9 @@ import Data.Map (Map)
 import qualified Data.Map as Map
 import Control.Monad.State
 
+import Agda.Syntax.Concrete (exprFieldA)
 import qualified Agda.Syntax.Internal as I
+import qualified Agda.Syntax.Internal.Pattern as IP
 import qualified Agda.Syntax.Common as Common
 import qualified Agda.Syntax.Abstract.Name as AN
 import qualified Agda.Syntax.Abstract as A
@@ -27,7 +29,7 @@ import Agda.TypeChecking.Monad.MetaVars (lookupMeta, withMetaInfo)
 import Agda.TypeChecking.Monad.Context (getContextArgs)
 import Agda.TypeChecking.Monad.Constraints (getAllConstraints)
 import Agda.TypeChecking.Substitute (piApply, applySubst)
-import Agda.TypeChecking.Telescope (renamingR)
+import Agda.TypeChecking.Telescope (piApplyM, renamingR)
 import qualified Agda.TypeChecking.Substitute as I (absBody)
 import Agda.TypeChecking.Reduce (Normalise, normalise, instantiate)
 import Agda.TypeChecking.EtaContract (etaContract)
@@ -40,10 +42,11 @@ import Agda.Auto.Syntax
 
 import Agda.Auto.CaseSplit hiding (lift)
 
+import Agda.Utils.Either
 import Agda.Utils.Except
   ( Error(strMsg)
   , ExceptT
-  , MonadError(catchError, throwError)
+  , MonadError(throwError)
   )
 import Agda.Utils.Lens
 
@@ -166,7 +169,7 @@ tomy imi icns typs = do
            localVars = map (snd . Common.unDom . ctxEntry) . envContext . clEnv $ minfo
        (targettype, localVars) <- lift $ withMetaInfo minfo $ do
         vs <- getContextArgs
-        let targettype = tt `piApply` permute (takeP (length vs) $ mvPermutation mv) vs
+        targettype <- tt `piApplyM` permute (takeP (length vs) $ mvPermutation mv) vs
         targettype <- norm targettype
         localVars <- mapM norm localVars
         return (targettype, localVars)
@@ -295,16 +298,16 @@ tomyClauses (cl:cls) = do
   Nothing -> cls'
 
 tomyClause :: I.Clause -> TOM (Maybe ([Pat O], MExp O))
-tomyClause cl@(I.Clause {I.clausePerm = Perm n ps, I.clauseBody = body}) = do
+tomyClause cl@(I.Clause {I.clauseBody = body}) = do
  let pats = I.clausePats cl
- pats' <- mapM tomyPat pats
+ pats' <- mapM tomyPat $ IP.unnumberPatVars pats
  body' <- tomyBody body
  return $ case body' of
            Just (body', _) -> Just (pats', body')
            Nothing -> Nothing
 
 
-tomyPat :: I.Arg I.Pattern -> TOM (Pat O)
+tomyPat :: Common.Arg I.Pattern -> TOM (Pat O)
 tomyPat p = case Common.unArg p of
  I.ProjP _ -> lift $ copatternsNotImplemented
  I.VarP n -> return $ PatVar (show n)
@@ -427,7 +430,6 @@ tomyExp v0 =
         _ -> tomyExp t
     I.DontCare _ -> return $ NotM $ dontCare
     I.Shared p -> tomyExp $ I.derefPtr p
-    I.ExtLam{} -> __IMPOSSIBLE__
 
 tomyExps :: I.Args -> TOM (MM (ArgList O) (RefInfo O))
 tomyExps [] = return $ NotM ALNil
@@ -457,7 +459,6 @@ fmExp m (I.Sort _) = False
 fmExp m (I.MetaV mid _) = mid == m
 fmExp m (I.DontCare _) = False
 fmExp m (I.Shared p) = fmExp m $ I.derefPtr p
-fmExp m I.ExtLam{} = __IMPOSSIBLE__
 
 fmExps :: I.MetaId -> I.Args -> Bool
 fmExps m [] = False
@@ -479,7 +480,7 @@ cnvh info = case Common.getHiding info of
     Common.Instance  -> Instance
     Common.Hidden    -> Hidden
 
-icnvh :: FMode -> I.ArgInfo
+icnvh :: FMode -> Common.ArgInfo
 icnvh h = (Common.setHiding h' Common.defaultArgInfo)
     where
     h' = case h of
@@ -584,13 +585,13 @@ modifyAbstractExpr = f
   f (A.Lam i (A.DomainFree info n) _) | show (A.nameConcrete n) == abslamvarname =
         A.AbsurdLam i $ Common.argInfoHiding info
   f (A.Lam i b e) = A.Lam i b (f e)
-  f (A.Rec i xs) = A.Rec i (map (\(n, e) -> (n, f e)) xs)
-  f (A.RecUpdate i e xs) = A.RecUpdate i (f e) (map (\(n, e) -> (n, f e)) xs)
+  f (A.Rec i xs) = A.Rec i (map (mapLeft (over exprFieldA f)) xs)
+  f (A.RecUpdate i e xs) = A.RecUpdate i (f e) (map (over exprFieldA f) xs)
   f (A.ScopedExpr i e) = A.ScopedExpr i (f e)
   f e = e
 
 modifyAbstractClause :: A.Clause -> A.Clause
-modifyAbstractClause (A.Clause lhs (A.RHS e) decls) = A.Clause lhs (A.RHS (modifyAbstractExpr e)) decls
+modifyAbstractClause (A.Clause lhs (A.RHS e) decls catchall) = A.Clause lhs (A.RHS (modifyAbstractExpr e)) decls catchall
 modifyAbstractClause cl = cl
 
 -- ---------------------------------
@@ -619,7 +620,7 @@ constructPats cmap mainm clause = do
         return (ns, HI hid (CSPatExp t2))
        I.ProjP{} -> copatternsNotImplemented
        _ -> __IMPOSSIBLE__
- (names, pats) <- cnvps [] (I.namedClausePats clause)
+ (names, pats) <- cnvps [] (IP.unnumberPatVars $ I.namedClausePats clause)
  return (reverse names, pats)
 
 
@@ -692,10 +693,10 @@ frommyClause (ids, pats, mrhs) = do
  return $ I.Clause
    { I.clauseRange = SP.noRange
    , I.clauseTel   = tel
-   , I.clausePerm  = cperm
-   , I.namedClausePats = applySubst (renamingR $ compactP cperm) ps
+   , I.namedClausePats = IP.numberPatVars cperm $ applySubst (renamingR $ compactP cperm) ps
    , I.clauseBody  = applySubst (renamingR cperm) <$> body
    , I.clauseType  = Nothing -- TODO: compute clause type
+   , I.clauseCatchall = False
    }
 
 contains_constructor :: [CSPat O] -> Bool
@@ -781,7 +782,6 @@ findClauseDeep m = do
       I.MetaV m' _  -> m == m'
       I.DontCare _ -> False
       I.Shared{} -> __IMPOSSIBLE__
-      I.ExtLam{} -> __IMPOSSIBLE__
     findMetas = any (findMeta . Common.unArg)
     findMetat (I.El _ e) = findMeta e
     toplevel e =
