@@ -14,7 +14,6 @@
 
 module Agda.Benchmarking where
 
-import Control.Monad.IO.Class
 import qualified Control.Exception as E
 
 import Data.IORef
@@ -52,6 +51,8 @@ data Phase
     -- ^ Generating highlighting info.
   | Serialization
     -- ^ Writing interface files.
+  | DeadCode
+    -- ^ Deac code elimination.
   | Graph
     -- ^ Subphase for 'Termination'.
   | RecCheck
@@ -78,6 +79,10 @@ data Phase
     -- ^ Subphase for 'Typing': free variable computation.
   | OccursCheck
     -- ^ Subphase for 'Typing': occurs check for solving metas.
+  | CheckLHS
+    -- ^ Subphase for 'Typing': checking the LHS
+  | UnifyIndices
+    -- ^ Subphase for 'CheckLHS': unification of the indices
   | InverseScopeLookup
     -- ^ Pretty printing names.
   deriving (Eq, Ord, Show, Enum, Bounded)
