@@ -1,5 +1,6 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE FlexibleContexts #-}
 
 module Agda.Compiler.Epic.Forcing where
 
@@ -20,7 +21,7 @@ import Agda.TypeChecking.Monad
 import Agda.TypeChecking.Rules.LHS.Problem (FlexibleVars, defaultFlexibleVar)
 import Agda.TypeChecking.Rules.LHS.Unify
 import Agda.TypeChecking.Substitute
-  (applySubst, apply, wkS, raiseS, dropS, (++#), TelV(..))
+  (applySubst, apply, piApply, wkS, raiseS, dropS, (++#), TelV(..))
 import qualified Agda.TypeChecking.Substitute as S
 import Agda.TypeChecking.Pretty as P
 import Agda.TypeChecking.Reduce
@@ -100,7 +101,7 @@ insertTele x 0 ins term (ExtendTel t to) = do
       , text "term:" <+> prettyTCM term
       , text "to:"   <+> prettyTCM (unAbs to)
       ]
-    (st, arg) <- case I.unEl . unDom $ t' of
+    (st, arg) <- case I.ignoreSharing . I.unEl . unDom $ t' of
             I.Def st es -> return (st, fromMaybe __IMPOSSIBLE__ $ I.allApplyElims es)
             s          -> do
               report 10 $ vcat
@@ -140,11 +141,10 @@ insertTele er n ins term (ExtendTel x xs) = do
 
 -- TODO: restore fields in ConHead
 mkCon :: QName -> Int -> Term
-mkCon c n = I.Con (I.ConHead c Inductive [])
-                  [ defaultArg $ I.Var i [] | i <- [n - 1, n - 2 .. 0] ]
+mkCon c n = I.Con (I.ConHead c Inductive []) $ map (defaultArg . I.var) $ downFrom n
 
 unifyI :: Telescope -> FlexibleVars -> Type -> Args -> Args -> Compile TCM [Maybe Term]
-unifyI tele flex typ a1 a2 = lift $ addCtxTel tele $ unifyIndices_ flex typ a1 a2
+unifyI tele flex typ a1 a2 = lift $ typeError $ NotImplemented "using the new unification algorithm for forcing"
 
 takeTele :: Int -> Telescope -> Telescope
 takeTele 0 _ = EmptyTel
@@ -206,7 +206,7 @@ forcedExpr vars tele expr = case expr of
                   then Branch t constr as <$> forcedExpr (replaceAt n vars as) tele'' e
                   else do
                     -- unify the telescope type with the return type of the constructor
-                    unif <- case (unEl ntyp, unEl ctyp) of
+                    unif <- case (I.ignoreSharing $ unEl ntyp, I.ignoreSharing $ unEl ctyp) of
                         (I.Def st es1, I.Def st' es2) | st == st' -> do
                             let a1 = fromMaybe __IMPOSSIBLE__ $ I.allApplyElims es1
                             let a2 = fromMaybe __IMPOSSIBLE__ $ I.allApplyElims es2
@@ -218,7 +218,7 @@ forcedExpr vars tele expr = case expr of
                               ]
                             unifyI (takeTele (n + length as) tele'')
                                    (map defaultFlexibleVar [0 .. n + length as])
-                                   (setType `apply` take typPars a1)
+                                   (setType `piApply` take typPars a1)
                                    (drop typPars a1)
                                    (drop typPars a2)
                         _ -> __IMPOSSIBLE__
@@ -283,6 +283,7 @@ replaceForced (vars,uvars) tele (fvar : fvars) unif e = do
 -- | Given a term containg the forced var, dig out the variable by inserting
 -- the proper case-expressions.
 buildTerm :: Var -> Nat -> Term -> Compile TCM (Expr -> Expr, Var)
+buildTerm var idx (I.Shared p) = buildTerm var idx $ I.derefPtr p
 buildTerm var idx (I.Var i _) | idx == i = return (id, var)
 buildTerm var idx (I.Con con args) = do
     let c = I.conName con
@@ -305,7 +306,7 @@ findPosition var ts = (listToMaybe . catMaybes <$>) . forM (zip [0..] ts) $ \ (n
         (return Nothing)
   where
     pred :: Term -> Compile TCM Bool
-    pred t = case t of
+    pred t = case I.ignoreSharing t of
       I.Var i _ | var == i -> return True
       I.Con c args         -> do
           forc <- getForcedArgs $ I.conName c
