@@ -128,7 +128,7 @@ translateDefn msharp (n, defini) =
 
 reverseCCBody :: Int -> CC.CompiledClauses -> CC.CompiledClauses
 reverseCCBody c cc = case cc of
-    CC.Case n (CC.Branches cop cbr lbr cabr) -> CC.Case (c+n)
+    CC.Case n (CC.Branches cop cbr lbr cabr) -> CC.Case (fmap (c+) n)
         $ CC.Branches cop (Map.map (fmap $ reverseCCBody c) cbr)
           (Map.map (reverseCCBody c) lbr)
           (fmap  (reverseCCBody c) cabr)
@@ -179,7 +179,7 @@ compileClauses name nargs c = do
   where
     compileClauses' :: [Var] -> Maybe Var -> CC.CompiledClauses -> Compile TCM Expr
     compileClauses' env omniDefault cc = case cc of
-        CC.Case n nc -> case length env <= n of
+        CC.Case (Arg _ n) nc -> case length env <= n of
            True -> __IMPOSSIBLE__
            False -> case CC.catchAllBranch nc of
             Nothing -> Case (Var (fromMaybe __IMPOSSIBLE__ $ env !!! n)) <$>
@@ -220,7 +220,7 @@ compileClauses name nargs c = do
 --   from patternmatching. Agda terms are in de Bruijn so we just check the new
 --   names in the position.
 substTerm :: [Var] -> T.Term -> Compile TCM Expr
-substTerm env term = case T.unSpine term of
+substTerm env term = case T.ignoreSharing $ T.unSpine term of
     T.Var ind es -> do
       let args = fromMaybe __IMPOSSIBLE__ $ allApplyElims es
       case length env <= ind of
@@ -253,12 +253,11 @@ substTerm env term = case T.unSpine term of
     T.Sort _  -> return UNIT
     T.MetaV _ _ -> return UNIT
     T.DontCare _ -> return UNIT
-    T.ExtLam{} -> __IMPOSSIBLE__
 
 -- | Translate Agda literals to our AUX definition
 substLit :: TL.Literal -> Compile TCM Lit
 substLit lit = case lit of
-  TL.LitInt    _ i -> return $ LInt i
+  TL.LitNat    _ i -> return $ LInt i
   TL.LitString _ s -> return $ LString s
   TL.LitChar   _ c -> return $ LChar c
   TL.LitFloat  _ f -> return $ LFloat f
