@@ -15,7 +15,7 @@ import qualified Data.Set as Set
 
 import Agda.Syntax.Common
 import Agda.Syntax.Internal as SI
-import Agda.TypeChecking.Monad
+import Agda.TypeChecking.Monad hiding (funInline)
 import Agda.TypeChecking.Substitute
 import Agda.TypeChecking.Telescope
 import Agda.TypeChecking.Pretty
@@ -71,7 +71,7 @@ xs +++ ys = unflattenTel names $ map (raise (size ys)) (flattenTel xs) ++ flatte
   where names = teleNames xs ++ teleNames ys
 
 -- | Can a datatype be inferred? If so, return the only possible value.
-inferable :: Set QName -> QName -> [SI.Arg Term] ->  Compile TCM (Maybe Expr)
+inferable :: Set QName -> QName -> [Arg Term] ->  Compile TCM (Maybe Expr)
 inferable visited dat args | dat `Set.member` visited = return Nothing
 inferable visited dat args = do
   lift $ reportSLn "epic.smashing" 10 $ "  inferring:" ++ (show dat)
@@ -94,7 +94,7 @@ inferable visited dat args = do
         defs <- lift $ use $ stImports . sigDefinitions
         let def = fromMaybe __IMPOSSIBLE__ $ HM.lookup c defs
         forc <- getForcedArgs c
-        TelV tel _ <- lift $ telView (defType def `apply` genericTake pars args)
+        TelV tel _ <- lift $ telView (defType def `piApply` genericTake pars args)
         tag <- getConstrTag c
         lift $ reportSDoc "epic.smashing" 10 $ nest 2 $ vcat
           [ text "inferableArgs!"
@@ -106,7 +106,7 @@ inferable visited dat args = do
 
 inferableTerm :: Set QName -> Term -> Compile TCM (Maybe Expr)
 inferableTerm visited t = do
-  case t of
+  case ignoreSharing t of
     Def q es    ->
       case allApplyElims es of
         Just vs -> inferable visited q vs
