@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP            #-}
 {-# LANGUAGE NamedFieldPuns #-}
 
 module Agda.Compiler.JS.Compiler where
@@ -27,13 +27,14 @@ import Agda.Syntax.Internal
   ( Name, Args, Type,
     Clause, Pattern, Pattern'(VarP,DotP,LitP,ConP,ProjP),
     ClauseBodyF(Body,NoBody,Bind),ClauseBody,
-    Term(Var,Lam,Lit,Level,Def,Con,Pi,Sort,MetaV,DontCare,Shared,ExtLam),
+    Term(Var,Lam,Lit,Level,Def,Con,Pi,Sort,MetaV,DontCare,Shared),
     unSpine, allApplyElims,
     conName,
     derefPtr,
     toTopLevelModuleName, clausePats, clauseBody, arity, unEl, unAbs )
+import Agda.Syntax.Internal.Pattern ( unnumberPatVars )
 import Agda.TypeChecking.Substitute ( absBody )
-import Agda.Syntax.Literal ( Literal(LitInt,LitFloat,LitString,LitChar,LitQName) )
+import Agda.Syntax.Literal ( Literal(LitNat,LitFloat,LitString,LitChar,LitQName,LitMeta) )
 import Agda.TypeChecking.Level ( reallyUnLevelView )
 import Agda.TypeChecking.Monad
   ( TCM, Definition(Defn), Interface,
@@ -56,8 +57,9 @@ import Agda.Utils.Monad ( (<$>), (<*>), ifM )
 import Agda.Utils.Pretty (prettyShow)
 import Agda.Utils.IO.UTF8 ( writeFile )
 import qualified Agda.Utils.HashMap as HMap
-import Agda.Compiler.MAlonzo.Misc ( curDefs, curIF, curMName, setInterface )
-import Agda.Compiler.MAlonzo.Primitives ( repl )
+import Agda.Compiler.Common
+  ( curDefs, curIF, curMName, setInterface, repl, inCompilerEnv,
+    IsMain (..), doCompile )
 
 import Agda.Compiler.JS.Syntax
   ( Exp(Self,Local,Global,Undefined,String,Char,Integer,Double,Lambda,Object,Apply,Lookup),
@@ -76,28 +78,11 @@ import Agda.Utils.Impossible ( Impossible(Impossible), throwImpossible )
 --------------------------------------------------
 
 compilerMain :: Interface -> TCM ()
-compilerMain mainI =
-  -- Preserve the state (the compiler modifies the state).
-  localTCState $ do
-
-    -- Compute the output directory.
-    opts <- commandLineOptions
-    compileDir <- case optCompileDir opts of
-      Just dir -> return dir
-      Nothing  -> do
-        -- The default output directory is the project root.
-        let tm = toTopLevelModuleName $ iModuleName mainI
-        f <- findFile tm
-        return $ filePath $ projectRoot f tm
-    setCommandLineOptions $
-      opts { optCompileDir = Just compileDir }
-
-    ignoreAbstractMode $ do
-      mapM_ (compile . miInterface) =<< (elems <$> getVisitedModules)
+compilerMain mainI = inCompilerEnv mainI $ do
+  doCompile IsMain mainI $ \_ -> compile
 
 compile :: Interface -> TCM ()
 compile i = do
-  setInterface i
   ifM uptodate noComp $ do
     yesComp
     writeModule =<< curModule
@@ -303,8 +288,9 @@ numPars (c : _) = genericLength (clausePats c)
 
 clause :: Clause -> TCM Case
 clause c = do
-  ps <- mapM (pattern . unArg) (clausePats c)
-  (av,bv,es) <- return (mapping (map unArg (clausePats c)))
+  let pats = unnumberPatVars $ clausePats c
+  ps <- mapM (pattern . unArg) pats
+  (av,bv,es) <- return (mapping (map unArg pats))
   e <- body (clauseBody c)
   return (Case ps (subst av es e))
 
@@ -417,7 +403,6 @@ term v = do
     (Sort  _)            -> return (String "*")
     (MetaV _ _)          -> return (Undefined)
     (DontCare _)         -> return (Undefined)
-    ExtLam{}             -> __IMPOSSIBLE__
 
 -- Check to see if a type is a singleton, and if so, return its only
 -- member.  Singleton types are of the form T1 -> ... -> Tn -> T where
@@ -454,11 +439,12 @@ qname q = do
   return (foldl Lookup e ls)
 
 literal :: Literal -> Exp
-literal (LitInt    _ x) = Integer x
+literal (LitNat    _ x) = Integer x
 literal (LitFloat  _ x) = Double  x
 literal (LitString _ x) = String  x
 literal (LitChar   _ x) = Char    x
 literal (LitQName  _ x) = String  (show x)
+literal LitMeta{}       = __IMPOSSIBLE__
 
 dummyLambda :: Int -> Exp -> Exp
 dummyLambda n = iterate' n (Lambda 0)
