@@ -6,7 +6,7 @@
 module Agda.Compiler.MAlonzo.Compiler where
 
 #if __GLASGOW_HASKELL__ <= 708
-import Prelude hiding (mapM_, mapM, sequence, concat)
+import Prelude hiding (foldl, mapM_, mapM, sequence, concat)
 #endif
 
 import Control.Applicative
@@ -14,7 +14,8 @@ import Control.Monad.Reader hiding (mapM_, forM_, mapM, forM, sequence)
 import Control.Monad.State  hiding (mapM_, forM_, mapM, forM, sequence)
 
 import Data.Generics.Geniplate
-import Data.Foldable
+import Data.Foldable hiding (any, foldr, sequence_)
+import Data.Function
 import qualified Data.List as List
 import Data.Map (Map)
 import qualified Data.Map as Map
@@ -32,9 +33,12 @@ import System.Directory (createDirectoryIfMissing)
 import System.FilePath hiding (normalise)
 
 import Agda.Compiler.CallCompiler
+import Agda.Compiler.Common
 import Agda.Compiler.MAlonzo.Misc
 import Agda.Compiler.MAlonzo.Pretty
 import Agda.Compiler.MAlonzo.Primitives
+import Agda.Compiler.ToTreeless
+import Agda.Compiler.Treeless.Unused
 
 import Agda.Interaction.FindFile
 import Agda.Interaction.Imports
@@ -44,6 +48,8 @@ import Agda.Syntax.Common
 import qualified Agda.Syntax.Abstract.Name as A
 import qualified Agda.Syntax.Concrete.Name as C
 import Agda.Syntax.Internal as I
+import Agda.Syntax.Internal.Names (namesIn)
+import qualified Agda.Syntax.Treeless as T
 import Agda.Syntax.Literal
 
 import Agda.TypeChecking.Monad
@@ -60,6 +66,7 @@ import Agda.TypeChecking.CompiledClause
 
 import Agda.Utils.FileName
 import Agda.Utils.Functor
+import Agda.Utils.IO.Directory
 import Agda.Utils.Lens
 import Agda.Utils.List
 import Agda.Utils.Maybe
@@ -70,41 +77,26 @@ import qualified Agda.Utils.HashMap as HMap
 import Agda.Utils.Singleton
 import Agda.Utils.Tuple
 
+import Paths_Agda
+
 #include "undefined.h"
 import Agda.Utils.Impossible
 
-compilerMain :: Bool -> Interface -> TCM ()
-compilerMain modIsMain mainI =
-  -- Preserve the state (the compiler modifies the state).
-  -- Andreas, 2014-03-23 But we might want to collect Benchmark info,
-  -- so use localTCState.
-  localTCState $ do
-
-    -- Compute the output directory.
-    opts <- commandLineOptions
-    compileDir <- case optCompileDir opts of
-      Just dir -> return dir
-      Nothing  -> do
-        -- The default output directory is the project root.
-        let tm = toTopLevelModuleName $ iModuleName mainI
-        f <- findFile tm
-        return $ filePath $ C.projectRoot f tm
-    setCommandLineOptions $
-      opts { optCompileDir = Just compileDir }
-
-    ignoreAbstractMode $ do
-      mapM_ (compile . miInterface) =<< (Map.elems <$> getVisitedModules)
-      writeModule rteModule
-      callGHC modIsMain mainI
+compilerMain :: IsMain -> Interface -> TCM ()
+compilerMain isMain i =
+  inCompilerEnv i $ do
+    doCompile isMain i $
+      \_ -> compile
+    copyRTEModules
+    callGHC isMain i
 
 compile :: Interface -> TCM ()
 compile i = do
-  setInterface i
   ifM uptodate noComp $ {- else -} do
     yesComp
     writeModule =<< decl <$> curHsMod <*> (definitions =<< curDefs) <*> imports
   where
-  decl mn ds imp = HS.Module dummy mn [] Nothing Nothing imp ds
+  decl mn ds imp = HS.Module dummy mn [] Nothing Nothing imp (map fakeDecl (reverse $ iHaskellCode i) ++ ds)
   uptodate = liftIO =<< (isNewerThan <$> outFile_ <*> ifile)
   ifile    = maybe __IMPOSSIBLE__ filePath <$>
                (findInterfaceFile . toTopLevelModuleName =<< curMName)
@@ -121,10 +113,18 @@ compile i = do
 imports :: TCM [HS.ImportDecl]
 imports = (++) <$> hsImps <*> imps where
   hsImps :: TCM [HS.ImportDecl]
-  hsImps = (List.map decl . Set.toList .
+  hsImps = ((unqualRTE :) . List.map decl . Set.toList .
             Set.insert mazRTE . Set.map HS.ModuleName) <$>
              getHaskellImports
 
+  unqualRTE :: HS.ImportDecl
+  unqualRTE = HS.ImportDecl dummy mazRTE False False False Nothing Nothing $ Just $
+#if MIN_VERSION_haskell_src_exts(1,17,0)
+              (False, [HS.IVar $ HS.Ident x | x <- [mazCoerceName, mazErasedName]])
+#else
+              (False, [HS.IVar HS.NoNamespace $ HS.Ident x | x <- [mazCoerceName, mazErasedName]])
+#endif
+
   imps :: TCM [HS.ImportDecl]
   imps = List.map decl . uniq <$>
            ((++) <$> importsForPrim <*> (List.map mazMod <$> mnames))
@@ -146,8 +146,11 @@ imports = (++) <$> hsImps <*> imps where
 definitions :: Definitions -> TCM [HS.Decl]
 definitions defs = do
   kit <- coinductionKit
-  HMap.foldr (liftM2 (++) . (definition kit <=< instantiateFull))
-             declsForPrim defs
+  concat <$>
+    (mapM (\(_, d) -> definition kit =<< instantiateFull d) $
+    -- sort defs to ensure consistent order (see Issue 1900)
+     sortDefs defs
+    )
 
 -- | Note that the INFINITY, SHARP and FLAT builtins are translated as
 -- follows (if a 'CoinductionKit' is given):
@@ -179,10 +182,13 @@ definition kit Defn{defName = q, defType = ty, defCompiledRep = compiled, theDef
     , nest 2 $ text (show d)
     ]
   checkTypeOfMain q ty $ do
-    (infodecl q :) <$> case d of
+    infodecl q <$> case d of
 
-      _ | Just (HsDefn ty hs) <- compiledHaskell compiled ->
-        return $ fbWithType ty (fakeExp hs)
+      _ | Just (HsDefn hsty hs) <- compiledHaskell compiled -> do
+        -- Make sure we have imports for all names mentioned in the type.
+        ty <- normalise ty
+        sequence_ [ xqual x (HS.Ident "_") | x <- Set.toList (namesIn ty) ]
+        return $ fbWithType hsty (fakeExp hs)
 
       -- Special treatment of coinductive builtins.
       Datatype{} | Just q == (nameOfInf <$> kit) -> do
@@ -225,21 +231,17 @@ definition kit Defn{defName = q, defType = ty, defCompiledRep = compiled, theDef
           ]
 
       Axiom{} -> return $ fb axiomErr
-      Primitive{ primClauses = [], primName = s } -> fb <$> primBody s
-      Primitive{ primClauses = cls } -> function Nothing $
-        functionFromClauses cls
+      Primitive{ primName = s } -> fb <$> primBody s
 
-      -- Currently, catchAllBranches cannot be handled properly by casetree
-      Function{ funCompiled = Just cc } | not (hasCatchAll cc) ->
-        function (exportHaskell compiled) $ functionFromCaseTree q cc
-      Function{ funClauses = cls } ->
-        function (exportHaskell compiled) $ functionFromClauses cls
+      Function{} -> function (exportHaskell compiled) $ functionViaTreeless q
 
       Datatype{ dataPars = np, dataIxs = ni, dataClause = cl, dataCons = cs }
         | Just (HsType ty) <- compiledHaskell compiled -> do
-        ccs <- List.concat <$> mapM checkConstructorType cs
-        cov <- checkCover q ty np cs
-        return $ tvaldecl q (dataInduction d) 0 (np + ni) [] (Just __IMPOSSIBLE__) ++ ccs ++ cov
+        ccscov <- ifM (noCheckCover q) (return []) $ do
+            ccs <- List.concat <$> mapM checkConstructorType cs
+            cov <- checkCover q ty np cs
+            return $ ccs ++ cov
+        return $ tvaldecl q (dataInduction d) 0 (np + ni) [] (Just __IMPOSSIBLE__) ++ ccscov
       Datatype{ dataPars = np, dataIxs = ni, dataClause = cl, dataCons = cs } -> do
         (ars, cds) <- unzip <$> mapM condecl cs
         return $ tvaldecl q (dataInduction d) (List.maximum (np:ars) - np) (np + ni) cds cl
@@ -264,21 +266,38 @@ definition kit Defn{defName = q, defType = ty, defCompiledRep = compiled, theDef
             tsig = HS.TypeSig dummy [HS.Ident name] (fakeType t)
 
             def :: HS.Decl
-            def = HS.FunBind [HS.Match dummy (HS.Ident name) [] Nothing (HS.UnGuardedRhs (hsVarUQ $ dsubname q 0)) emptyBinds]
+            def = HS.FunBind [HS.Match dummy (HS.Ident name) [] Nothing (HS.UnGuardedRhs (hsVarUQ $ dname q)) emptyBinds]
         return ([tsig,def] ++ ccls)
 
-  functionFromClauses :: [Clause] -> TCM [HS.Decl]
-  functionFromClauses cls = mapM (clause q Nothing) (tag 0 cls)
+  functionViaTreeless :: QName -> TCM [HS.Decl]
+  functionViaTreeless q = caseMaybeM (toTreeless q) (pure []) $ \ treeless -> do
 
-  functionFromCaseTree :: QName -> CompiledClauses -> TCM [HS.Decl]
-  functionFromCaseTree q cc = do
-    e <- casetree cc `runReaderT` initCCEnv (Just q)
-    return $ [HS.FunBind [HS.Match dummy (dsubname q 0) [] Nothing (HS.UnGuardedRhs e) emptyBinds]]
+    used <- getCompiledArgUse q
+    let dostrip = any not used
 
-  tag :: Nat -> [Clause] -> [(Nat, Bool, Clause)]
-  tag _ []       = []
-  tag i [cl]     = (i, True , cl) : []
-  tag i (cl:cls) = (i, False, cl) : tag (i + 1) cls
+    e <- if dostrip then closedTerm (stripUnusedArguments used treeless)
+                    else closedTerm treeless
+    let (ps, b) = lamView e
+        lamView e =
+          case stripTopCoerce e of
+            HS.Lambda _ ps b -> (ps, b)
+            b                -> ([], b)
+        stripTopCoerce (HS.Lambda i ps b) = HS.Lambda i ps $ stripTopCoerce b
+        stripTopCoerce e =
+          case hsAppView e of
+            [c,  e] | c == mazCoerce -> e
+            _                        -> e
+
+        funbind f ps b = HS.FunBind [HS.Match dummy f ps Nothing (HS.UnGuardedRhs b) emptyBinds]
+
+    -- The definition of the non-stripped function
+    (ps0, _) <- lamView <$> closedTerm (foldr ($) T.TErased $ replicate (length used) T.TLam)
+    let b0 = foldl HS.App (hsVarUQ $ duname q) [ hsVarUQ x | (~(HS.PVar x), True) <- zip ps0 used ]
+
+    return $ if dostrip
+      then [ funbind (dname q) ps0 b0
+           , funbind (duname q) ps b ]
+      else [ funbind (dname q) ps b ]
 
   mkwhere :: [HS.Decl] -> [HS.Decl]
   mkwhere (HS.FunBind [m0, HS.Match _     dn ps mt rhs emptyBinds] :
@@ -308,11 +327,8 @@ definition kit Defn{defName = q, defType = ty, defCompiledRep = compiled, theDef
 -- | Environment for naming of local variables.
 --   Invariant: @reverse ccCxt ++ ccNameSupply@
 data CCEnv = CCEnv
-  { ccFunName    :: Maybe QName -- ^ Agda function we are currently compiling.
-  , ccNameSupply :: NameSupply  -- ^ Supply of fresh names
+  { ccNameSupply :: NameSupply  -- ^ Supply of fresh names
   , ccCxt        :: CCContext   -- ^ Names currently in scope
-  , ccCatchAll   :: Maybe CompiledClauses  -- ^ Naive catch-all implementation.
-      -- If an inner case has no catch-all clause, we use the one from its parent.
   }
 
 type NameSupply = [HS.Name]
@@ -325,149 +341,28 @@ mapContext :: (CCContext -> CCContext) -> CCEnv -> CCEnv
 mapContext f e = e { ccCxt = f (ccCxt e) }
 
 -- | Initial environment for expression generation.
-initCCEnv :: Maybe QName -> CCEnv
-initCCEnv q = CCEnv
-  { ccFunName    = q
-  , ccNameSupply = map (ihname "v") [0..]  -- DON'T CHANGE THESE NAMES!
+initCCEnv :: CCEnv
+initCCEnv = CCEnv
+  { ccNameSupply = map (ihname "v") [0..]  -- DON'T CHANGE THESE NAMES!
   , ccCxt        = []
-  , ccCatchAll   = Nothing
   }
 
 -- | Term variables are de Bruijn indices.
 lookupIndex :: Int -> CCContext -> HS.Name
 lookupIndex i xs = fromMaybe __IMPOSSIBLE__ $ xs !!! i
 
--- | Case variables are de Bruijn levels.
-lookupLevel :: Int -> CCContext -> HS.Name
-lookupLevel l xs = fromMaybe __IMPOSSIBLE__ $ xs !!! (length xs - 1 - l)
-
 type CC = ReaderT CCEnv TCM
 
--- | Compile a case tree into nested case and record expressions.
-casetree :: CompiledClauses -> CC HS.Exp
-casetree cc = do
-  case cc of
-    Fail -> return $ rtmError $ "Impossible Clause Body"
-    Done xs v -> lambdasUpTo (length xs) $ hsCast <$> term v
-    Case n (Branches True conBrs _ _) -> lambdasUpTo n $ do
-      mkRecord =<< mapM casetree (content <$> conBrs)
-    Case n (Branches False conBrs litBrs catchAll) -> lambdasUpTo (n + 1) $ do
-      x <- lookupLevel n <$> asks ccCxt
-      HS.Case (hsCast $ hsVarUQ x) <$> do
-        updateCatchAll catchAll $ do
-          br1 <- conAlts n conBrs
-          br2 <- litAlts litBrs
-          br3 <- catchAllAlts =<< asks ccCatchAll
-          return (br1 ++ br2 ++ br3)
-
--- | Replace the current catch-all clause with a new one, if given.
-updateCatchAll :: Maybe CompiledClauses -> CC a -> CC a
-updateCatchAll Nothing   = id
-updateCatchAll (Just cc) = local $ \ e -> e { ccCatchAll = Just cc }
-
--- -- | Replace de Bruijn index by term in catch-all tree.
--- substCatchAll :: Int -> I.Term -> CC a -> CC a
--- substCatchAll i t = local $ \ e -> e { ccCatchAll = subst i t $ ccCatchAll e }
-
-conAlts :: Int -> Map QName (WithArity CompiledClauses) -> CC [HS.Alt]
-conAlts x br = forM (Map.toList br) $ \ (c, WithArity n cc) -> do
-  c <- lift $ conhqn c
-  -- TODO: substitute in ccCatchAll somehow
-  replaceVar x n $ \ xs -> do
-    branch (HS.PApp c $ map HS.PVar xs) cc
-
-litAlts :: Map Literal CompiledClauses -> CC [HS.Alt]
-litAlts br = forM (Map.toList br) $ \ (l, cc) ->
-  -- TODO: substitute in ccCatchAll somehow
-  branch (HS.PLit HS.Signless $ hslit l) cc
-
-catchAllAlts :: Maybe CompiledClauses -> CC [HS.Alt]
-catchAllAlts (Just cc) = mapM (branch HS.PWildCard) [cc]
-catchAllAlts Nothing   = do
-  q <- fromMaybe __IMPOSSIBLE__ <$> asks ccFunName
-  return [ HS.Alt dummy HS.PWildCard (HS.UnGuardedRhs $ rtmIncompleteMatch q) emptyBinds ]
-
-branch :: HS.Pat -> CompiledClauses -> CC HS.Alt
-branch pat cc = do
-  e <- casetree cc
-  return $ HS.Alt dummy pat (HS.UnGuardedRhs e) emptyBinds
-
--- | Replace de Bruijn Level @x@ by @n@ new variables.
-replaceVar :: Int -> Int -> ([HS.Name] -> CC a) -> CC a
-replaceVar x n cont = do
+freshNames :: Int -> ([HS.Name] -> CC a) -> CC a
+freshNames n _ | n < 0 = __IMPOSSIBLE__
+freshNames n cont = do
   (xs, rest) <- splitAt n <$> asks ccNameSupply
-  let upd cxt = ys ++ reverse xs ++ zs -- We reverse xs to get nicer names.
-       where
-         -- compute the de Bruijn index
-         i = length cxt - 1 - x
-         -- discard index i
-         (ys, _:zs) = splitAt i cxt
-  local (mapNameSupply (const rest) . mapContext upd) $
-    cont xs
-
--- | Precondition: Map not empty.
-mkRecord :: Map QName HS.Exp -> CC HS.Exp
-mkRecord fs = lift $ do
-  -- Get the name of the first field
-  let p1 = fst $ fromMaybe __IMPOSSIBLE__ $ headMaybe $ Map.toList fs
-  -- Use the field name to get the record constructor and the field names.
-  ConHead c _ind xs <- recConFromProj p1
-  -- Convert the constructor to a Haskell name
-  c <- conhqn c
-  let (args :: [HS.Exp]) = for xs $ \ x -> fromMaybe __IMPOSSIBLE__ $ Map.lookup x fs
-  return $ hsCast $ List.foldl HS.App (HS.Con c) args
-
--- -- | Precondition: Map not empty.
--- MAlonzo does not translate field names in data decls.
--- mkRecord :: Map QName HS.Exp -> CC HS.Exp
--- mkRecord fs = lift $ do
---   -- Get the list of (field name, expression pairs)
---   let l = Map.toList fs
---   -- Get the name of the first field
---   let p1 = fst $ fromMaybe __IMPOSSIBLE__ $ headMaybe l
---   -- Use the field name to get the record constructor
---   c <- I.conName <$> recConFromProj p1
---   -- Convert it to a Haskell name
---   c <- conhqn c
---   -- Convert the field names into Haskell names
---   fs <- mapM (mapFstM (xhqn "d")) l
---   return $ hsCast $ HS.RecConstr c $ map (uncurry HS.FieldUpdate) fs
-
-recConFromProj :: QName -> TCM I.ConHead
-recConFromProj q = do
-  caseMaybeM (isProjection q) __IMPOSSIBLE__ $ \ proj -> do
-    let d = projFromType proj
-    getRecordConstructor d
-
--- | Introduce lambdas such that @n@ variables are in scope.
-lambdasUpTo :: Int -> CC HS.Exp -> CC HS.Exp
-lambdasUpTo n cont = do
-  diff <- (n -) . length <$> asks ccCxt
-  lambdas diff cont
-
--- | Introduce n lambdas.
-lambdas :: Int -> CC HS.Exp -> CC HS.Exp
-lambdas diff cont = intros diff $ \ xs -> mkLams xs <$> cont
-
--- -- | Introduce variables such that @n@ are in scope.
--- introsUpTo :: Int -> ([HS.Name] -> CC HS.Exp) -> CC HS.Exp
--- introsUpTo n cont = do
---   diff <- (n -) . length <$> asks ccCxt
---   intros diff cont
+  local (mapNameSupply (const rest)) $ cont xs
 
 -- | Introduce n variables into the context.
-intros :: Int -> ([HS.Name] -> CC HS.Exp) -> CC HS.Exp
-intros diff cont = do
-  if diff <= 0 then cont [] else do
-    -- Need diff lambdas to continue.
-    (xs, rest) <- splitAt diff <$> asks ccNameSupply
-    local (mapNameSupply (const rest) . mapContext (reverse xs ++)) $
-      cont xs
-
--- | Prefix a Haskell expression with lambda abstractions.
-mkLams :: [HS.Name] -> HS.Exp -> HS.Exp
-mkLams [] e = e
-mkLams xs e = HS.Lambda dummy (map HS.PVar xs) e
+intros :: Int -> ([HS.Name] -> CC a) -> CC a
+intros n cont = freshNames n $ \xs ->
+  local (mapContext (reverse xs ++)) $ cont xs
 
 checkConstructorType :: QName -> TCM [HS.Decl]
 checkConstructorType q = do
@@ -496,164 +391,137 @@ checkCover q ty n cs = do
                                 Nothing (HS.UnGuardedRhs rhs) emptyBinds]
          ]
 
--- | Move somewhere else!
-conArityAndPars :: QName -> TCM (Nat, Nat)
-conArityAndPars q = do
-  def <- getConstInfo q
-  TelV tel _ <- telView $ defType def
-  let Constructor{ conPars = np } = theDef def
-      n = length (telToList tel)
-  return (n - np, np)
-
-clause :: QName -> Maybe String -> (Nat, Bool, Clause) -> TCM HS.Decl
-clause q maybeName (i, isLast, Clause{ namedClausePats = ps, clauseBody = b }) =
-  HS.FunBind . (: cont) <$> main where
-  main = match <$> argpatts ps (bvars b (0::Nat)) <*> clausebody b
-  cont | isLast && List.any isCon ps = [match (List.map HS.PVar cvs) failrhs]
-       | isLast                      = []
-       | otherwise                   = [match (List.map HS.PVar cvs) crhs]
-  cvs  = List.map (ihname "v") [0 .. length ps - 1]
-  crhs = hsCast$ List.foldl HS.App (hsVarUQ $ dsubname q (i + 1)) (List.map hsVarUQ cvs)
-  failrhs = rtmIncompleteMatch q  -- Andreas, 2011-11-16 call to RTE instead of inlined error
---  failrhs = rtmError $ "incomplete pattern matching: " ++ show q
-  match hps rhs = HS.Match dummy (maybe (dsubname q i) HS.Ident maybeName) hps Nothing
-                           (HS.UnGuardedRhs rhs) emptyBinds
-  bvars (Body _)           _ = []
-  bvars (Bind (Abs _ b'))  n = HS.PVar (ihname "v" n) : bvars b' (n + 1)
-  bvars (Bind (NoAbs _ b)) n = HS.PWildCard : bvars b n
-  bvars NoBody             _ = repeat HS.PWildCard -- ?
-
-  isCon (Arg _ (Named _ ConP{})) = True
-  isCon _                        = False
-
--- argpatts aps xs = hps
--- xs is alist of haskell *variables* in form of patterns (because of wildcard)
-argpatts :: [I.NamedArg Pattern] -> [HS.Pat] -> TCM [HS.Pat]
-argpatts ps0 bvs = evalStateT (List.concat <$> mapM pat' ps0) bvs
-  where
-  pat :: Pattern -> StateT [HS.Pat] TCM [HS.Pat]
-  pat   (ProjP p  ) = do
-    kit <- lift coinductionKit
-    -- Sharps and flats are erased from compiled code
-    if Just p == fmap nameOfFlat kit then return [] else
-      lift $ typeError $ NotImplemented $ "Compilation of copatterns"
-  pat   (VarP _   ) = do v <- gets head; modify tail; return [v]
-  pat   (DotP _   ) = pat (VarP dummy) -- WHY NOT: return HS.PWildCard -- SEE ABOVE
-  pat   (LitP (LitQName _ x)) = return [ litqnamepat x ]
-  pat   (LitP l   ) = return [ HS.PLit HS.Signless $ hslit l ]
-
-  pat p@(ConP c _ ps) = do
-    -- Note that irr is applied once for every subpattern, so in the
-    -- worst case it is quadratic in the size of the pattern. I
-    -- suspect that this will not be a problem in practice, though.
-    irrefutable <- lift $ irr p
-    let tilde :: HS.Pat -> HS.Pat
-        tilde = if tildesEnabled && irrefutable
-                then HS.PParen . HS.PIrrPat
-                else id
-    ((:[]) . tilde . HS.PParen) <$>
-      (HS.PApp <$> lift (conhqn $ conName c) <*> (List.concat <$> mapM pat' ps))
-
-  {- Andreas, 2013-02-15 this triggers Issue 794,
-     because it fails to count the variables bound in p,
-     thus, the following variables bound by patterns do
-     not correspond to the according rhs-variables.
-
-  -- Andreas, 2010-09-29
-  -- do not match against irrelevant stuff
-  pat' a | isIrrelevant a = return $ HS.PWildCard
--}
-  pat' :: I.NamedArg Pattern -> StateT [HS.Pat] TCM [HS.Pat]
-  pat' a = pat $ namedArg a
-
-  tildesEnabled = False
-
-  -- | Is the pattern irrefutable?
-  irr :: Pattern -> TCM Bool
-  irr (ProjP {})  = __IMPOSSIBLE__
-  irr (VarP {})   = return True
-  irr (DotP {})   = return True
-  irr (LitP {})   = return False
-  irr (ConP c _ ps) =
-    (&&) <$> singleConstructorType (conName c)
-         <*> (andM $ List.map irr' ps)
-
-  -- | Irrelevant patterns are naturally irrefutable.
-  irr' :: I.NamedArg Pattern -> TCM Bool
-  irr' a | isIrrelevant a = return $ True
-  irr' a = irr $ namedArg a
-
-clausebody :: ClauseBody -> TCM HS.Exp
-clausebody b0 = go 0 b0 where
-  go n (Body tm       )   = hsCast <$> do
-    runReaderT (intros n $ const $ term tm) (initCCEnv Nothing)
-  go n (Bind (Abs _ b))   = go (n+1) b
-  go n (Bind (NoAbs _ b)) = go n b
-  go n NoBody             = return $ rtmError $ "Impossible Clause Body"
-
-
-closedTerm :: Term -> TCM HS.Exp
-closedTerm v = term v `runReaderT` initCCEnv Nothing
+closedTerm :: T.TTerm -> TCM HS.Exp
+closedTerm v = hsCast <$> term v `runReaderT` initCCEnv
 
 -- | Extract Agda term to Haskell expression.
---   Irrelevant arguments are extracted as @()@.
+--   Erased arguments are extracted as @()@.
 --   Types are extracted as @()@.
---   @DontCare@ outside of irrelevant arguments is extracted as @error@.
-term :: Term -> CC HS.Exp
-term tm0 = case unSpine $ ignoreSharing tm0 of
-  Var   i es -> do
+term :: T.TTerm -> CC HS.Exp
+term tm0 = case tm0 of
+  T.TVar i -> do
     x <- lookupIndex i <$> asks ccCxt
-    apps (hsVarUQ x) $ fromMaybe __IMPOSSIBLE__ $ allApplyElims es
-  Lam   _ at -> lambdas 1 $ term $ absBody at
-  Lit   l    -> lift $ literal l
-  Def   q es -> do
-    let Just as = allApplyElims es
-    q <- lift $ xhqn "d" q
-    HS.Var q `apps` as
-  Con   c as -> do
-    let q = conName c
+    return $ hsVarUQ x
+  T.TApp (T.TDef f) ts -> do
+    used <- lift $ getCompiledArgUse f
+    if any not used && length ts >= length used
+      then do
+        f <- lift $ HS.Var <$> xhqn "du" f  -- used stripped function
+        f `apps` [ t | (t, True) <- zip ts $ used ++ repeat True ]
+      else do
+        t' <- term (T.TDef f)
+        t' `apps` ts
+  T.TApp t ts -> do
+    t' <- term t
+    t' `apps` ts
+  T.TLam at -> do
+    (nm:_) <- asks ccNameSupply
+    intros 1 $ \ [x] ->
+      hsLambda [HS.PVar x] <$> term at
+  T.TLet t1 t2 -> do
+    t1' <- term t1
+    intros 1 $ \[x] -> do
+      t2' <- term t2
+      return $ hsLet x (hsCast t1') t2'
+
+  T.TCase sc ct def alts -> do
+    sc' <- term (T.TVar sc)
+    alts' <- traverse (alt sc) alts
+    def' <- term def
+    let defAlt = HS.Alt dummy HS.PWildCard (HS.UnGuardedRhs def') emptyBinds
+
+    return $ HS.Case (hsCast sc') (alts' ++ [defAlt])
+
+  T.TLit l -> lift $ literal l
+  T.TDef q -> do
+    HS.Var <$> (lift $ xhqn "d" q)
+  T.TCon q -> do
     kit <- lift coinductionKit
     if Just q == (nameOfSharp <$> kit)
-      then (`apps` as) . HS.Var =<< lift (xhqn "d" q)
-      else (`apps` as) . HS.Con =<< lift (conhqn q)
-  Level l    -> term =<< lift (reallyUnLevelView l)
-  Pi    _ _  -> return HS.unit_con
-  Sort  _    -> return HS.unit_con
-  MetaV _ _  -> mazerror "hit MetaV"
-  DontCare _ -> return $ rtmError $ "hit DontCare"
-  Shared{}   -> __IMPOSSIBLE__
-  ExtLam{}   -> __IMPOSSIBLE__
-  where apps =  foldM (\ h a -> HS.App h <$> term' a)
+      then HS.Var <$> lift (xhqn "d" q)
+      else hsCast' . HS.Con <$> lift (conhqn q)
+  T.TPrim p  -> return $ compilePrim p
+  T.TUnit    -> return HS.unit_con
+  T.TSort    -> return HS.unit_con
+  T.TErased  -> return $ hsVarUQ $ HS.Ident mazErasedName
+  T.TError e -> return $ case e of
+    T.TUnreachable ->  rtmUnreachableError
+  where apps =  foldM (\ h a -> HS.App h <$> term a)
 
--- | Irrelevant arguments are replaced by Haskells' ().
-term' :: I.Arg Term -> CC HS.Exp
-term' a | isIrrelevant a = return HS.unit_con
-term' a = term $ unArg a
+compilePrim :: T.TPrim -> HS.Exp
+compilePrim s =
+  case s of
+    T.PQuot -> fakeExp "(Prelude.quot :: Integer -> Integer -> Integer)"
+    T.PRem -> fakeExp "(Prelude.rem :: Integer -> Integer -> Integer)"
+    T.PSub -> fakeExp "((Prelude.-) :: Integer -> Integer -> Integer)"
+    T.PAdd -> fakeExp "((Prelude.+) :: Integer -> Integer -> Integer)"
+    T.PMul -> fakeExp "((Prelude.*) :: Integer -> Integer -> Integer)"
+    T.PGeq -> fakeExp "((Prelude.>=) :: Integer -> Integer -> Bool)"
+    T.PLt  -> fakeExp "((Prelude.<) :: Integer -> Integer -> Bool)"
+    T.PEq  -> fakeExp "((Prelude.==) :: Integer -> Integer -> Bool)"
+    T.PSeq -> HS.Var (hsName "seq")
+    -- primitives only used by GuardsToPrims transformation, which MAlonzo doesn't use
+    T.PIf  -> __IMPOSSIBLE__
+
+alt :: Int -> T.TAlt -> CC HS.Alt
+alt sc a = do
+  case a of
+    T.TACon {} -> do
+      intros (T.aArity a) $ \xs -> do
+        hConNm <- lift $ conhqn $ T.aCon a
+        mkAlt (HS.PApp hConNm $ map HS.PVar xs)
+    T.TAGuard g b -> do
+      g <- term g
+      b <- term b
+      return $ HS.Alt dummy HS.PWildCard
+                      (HS.GuardedRhss [HS.GuardedRhs dummy [HS.Qualifier g] b])
+                      emptyBinds
+    T.TALit { T.aLit = (LitQName _ q) } -> mkAlt (litqnamepat q)
+    T.TALit { T.aLit = (LitString _ s) , T.aBody = b } -> do
+      b <- term b
+      sc <- term (T.TVar sc)
+      let guard =
+            HS.Var (HS.UnQual (HS.Ident "(==)")) `HS.App`
+              sc`HS.App`
+              litString s
+      return $ HS.Alt dummy HS.PWildCard
+                      (HS.GuardedRhss [HS.GuardedRhs dummy [HS.Qualifier guard] b])
+                      emptyBinds
+    T.TALit {} -> mkAlt (HS.PLit HS.Signless $ hslit $ T.aLit a)
+  where
+    mkAlt :: HS.Pat -> CC HS.Alt
+    mkAlt pat = do
+        body' <- term $ T.aBody a
+        return $ HS.Alt dummy pat (HS.UnGuardedRhs $ hsCast body') emptyBinds
 
 literal :: Literal -> TCM HS.Exp
 literal l = case l of
-  LitInt    _ _   -> do toN <- bltQual "NATURAL" mazIntegerToNat
-                        return $ HS.Var toN `HS.App` typed "Integer"
+  LitNat    _ _   -> return $ typed "Integer"
   LitFloat  _ _   -> return $ typed "Double"
   LitQName  _ x   -> return $ litqname x
+  LitString _ s   -> return $ litString s
   _               -> return $ l'
   where l'    = HS.Lit $ hslit l
         typed = HS.ExpTypeSig dummy l' . HS.TyCon . rtmQual
 
 hslit :: Literal -> HS.Literal
-hslit l = case l of LitInt    _ x -> HS.Int    x
+hslit l = case l of LitNat    _ x -> HS.Int    x
                     LitFloat  _ x -> HS.Frac   (toRational x)
-                    LitString _ x -> HS.String x
                     LitChar   _ x -> HS.Char   x
                     LitQName  _ x -> __IMPOSSIBLE__
+                    LitString _ _ -> __IMPOSSIBLE__
+                    LitMeta{}     -> __IMPOSSIBLE__
+
+litString :: String -> HS.Exp
+litString s =
+  HS.Var (HS.Qual (HS.ModuleName "Data.Text") (HS.Ident "pack")) `HS.App`
+    (HS.Lit $ HS.String s)
 
 litqname :: QName -> HS.Exp
 litqname x =
   HS.Con (HS.Qual mazRTE $ HS.Ident "QName") `HS.App`
-  HS.Lit (HS.Int n) `HS.App`
-  HS.Lit (HS.Int m) `HS.App`
-  (rtmError "primQNameType: not implemented") `HS.App`
-  (rtmError "primQNameDefinition: not implemented") `HS.App`
+  hsTypedInt n `HS.App`
+  hsTypedInt m `HS.App`
   HS.Lit (HS.String $ show x )
   where
     NameId n m = nameId $ qnameName x
@@ -663,8 +531,6 @@ litqnamepat x =
   HS.PApp (HS.Qual mazRTE $ HS.Ident "QName")
           [ HS.PLit HS.Signless (HS.Int n)
           , HS.PLit HS.Signless (HS.Int m)
-          , HS.PWildCard
-          , HS.PWildCard
           , HS.PWildCard]
   where
     NameId n m = nameId $ qnameName x
@@ -700,8 +566,9 @@ tvaldecl q ind ntv npar cds cl =
     (Inductive, [HS.RecDecl _ [_]]) -> HS.NewType
     _                               -> HS.DataType
 
-infodecl :: QName -> HS.Decl
-infodecl q = fakeD (unqhname "name" q) $ show $ prettyShow q
+infodecl :: QName -> [HS.Decl] -> [HS.Decl]
+infodecl _ [] = []
+infodecl q ds = fakeD (unqhname "name" q) (show $ prettyShow q) : ds
 
 --------------------------------------------------
 -- Inserting unsafeCoerce
@@ -718,22 +585,50 @@ hsCast = addcast . go where
   go e = [e]
 -}
 
-hsCast e = mazCoerce `HS.App` hsCast' e
+-- TODO: what's the specification for hsCast, hsCast' and hsCoerce???
+hsCast e = hsCoerce (hsCast' e)
 
 hsCast' :: HS.Exp -> HS.Exp
-hsCast' (HS.App e1 e2)     = hsCast' e1 `HS.App` (hsCoerce $ hsCast' e2)
-hsCast' (HS.Lambda _ ps e) = HS.Lambda dummy ps $ hsCast' e
-hsCast' e = e
+hsCast' (HS.InfixApp e1 op e2) = hsCoerce $ HS.InfixApp (hsCast' e1) op (hsCast' e2)
+hsCast' (HS.Lambda _ ps e)     = HS.Lambda dummy ps $ hsCast' e
+hsCast' (HS.Let bs e)          = HS.Let bs $ hsCast' e
+hsCast' (HS.Case sc alts)      = HS.Case (hsCast' sc) (map (hsMapAlt hsCast') alts)
+hsCast' e =
+  case hsAppView e of
+    f : es -> foldl HS.App (hsCoerce f) (map hsCastApp es)
+    _      -> __IMPOSSIBLE__
+
+-- We still have to coerce function applications in arguments to coerced
+-- functions.
+hsCastApp :: HS.Exp -> HS.Exp
+hsCastApp (HS.Lambda i ps b) = HS.Lambda i ps (hsCastApp b)
+hsCastApp (HS.Case sc bs) = HS.Case (hsCastApp sc) (map (hsMapAlt hsCastApp) bs)
+hsCastApp (HS.InfixApp e1 op e2) = HS.InfixApp (hsCastApp e1) op (hsCastApp e2)
+hsCastApp e =
+  case hsAppView e of
+    f : es@(_:_) -> foldl HS.App (hsCoerce f) $ map hsCastApp es
+    _ -> e
 
 -- No coercion for literal integers
 hsCoerce :: HS.Exp -> HS.Exp
 hsCoerce e@(HS.ExpTypeSig _ (HS.Lit (HS.Int{})) _) = e
-hsCoerce e = HS.App mazCoerce e
+hsCoerce (HS.Case sc alts) = HS.Case sc (map (hsMapAlt hsCoerce) alts)
+hsCoerce (HS.Let bs e) = HS.Let bs $ hsCoerce e
+hsCoerce e =
+  case hsAppView e of
+    c : _ | c == mazCoerce || c == mazIncompleteMatch -> e
+    _ -> mazCoerce `HS.App` e
 
 --------------------------------------------------
 -- Writing out a haskell module
 --------------------------------------------------
 
+copyRTEModules :: TCM ()
+copyRTEModules = do
+  dataDir <- lift getDataDir
+  let srcDir = dataDir </> "MAlonzo" </> "src"
+  (lift . copyDirContent srcDir) =<< compileDir
+
 writeModule :: HS.Module -> TCM ()
 writeModule (HS.Module l m ps w ex imp ds) = do
   -- Note that GHC assumes that sources use ASCII or UTF-8.
@@ -749,40 +644,6 @@ writeModule (HS.Module l m ps w ex imp ds) = do
         , "Rank2Types"
         ]
 
-rteModule :: HS.Module
-rteModule = ok $ parse $ unlines
-  [ "module " ++ prettyPrint mazRTE ++ " where"
-  , "import Unsafe.Coerce"
-  , ""
-  , "-- Special version of coerce that plays well with rules."
-  , "{-# INLINE [1] mazCoerce #-}"
-  , "mazCoerce = Unsafe.Coerce.unsafeCoerce"
-  , "{-# RULES \"coerce-id\" forall (x :: a) . mazCoerce x = x #-}"
-  , ""
-  , "-- Builtin QNames, the third field is for the type."
-  , "data QName a b = QName { nameId, moduleId :: Integer, qnameType :: a, qnameDefinition :: b, qnameString :: String}"
-  , "instance Eq (QName a b) where"
-  , "  QName a b _ _ _ == QName c d _ _ _ = (a, b) == (c, d)"
-  , ""
-  , "mazIncompleteMatch :: String -> a"
-  , "mazIncompleteMatch s = error (\"MAlonzo Runtime Error: incomplete pattern matching: \" ++ s)"
-  ]
-  where
-    parse :: String -> HS.ParseResult HS.Module
-    parse = HS.parseModuleWithMode
-              HS.defaultParseMode
-                { HS.extensions = [ HS.EnableExtension HS.ExplicitForAll ] }
-
-    ok :: HS.ParseResult HS.Module -> HS.Module
-    ok (HS.ParseOk d)   = d
-    ok HS.ParseFailed{} = __IMPOSSIBLE__
-
-compileDir :: TCM FilePath
-compileDir = do
-  mdir <- optCompileDir <$> commandLineOptions
-  case mdir of
-    Just dir -> return dir
-    Nothing  -> __IMPOSSIBLE__
 
 outFile' :: (HS.Pretty a, TransformBi HS.ModuleName (Wrap a)) =>
             a -> TCM (FilePath, FilePath)
@@ -803,7 +664,7 @@ outFile m = snd <$> outFile' m
 outFile_ :: TCM FilePath
 outFile_ = outFile =<< curHsMod
 
-callGHC :: Bool -> Interface -> TCM ()
+callGHC :: IsMain -> Interface -> TCM ()
 callGHC modIsMain i = do
   setInterface i
   mdir          <- compileDir
@@ -817,11 +678,11 @@ callGHC modIsMain i = do
 
   let overridableArgs =
         [ "-O"] ++
-        (if modIsMain then ["-o", mdir </> show (nameConcrete outputName)] else []) ++
+        (if modIsMain == IsMain then ["-o", mdir </> show (nameConcrete outputName)] else []) ++
         [ "-Werror"]
       otherArgs       =
         [ "-i" ++ mdir] ++
-        (if modIsMain then ["-main-is", hsmod] else []) ++
+        (if modIsMain == IsMain then ["-main-is", hsmod] else []) ++
         [ fp
         , "--make"
         , "-fwarn-incomplete-patterns"
@@ -833,4 +694,5 @@ callGHC modIsMain i = do
   -- Note: Some versions of GHC use stderr for progress reports. For
   -- those versions of GHC we don't print any progress information
   -- unless an error is encountered.
-  callCompiler compiler args
+  doCall <- optGhcCallGhc <$> commandLineOptions
+  callCompiler doCall compiler args
