@@ -2,8 +2,7 @@
 
 module Agda.Compiler.MAlonzo.Misc where
 
-import Control.Monad.State
-
+import Control.Monad.State (gets)
 import Data.List as List
 import Data.Map as Map
 import Data.Set as Set
@@ -11,6 +10,8 @@ import Data.Function
 
 import qualified Language.Haskell.Exts.Syntax as HS
 
+import Agda.Compiler.Common
+
 import Agda.Syntax.Common
 import Agda.Syntax.Internal
 
@@ -28,39 +29,9 @@ import Agda.Utils.Impossible
 -- Setting up Interface before compile
 --------------------------------------------------
 
-setInterface :: Interface -> TCM ()
-setInterface i = do
-  stImportedModules .= Set.empty
-  stCurrentModule   .= Just (iModuleName i)
-
-curIF :: TCM Interface
-curIF = do
-  mName <- use stCurrentModule
-  case mName of
-    Nothing   -> __IMPOSSIBLE__
-    Just name -> do
-      mm <- getVisitedModule (toTopLevelModuleName name)
-      case mm of
-        Nothing -> __IMPOSSIBLE__
-        Just mi -> return $ miInterface mi
-
-curSig :: TCM Signature
-curSig = iSignature <$> curIF
-
-curMName :: TCM ModuleName
-curMName = sigMName <$> curSig
-
 curHsMod :: TCM HS.ModuleName
 curHsMod = mazMod <$> curMName
 
-curDefs :: TCM Definitions
-curDefs = (^. sigDefinitions) <$> curSig
-
-sigMName :: Signature -> ModuleName
-sigMName sig = case Map.keys (sig ^. sigSections) of
-  []    -> __IMPOSSIBLE__
-  m : _ -> m
-
 --------------------------------------------------
 -- utilities for haskell names
 --------------------------------------------------
@@ -88,34 +59,13 @@ unqhname s q = ihname s (idnum $ nameId $ qnameName $ q)
 
 -- the toplevel module containing the given one
 tlmodOf :: ModuleName -> TCM HS.ModuleName
-tlmodOf = fmap mazMod . tlmname
+tlmodOf = fmap mazMod . topLevelModuleName
 
-tlmname :: ModuleName -> TCM ModuleName
-tlmname m = do
-  -- get the names of the visited modules
-  visited <- List.map (iModuleName . miInterface) . Map.elems <$>
-    getVisitedModules
-  -- find the module with the longest matching prefix to m
-  let ms = sortBy (compare `on` (length . mnameToList)) $
-       List.filter (\ m' -> mnameToList m' `isPrefixOf` mnameToList m) visited
-  case ms of
-    (m' : _) -> return m'
-    -- if we did not get anything, it may be because m is a section
-    -- (a module _ )
-    []       -> curMName
-{-  NOT IMPOSSIBLE!
-    _ -> do
-      reportSDoc "impossible" 10 $ sep
-        [ text $ "m = " ++ show m
-        , text $ "should be contained in the visited modules " ++ show visited
-        ]
-      __IMPOSSIBLE__
--}
 
 -- qualify HS.Name n by the module of QName q, if necessary;
 -- accumulates the used module in stImportedModules at the same time.
 xqual :: QName -> HS.Name -> TCM HS.QName
-xqual q n = do m1 <- tlmname (qnameModule q)
+xqual q n = do m1 <- topLevelModuleName (qnameModule q)
                m2 <- curMName
                if m1 == m2 then return (HS.UnQual n)
                   else addImport m1 >> return (HS.Qual (mazMod m1) n)
@@ -123,13 +73,16 @@ xqual q n = do m1 <- tlmname (qnameModule q)
 xhqn :: String -> QName -> TCM HS.QName
 xhqn s q = xqual q (unqhname s q)
 
+hsName :: String -> HS.QName
+hsName s = HS.UnQual (HS.Ident s)
+
 -- always use the original name for a constructor even when it's redefined.
 conhqn :: QName -> TCM HS.QName
 conhqn q = do
     cq  <- canonicalName q
     def <- getConstInfo cq
     case (compiledHaskell (defCompiledRep def), theDef def) of
-      (Just (HsDefn _ hs), Constructor{}) -> return $ HS.UnQual $ HS.Ident hs
+      (Just (HsDefn _ hs), Constructor{}) -> return $ hsName hs
       _                                   -> xhqn "C" cq
 
 -- qualify name s by the module of builtin b
@@ -138,14 +91,58 @@ bltQual b s = do
   Def q _ <- ignoreSharing <$> getBuiltin b
   xqual q (HS.Ident s)
 
--- Sub-naming for cascaded definitions for consecutive clauses.
-dsubname :: QName -> Nat -> HS.Name
-dsubname q i | i == 0    = unqhname "d"                     q
-             | otherwise = unqhname ("d_" ++ show i ++ "_") q
+dname :: QName -> HS.Name
+dname q = unqhname "d" q
+
+-- | Name for definition stripped of unused arguments
+duname :: QName -> HS.Name
+duname q = unqhname "du" q
+
+hsPrimOp :: String -> HS.QOp
+hsPrimOp s = HS.QVarOp $ HS.UnQual $ HS.Symbol s
+
+hsPrimOpApp :: String -> HS.Exp -> HS.Exp -> HS.Exp
+hsPrimOpApp op e e1 = HS.InfixApp e (hsPrimOp op) e1
+
+hsInt :: Integer -> HS.Exp
+hsInt n = HS.Lit (HS.Int n)
+
+hsTypedInt :: Integer -> HS.Exp
+hsTypedInt n = HS.ExpTypeSig dummy (HS.Lit (HS.Int n)) (HS.TyCon (hsName "Integer"))
+
+hspLet :: HS.Pat -> HS.Exp -> HS.Exp -> HS.Exp
+hspLet p e b =
+  HS.Let (HS.BDecls [HS.PatBind dummy p (HS.UnGuardedRhs e) emptyBinds]) b
+
+hsLet :: HS.Name -> HS.Exp -> HS.Exp -> HS.Exp
+hsLet x e b =
+  HS.Let (HS.BDecls [HS.FunBind [HS.Match dummy x [] Nothing (HS.UnGuardedRhs e) emptyBinds]]) b
 
 hsVarUQ :: HS.Name -> HS.Exp
 hsVarUQ = HS.Var . HS.UnQual
 
+hsAppView :: HS.Exp -> [HS.Exp]
+hsAppView = reverse . view
+  where
+    view (HS.App e e1) = e1 : view e
+    view (HS.InfixApp e1 op e2) = [e2, e1, hsOpToExp op]
+    view e = [e]
+
+hsOpToExp :: HS.QOp -> HS.Exp
+hsOpToExp (HS.QVarOp x) = HS.Var x
+hsOpToExp (HS.QConOp x) = HS.Con x
+
+hsLambda :: [HS.Pat] -> HS.Exp -> HS.Exp
+hsLambda ps (HS.Lambda i ps1 e) = HS.Lambda i (ps ++ ps1) e
+hsLambda ps e = HS.Lambda dummy ps e
+
+hsMapAlt :: (HS.Exp -> HS.Exp) -> HS.Alt -> HS.Alt
+hsMapAlt f (HS.Alt i p rhs wh) = HS.Alt i p (hsMapRHS f rhs) wh
+
+hsMapRHS :: (HS.Exp -> HS.Exp) -> HS.Rhs -> HS.Rhs
+hsMapRHS f (HS.UnGuardedRhs def) = HS.UnGuardedRhs (f def)
+hsMapRHS f (HS.GuardedRhss es)   = HS.GuardedRhss [ HS.GuardedRhs i g (f e) | HS.GuardedRhs i g e <- es ]
+
 --------------------------------------------------
 -- Hard coded module names
 --------------------------------------------------
@@ -165,9 +162,16 @@ mazMod = mazMod' . prettyShow
 mazerror :: String -> a
 mazerror msg = error $ mazstr ++ ": " ++ msg
 
+mazCoerceName :: String
+mazCoerceName = "coe"
+
+mazErasedName :: String
+mazErasedName = "erased"
+
 mazCoerce :: HS.Exp
 -- mazCoerce = HS.Var $ HS.Qual unsafeCoerceMod (HS.Ident "unsafeCoerce")
-mazCoerce = HS.Var $ HS.Qual mazRTE $ HS.Ident "mazCoerce"
+-- mazCoerce = HS.Var $ HS.Qual mazRTE $ HS.Ident mazCoerceName
+mazCoerce = HS.Var $ HS.UnQual $ HS.Ident mazCoerceName
 
 -- Andreas, 2011-11-16: error incomplete match now RTE-call
 mazIncompleteMatch :: HS.Exp
@@ -176,13 +180,15 @@ mazIncompleteMatch = HS.Var $ HS.Qual mazRTE $ HS.Ident "mazIncompleteMatch"
 rtmIncompleteMatch :: QName -> HS.Exp
 rtmIncompleteMatch q = mazIncompleteMatch `HS.App` hsVarUQ (unqhname "name" q)
 
+mazUnreachableError :: HS.Exp
+mazUnreachableError = HS.Var $ HS.Qual mazRTE $ HS.Ident "mazUnreachableError"
+
+rtmUnreachableError :: HS.Exp
+rtmUnreachableError = mazUnreachableError
+
 mazRTE :: HS.ModuleName
 mazRTE = HS.ModuleName "MAlonzo.RTE"
 
--- for Runtime module: Not really used (Runtime modules has been abolished).
-rtmMod :: HS.ModuleName
-rtmMod = mazMod' "Runtime"
-
 rtmQual :: String -> HS.QName
 rtmQual = HS.UnQual . HS.Ident
 
@@ -218,6 +224,9 @@ fakeType = HS.TyVar . HS.Ident
 fakeExp :: String -> HS.Exp
 fakeExp = HS.Var . HS.UnQual . HS.Ident
 
+fakeDecl :: String -> HS.Decl
+fakeDecl s = HS.TypeSig dummy [HS.Ident (s ++ " {- OMG hack")] (HS.TyVar $ HS.Ident "-}")
+
 dummy :: a
 dummy = error "MAlonzo : this dummy value should not have been eval'ed."
 
