@@ -13,6 +13,8 @@ module Agda.Interaction.EmacsCommand
   , displayRunningInfo
   ) where
 
+import qualified Data.List as List
+
 import Agda.Utils.Pretty
 import Agda.Utils.String
 
@@ -34,8 +36,14 @@ instance Pretty a => Pretty (Lisp a) where
   pretty (L xs)     = parens (hsep (map pretty xs))
   pretty (Q x)      = text "'" <> pretty x
 
-instance Pretty a => Show (Lisp a) where
-  show = show . pretty
+instance Show (Lisp String) where
+  showsPrec _ (A a)      = showString a
+  showsPrec p (Cons a b) = showString "(" . showsPrec p a . showString " . " .
+                                            showsPrec p b . showString ")"
+  showsPrec p (L xs)     = showString "(" . foldr (.) (showString ")")
+                                              (List.intersperse (showString " ")
+                                                 (map (showsPrec p) xs))
+  showsPrec p (Q x)      = showString "'" . showsPrec p x
 
 -- | Formats a response command.
 --
