@@ -20,11 +20,13 @@ import Agda.Utils.String
 import Agda.Syntax.Common
 
 import Agda.TypeChecking.Monad
+import qualified Agda.TypeChecking.Monad.Benchmark as Bench
 
 import Agda.Interaction.Response as R
 import Agda.Interaction.InteractionTop
 import Agda.Interaction.EmacsCommand
 import Agda.Interaction.Highlighting.Emacs
+import Agda.Interaction.Options
 
 import Agda.Version
 
@@ -36,26 +38,34 @@ import Agda.Version
 --   'mimicGHCi' reads the Emacs frontend commands from stdin,
 --   interprets them and print the result into stdout.
 
-mimicGHCi :: TCM ()
-mimicGHCi = do
-
+mimicGHCi :: TCM () -> TCM ()
+mimicGHCi setup = do
     liftIO $ do
-      hSetBuffering stdout NoBuffering
+      hSetBuffering stdout LineBuffering
+      hSetBuffering stdin  LineBuffering
       hSetEncoding  stdout utf8
       hSetEncoding  stdin  utf8
 
     setInteractionOutputCallback $
         liftIO . mapM_ print <=< lispifyResponse
 
+    handleCommand_ (lift setup) `evalStateT` initCommandState
+
     opts <- commandLineOptions
-    _ <- interact' `runStateT` initCommandState { optionsOnReload = opts }
+    _ <- interact' `runStateT` initCommandState { optionsOnReload = opts{ optAbsoluteIncludePaths = [] } }
     return ()
   where
 
     interact' :: CommandM ()
     interact' = do
-        liftIO $ putStr "Agda2> "
-        unlessM (liftIO isEOF) $ do
+      Bench.reset
+      done <- Bench.billTo [] $ do
+
+        liftIO $ do
+          putStr "Agda2> "
+          hFlush stdout
+        done <- liftIO isEOF
+        unless done $ do
             r <- liftIO getLine
             _ <- return $! length r     -- force to read the full input line
             case dropWhile isSpace r of
@@ -65,8 +75,10 @@ mimicGHCi = do
                     Just (x, "")  -> runInteraction x
                     Just (_, rem) -> liftIO $ putStrLn $ "not consumed: " ++ rem
                     _ ->             liftIO $ putStrLn $ "cannot read: " ++ r
-            interact'
+        return done
 
+      lift Bench.print
+      unless done interact'
 
 -- | Convert Response to an elisp value for the interactive emacs frontend.
 
@@ -79,12 +91,13 @@ lispifyResponse (Resp_DisplayInfo info) = return $ case info of
     Info_AllGoals s -> f s "*All Goals*"
     Info_Auto s -> f s "*Auto*"
     Info_Error s -> f s "*Error*"
-
+    Info_Time s -> f (render s) "*Time*"
     Info_NormalForm s -> f (render s) "*Normal Form*"   -- show?
     Info_InferredType s -> f (render s) "*Inferred Type*"
     Info_CurrentGoal s -> f (render s) "*Current Goal*"
     Info_GoalType s -> f (render s) "*Goal type etc.*"
     Info_ModuleContents s -> f (render s) "*Module contents*"
+    Info_SearchAbout s -> f (render s) "*Search About*"
     Info_WhyInScope s -> f (render s) "*Scope Info*"
     Info_Context s -> f (render s) "*Context*"
     Info_HelperFunction s -> [ L [ A "agda2-info-action-and-copy"
@@ -114,7 +127,7 @@ lispifyResponse (Resp_Status s)
 
 lispifyResponse (Resp_JumpToError f p) = return
   [ lastTag 3 $
-      L [ A "agda2-goto", Q $ L [A (quote f), A ".", A (show p)] ]
+      L [ A "agda2-maybe-goto", Q $ L [A (quote f), A ".", A (show p)] ]
   ]
 lispifyResponse (Resp_InteractionPoints is) = return
   [ lastTag 1 $
