@@ -1,6 +1,6 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP              #-}
 {-# LANGUAGE FlexibleContexts #-}
-{-# LANGUAGE RelaxedPolyRec #-}
+{-# LANGUAGE RelaxedPolyRec   #-}
 
 -- | Generates data used for precise syntax highlighting.
 
@@ -8,6 +8,7 @@ module Agda.Interaction.Highlighting.Generate
   ( Level(..)
   , generateAndPrintSyntaxInfo
   , generateTokenInfo, generateTokenInfoFromString
+  , printSyntaxInfo
   , printErrorInfo, errorHighlighting
   , printUnsolvedInfo
   , printHighlightingInfo
@@ -22,23 +23,19 @@ import Prelude hiding (null)
 
 import Control.Monad
 import Control.Monad.Trans
-import Control.Monad.State
 import Control.Monad.Reader
 import Control.Applicative
-import Control.Arrow ((***), first, second)
+import Control.Arrow (second)
 
 import Data.Monoid
 import Data.Generics.Geniplate
-import Data.HashSet (HashSet)
-import qualified Data.HashSet as HSet
 import qualified Data.Map as Map
 import Data.Maybe
 import Data.List ((\\), isPrefixOf)
-import qualified Data.Foldable as Fold (toList, fold, foldMap)
-import Data.IntMap (IntMap)
+import qualified Data.Foldable as Fold (fold, foldMap)
 import qualified Data.IntMap as IntMap
+import Data.Void
 
-import Agda.Interaction.FindFile
 import Agda.Interaction.Response (Response(Resp_HighlightingInfo))
 import Agda.Interaction.Highlighting.Precise hiding (tests)
 import Agda.Interaction.Highlighting.Range   hiding (tests)
@@ -48,11 +45,9 @@ import Agda.TypeChecking.MetaVars (isBlockedTerm)
 import Agda.TypeChecking.Monad
   hiding (MetaInfo, Primitive, Constructor, Record, Function, Datatype)
 import qualified Agda.TypeChecking.Monad as M
-import Agda.TypeChecking.Pretty
-import qualified Agda.TypeChecking.Reduce as R
 
 import qualified Agda.Syntax.Abstract as A
-import Agda.Syntax.Common (Delayed(..))
+import Agda.Syntax.Concrete (FieldAssignment'(..))
 import qualified Agda.Syntax.Common as Common
 import qualified Agda.Syntax.Concrete as C
 import qualified Agda.Syntax.Info as SI
@@ -67,6 +62,7 @@ import Agda.Utils.Functor
 import Agda.Utils.Lens
 import Agda.Utils.List
 import Agda.Utils.Maybe
+import qualified Agda.Utils.Maybe.Strict as Strict
 import Agda.Utils.Null
 import Agda.Utils.TestHelpers
 import Agda.Utils.HashMap (HashMap)
@@ -157,6 +153,8 @@ generateAndPrintSyntaxInfo decl hlLevel = do
             Partial {} -> "(first approximation)"
         ++ "."
 
+  reportSLn "highlighting.names" 60 $ "highlighting names = " ++ show names
+
   M.ignoreAbstractMode $ do
     modMap <- sourceToModule
     kinds  <- nameKinds hlLevel decl
@@ -217,6 +215,7 @@ generateAndPrintSyntaxInfo decl hlLevel = do
     , Fold.foldMap getLam         $ universeBi decl
     , Fold.foldMap getTyped       $ universeBi decl
     , Fold.foldMap getPattern     $ universeBi decl
+    , Fold.foldMap getPatternSyn  $ universeBi decl
     , Fold.foldMap getExpr        $ universeBi decl
     , Fold.foldMap getPatSynArgs  $ universeBi decl
     , Fold.foldMap getModuleName  $ universeBi decl
@@ -231,6 +230,9 @@ generateAndPrintSyntaxInfo decl hlLevel = do
     patsyn n = nameToFileA modMap file n True $ \isOp ->
                   mempty { aspect = Just $ Name (Just $ Constructor Common.Inductive) isOp }
 
+    macro n = nameToFileA modMap file n True $ \isOp ->
+                  mempty { aspect = Just $ Name (Just Macro) isOp }
+
     field m n = nameToFile modMap file m n
                            (\isOp -> mempty { aspect = Just $ Name (Just Field) isOp })
                            Nothing
@@ -247,9 +249,10 @@ generateAndPrintSyntaxInfo decl hlLevel = do
                            (A.nameBindingSite n))
 
     getVarAndField :: A.Expr -> File
-    getVarAndField (A.Var x)    = bound x
-    getVarAndField (A.Rec _ fs) = mconcat $ map (field [] . fst) fs
-    getVarAndField _            = mempty
+    getVarAndField (A.Var x)            = bound x
+    getVarAndField (A.Rec       _ fs)   = mconcat [ field [] x | Left (FieldAssignment x _) <- fs ]
+    getVarAndField (A.RecUpdate _ _ fs) = mconcat [ field [] x |      (FieldAssignment x _) <- fs ]
+    getVarAndField _                    = mempty
 
     -- Ulf, 2014-04-09: It would be nicer to have it on Named_ a, but
     -- you can't have polymorphic functions in universeBi.
@@ -257,10 +260,11 @@ generateAndPrintSyntaxInfo decl hlLevel = do
     getNamedArg x = singleton (rToR $ P.getRange x) mempty{ aspect = Just $ Name (Just Argument) False }
 
     getLet :: A.LetBinding -> File
-    getLet (A.LetBind _ _ x _ _) = bound x
-    getLet A.LetPatBind{}        = mempty
-    getLet A.LetApply{}          = mempty
-    getLet A.LetOpen{}           = mempty
+    getLet (A.LetBind _ _ x _ _)     = bound x
+    getLet A.LetPatBind{}            = mempty
+    getLet A.LetApply{}              = mempty
+    getLet A.LetOpen{}               = mempty
+    getLet (A.LetDeclaredVariable x) = bound x
 
     getLam :: A.LamBinding -> File
     getLam (A.DomainFree _ x) = bound x
@@ -274,21 +278,29 @@ generateAndPrintSyntaxInfo decl hlLevel = do
     getPatSynArgs (A.PatternSynDef _ xs _) = mconcat $ map (bound . Common.unArg) xs
     getPatSynArgs _                        = mempty
 
-    getPattern :: A.Pattern -> File
-    getPattern (A.VarP x)    = bound x
-    getPattern (A.AsP _ x _) = bound x
-    getPattern (A.DotP pi _) =
+    getPattern' :: A.Pattern' e -> File
+    getPattern' (A.VarP x)    = bound x
+    getPattern' (A.AsP _ x _) = bound x
+    getPattern' (A.DotP pi _) =
       singleton (rToR $ P.getRange pi)
                 (mempty { otherAspects = [DottedPattern] })
-    getPattern (A.PatternSynP _ q _) = patsyn q
-    getPattern _             = mempty
+    getPattern' (A.PatternSynP _ q _) = patsyn q
+    getPattern' (A.RecP _ fs) = mconcat [ field [] x | FieldAssignment x _ <- fs ]
+    getPattern' _             = mempty
+
+    getPattern :: A.Pattern -> File
+    getPattern = getPattern'
+
+    getPatternSyn :: A.Pattern' Void -> File
+    getPatternSyn = getPattern'
 
     getExpr :: A.Expr -> File
     getExpr (A.PatternSyn q) = patsyn q
+    getExpr (A.Macro q)      = macro q
     getExpr _                = mempty
 
     getFieldDecl :: A.Declaration -> File
-    getFieldDecl (A.RecDef _ _ _ _ _ _ fs) = Fold.foldMap extractField fs
+    getFieldDecl (A.RecDef _ _ _ _ _ _ _ fs) = Fold.foldMap extractField fs
       where
       extractField (A.ScopedDecl _ ds) = Fold.foldMap extractField ds
       extractField (A.Field _ x _)     = field (concreteQualifier x)
@@ -303,7 +315,7 @@ generateAndPrintSyntaxInfo decl hlLevel = do
       isTopLevelModule =
         case catMaybes $
              map (join .
-                  fmap P.srcFile .
+                  fmap (Strict.toLazy . P.srcFile) .
                   P.rStart .
                   A.nameBindingSite) xs of
           f : _ -> Map.lookup f modMap ==
@@ -351,11 +363,12 @@ tokenHighlighting = merge . map tokenToCFile
   tokenToCFile (T.TokKeyword T.KwForall i)      = aToF Symbol (P.getRange i)
   tokenToCFile (T.TokKeyword _ i)               = aToF Keyword (P.getRange i)
   tokenToCFile (T.TokSymbol  _ i)               = aToF Symbol (P.getRange i)
-  tokenToCFile (T.TokLiteral (L.LitInt    r _)) = aToF Number r
+  tokenToCFile (T.TokLiteral (L.LitNat    r _)) = aToF Number r
   tokenToCFile (T.TokLiteral (L.LitFloat  r _)) = aToF Number r
   tokenToCFile (T.TokLiteral (L.LitString r _)) = aToF String r
   tokenToCFile (T.TokLiteral (L.LitChar   r _)) = aToF String r
   tokenToCFile (T.TokLiteral (L.LitQName  r _)) = aToF String r
+  tokenToCFile (T.TokLiteral (L.LitMeta r _ _)) = aToF String r
   tokenToCFile (T.TokComment (i, _))            = aToF Comment (P.getRange i)
   tokenToCFile (T.TokTeX (i, _))                = aToF Comment (P.getRange i)
   tokenToCFile (T.TokId {})                     = mempty
@@ -384,7 +397,7 @@ nameKinds hlLevel decl = do
       -- Traverses the syntax tree and constructs a map from qualified
       -- names to name kinds. TODO: Handle open public.
   let syntax = foldr ($) HMap.empty $ map declToKind $ universeBi decl
-  let merged = HMap.unions [local, imported, syntax]
+  let merged = unions [local, imported, syntax]
   return (\n -> HMap.lookup n merged)
   where
   fix = HMap.map (defnToKind . theDef) . (^. sigDefinitions)
@@ -394,10 +407,12 @@ nameKinds hlLevel decl = do
   -- are thrown away whenever possible. The 'declToKind' function
   -- below can return several explanations for one qualified name; the
   -- 'Postulate's are bogus.
-  insert = HMap.insertWith dropPostulates
-    where
-    dropPostulates Postulate k = k
-    dropPostulates k         _ = k
+  merge Postulate k = k
+  merge _     Macro = Macro  -- If the abstract syntax says macro, it's a macro.
+  merge k         _ = k
+
+  unions = foldr (HMap.unionWith merge) HMap.empty
+  insert = HMap.insertWith merge
 
   defnToKind :: Defn -> NameKind
   defnToKind   M.Axiom{}                           = Postulate
@@ -410,7 +425,9 @@ nameKinds hlLevel decl = do
 
   declToKind :: A.Declaration ->
                 HashMap A.QName NameKind -> HashMap A.QName NameKind
-  declToKind (A.Axiom _ _ _ q _)    = insert q Postulate
+  declToKind (A.Axiom _ i _ q _)
+    | SI.defMacro i == Common.MacroDef = insert q Macro
+    | otherwise                        = insert q Postulate
   declToKind (A.Field _ q _)        = insert q Field -- Function
     -- Note that the name q can be used both as a field name and as a
     -- projection function. Highlighting of field names is taken care
@@ -424,8 +441,9 @@ nameKinds hlLevel decl = do
   declToKind (A.ScopedDecl {})      = id
   declToKind (A.Open {})            = id
   declToKind (A.PatternSynDef q _ _) = insert q (Constructor Common.Inductive)
-  declToKind (A.FunDef  _ q _ _)    = insert q Function
-  declToKind (A.UnquoteDecl _ _ q _) = insert q Function
+  declToKind (A.FunDef  _ q _ _)     = insert q Function
+  declToKind (A.UnquoteDecl _ _ qs _) = foldr (\ q f -> insert q Function . f) id qs
+  declToKind (A.UnquoteDef _ qs _)    = foldr (\ q f -> insert q Function . f) id qs
   declToKind (A.DataSig _ q _ _)    = insert q Datatype
   declToKind (A.DataDef _ q _ cs)   = \m ->
                                       insert q Datatype $
@@ -433,7 +451,7 @@ nameKinds hlLevel decl = do
                                                           (Constructor Common.Inductive))
                                             m cs
   declToKind (A.RecSig _ q _ _)     = insert q Record
-  declToKind (A.RecDef _ q _ c _ _ _) = insert q Record .
+  declToKind (A.RecDef _ q _ _ c _ _ _) = insert q Record .
                                       case c of
                                         Nothing -> id
                                         Just q  -> insert q (Constructor Common.Inductive)
@@ -456,7 +474,8 @@ generateConstructorInfo modMap file kinds decl = do
   -- Get boundaries of current declaration.
   -- @noRange@ should be impossible, but in case of @noRange@
   -- it makes sense to return the empty File.
-  ifNull (P.getRange decl) (return mempty) $ \ (P.Range is) -> do
+  ifNull (P.rangeIntervals $ P.getRange decl)
+         (return mempty) $ \is -> do
     let start = fromIntegral $ P.posPos $ P.iStart $ head is
         end   = fromIntegral $ P.posPos $ P.iEnd   $ last is
 
@@ -470,6 +489,12 @@ generateConstructorInfo modMap file kinds decl = do
     let files = for constrs $ \ q -> generate modMap file kinds $ A.AmbQ [q]
     return $ Fold.fold files
 
+printSyntaxInfo :: P.Range -> TCM ()
+printSyntaxInfo r = do
+  syntaxInfo <- use stSyntaxInfo
+  ifTopLevelAndHighlightingLevelIs NonInteractive $
+      printHighlightingInfo (selectC r syntaxInfo)
+
 
 -- | Prints syntax highlighting info for an error.
 
@@ -599,7 +624,7 @@ nameToFile :: SourceToModule
            -> File
 nameToFile modMap file xs x m mR =
   -- We don't care if we get any funny ranges.
-  if all (== Just file) fileNames then
+  if all (== Strict.Just file) fileNames then
     several (map rToR rs)
             ((m $ C.isOperator x) { definitionSite = mFilePos })
    else
@@ -609,7 +634,7 @@ nameToFile modMap file xs x m mR =
   rs         = map P.getRange (x : xs)
   mFilePos   = do
     r <- mR
-    P.Pn { P.srcFile = Just f, P.posPos = p } <- P.rStart r
+    P.Pn { P.srcFile = Strict.Just f, P.posPos = p } <- P.rStart r
     mod <- Map.lookup f modMap
     return (mod, fromIntegral p)
 
@@ -652,9 +677,7 @@ storeDisambiguatedName :: A.QName -> TCM ()
 storeDisambiguatedName q = whenJust (start $ P.getRange q) $ \ i ->
   stDisambiguatedNames %= IntMap.insert i q
   where
-    start (P.Range [])    = Nothing
-    start (P.Range (i:_)) = Just $ fromIntegral $ P.posPos $ P.iStart i
-    -- TODO: Move start to Agda.Syntax.Position
+  start r = fromIntegral . P.posPos <$> P.rStart' r
 
 ------------------------------------------------------------------------
 -- All tests
