@@ -23,7 +23,6 @@ import qualified Data.ByteString    as BS
 
 import qualified Data.IntMap as IntMap
 import qualified Data.List   as List
-import qualified Data.Map    as Map
 
 import Paths_Agda
 
@@ -37,7 +36,6 @@ import Agda.Interaction.Options
 import Agda.Compiler.CallCompiler
 import qualified Agda.Utils.IO.UTF8 as UTF8
 import Agda.Utils.FileName (filePath)
-import Agda.Utils.Pretty (pretty, render)
 
 import Agda.Utils.Except ( ExceptT, MonadError(throwError), runExceptT )
 
@@ -340,29 +338,35 @@ code = do
 
   case aspect (info tok') of
     Nothing -> output $ escape tok
---    Just a  -> output $ cmdPrefix <+> T.pack (cmd a) <+> cmdArg (escape tok)
---  Andreas, 2014-02-17 preliminary fix for issue 1062
-    Just a  -> case cmd a of
-      "" -> output $ escape tok
-      s  -> output $ cmdPrefix <+> T.pack s <+> cmdArg (escape tok)
+    Just a  -> output $ cmdPrefix <+> T.pack (cmd a) <+> cmdArg (escape tok)
 
   code
 
   where
   cmd :: Aspect -> String
---  cmd (Name mKind _) = maybe __IMPOSSIBLE__ showKind mKind
---  Andreas, 2014-02-17 preliminary fix for issue 1062
-  cmd (Name mKind _) = maybe "" showKind mKind
-    where
-    showKind :: NameKind -> String
-    showKind (Constructor Inductive)   = "InductiveConstructor"
-    showKind (Constructor CoInductive) = "CoinductiveConstructor"
-    -- Andreas, 2014-02-17
-    -- It might be boring boilerplate, but please spell out the
-    -- remaining cases instead of using the brittle @show@ function.
-    -- What if a constructor in @NameKind@ gets renamed?
-    showKind k                         = show k
-  cmd a              = show a
+  cmd a = let s = show a in case a of
+    Comment        -> s
+    Keyword        -> s
+    String         -> s
+    Number         -> s
+    Symbol         -> s
+    PrimitiveType  -> s
+    Name mKind _   -> maybe __IMPOSSIBLE__ showKind mKind
+      where
+      showKind :: NameKind -> String
+      showKind n = let s = show n in case n of
+        Bound                     -> s
+        Constructor Inductive     -> "InductiveConstructor"
+        Constructor CoInductive   -> "CoinductiveConstructor"
+        Datatype                  -> s
+        Field                     -> s
+        Function                  -> s
+        Module                    -> s
+        Postulate                 -> s
+        Primitive                 -> s
+        Record                    -> s
+        Argument                  -> s
+        Macro                     -> s
 
 -- Escapes special characters.
 escape :: Text -> Text
@@ -378,9 +382,9 @@ escape (T.uncons -> Just (c, s)) = T.pack (replace c) <+> escape s
     '$'  -> "\\$"
     '&'  -> "\\&"
     '%'  -> "\\%"
-    '~'  -> "\\textasciitilde"
-    '^'  -> "\\textasciicircum"
-    '\\' -> "\\textbackslash"
+    '~'  -> "\\textasciitilde{}"
+    '^'  -> "\\textasciicircum{}"
+    '\\' -> "\\textbackslash{}"
     -- Escaping newlines seems to fix the problem caused by pattern
     -- synonyms.
     '\n' -> "\\<\\\\\n\\>"
