@@ -726,6 +726,11 @@ buildInterface file topLevel syntaxInfo previousHsImports previousHsImportsUHC p
     -- Andreas, Makoto, 2014-10-18 AIM XX: repeating the experiment
     -- with discarding also the nameBindingSite in QName:
     -- Saves 10% on serialization time (and file size)!
+    --
+    -- NOTE: We no longer discard all nameBindingSites (but the commit
+    -- that introduced this change seems to have made Agda a bit
+    -- faster and interface file sizes a bit smaller, at least for the
+    -- standard library).
     builtin <- use stLocalBuiltins
     ms      <- getImports
     mhs     <- mapM (\ m -> (m,) <$> moduleHash m) $ Set.toList ms
@@ -736,7 +741,8 @@ buildInterface file topLevel syntaxInfo previousHsImports previousHsImportsUHC p
     -- Non-closed display forms are not applicable outside the module anyway,
     -- and should be dead-code eliminated (#1928).
     display <- HMap.filter (not . null) . HMap.map (filter isClosed) <$> use stImportsDisplayForms
-    (display, sig) <- second killRange <$> (eliminateDeadCode display =<< getSignature)
+    -- TODO: Kill some ranges?
+    (display, sig) <- eliminateDeadCode display =<< getSignature
     -- Andreas, 2015-02-09 kill ranges in pattern synonyms before
     -- serialization to avoid error locations pointing to external files
     -- when expanding a pattern synoym.
