@@ -1,14 +1,14 @@
-{-# OPTIONS -fno-cse #-}
-
-{-# LANGUAGE CPP #-}
-{-# LANGUAGE DeriveFoldable #-}
-{-# LANGUAGE DeriveFunctor #-}
-{-# LANGUAGE DeriveTraversable #-}
-{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE CPP                   #-}
+{-# LANGUAGE DeriveFoldable        #-}
+{-# LANGUAGE DeriveFunctor         #-}
+{-# LANGUAGE DeriveTraversable     #-}
+{-# LANGUAGE FlexibleInstances     #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
-{-# LANGUAGE Rank2Types #-}
-{-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE TupleSections #-}
+{-# LANGUAGE Rank2Types            #-}
+{-# LANGUAGE ScopedTypeVariables   #-}
+{-# LANGUAGE TupleSections         #-}
+
+{-# OPTIONS_GHC -fno-cse #-}
 
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
@@ -63,6 +63,7 @@ import Agda.Interaction.FindFile
 import Agda.Interaction.Options
 import Agda.Interaction.Options.Lenses as Lenses
 import Agda.Interaction.MakeCase
+import Agda.Interaction.SearchAbout
 import Agda.Interaction.Response hiding (Function, ExtendedLambda)
 import qualified Agda.Interaction.Response as R
 import qualified Agda.Interaction.BasicOps as B
@@ -71,6 +72,7 @@ import qualified Agda.Interaction.Imports as Imp
 import Agda.Interaction.Highlighting.Generate
 import qualified Agda.Interaction.Highlighting.Range as H
 
+import Agda.Compiler.Common (IsMain (..))
 import qualified Agda.Compiler.Epic.Compiler as Epic
 import qualified Agda.Compiler.MAlonzo.Compiler as MAlonzo
 import qualified Agda.Compiler.JS.Compiler as JS
@@ -89,6 +91,7 @@ import Agda.Utils.Hash
 import qualified Agda.Utils.HashMap as HMap
 import Agda.Utils.Lens
 import Agda.Utils.Maybe
+import qualified Agda.Utils.Maybe.Strict as Strict
 import Agda.Utils.Monad
 import Agda.Utils.Null
 import Agda.Utils.Pretty
@@ -116,7 +119,7 @@ data CommandState = CommandState
     -- the file when it was last loaded.
   , optionsOnReload :: CommandLineOptions
     -- ^ Reset the options on each reload to these.
-  , oldInteractionScopes :: OldInteractionScopes
+  , oldInteractionScopes :: !OldInteractionScopes
     -- ^ We remember (the scope of) old interaction points to make it
     --   possible to parse and compute highlighting information for the
     --   expression that it got replaced by.
@@ -202,56 +205,27 @@ getOldInteractionScope ii = do
     Nothing    -> fail $ "not an old interaction point: " ++ show ii
     Just scope -> return scope
 
--- | Run an 'IOTCM' value, catch the exceptions, emit output
---
---   If an error happens the state of 'CommandM' does not change,
---   but stPersistent may change (which contains successfully
---   loaded interfaces for example).
+-- | Do setup and error handling for a command.
 
-runInteraction :: IOTCM -> CommandM ()
-runInteraction (IOTCM current highlighting highlightingMethod cmd)
-    = handleNastyErrors
-    $ inEmacs
-    $ do
-        current <- liftIO $ absolute current
+handleCommand_ :: CommandM () -> CommandM ()
+handleCommand_ = handleCommand id (return ())
 
-        res <- (`catchErr` (return . Just)) $ do
-
-            -- Raises an error if the given file is not the one currently
-            -- loaded.
-            cf <- gets theCurrentFile
-            when (not (independent cmd) && Just current /= (fst <$> cf)) $
-                lift $ typeError $ GenericError "Error: First load the file."
-
-            withCurrentFile $ interpret cmd
-
-            cf <- gets theCurrentFile
-            when (Just current == (fst <$> cf)) $
-                putResponse . Resp_InteractionPoints =<< gets theInteractionPoints
-            return Nothing
-
-        maybe (return ()) handleErr res
+handleCommand :: (forall a. CommandM a -> CommandM a) -> CommandM () -> CommandM () -> CommandM ()
+handleCommand wrap onFail cmd = handleNastyErrors $ wrap $ do
+    res <- (`catchErr` (return . Just)) $ Nothing <$ cmd
+    maybe (return ()) (\ e -> onFail >> handleErr e) res
 
   where
     -- Preserves state so we can do unsolved meta highlighting
     catchErr :: CommandM a -> (TCErr -> CommandM a) -> CommandM a
     catchErr m h = do
       s       <- get
-      -- If an independent command fails we should reset theCurrentFile (Issue853).
-      let sErr | independent cmd = s { theCurrentFile = Nothing }
-               | otherwise       = s
-      (x, s') <- lift $ do
-           disableDestructiveUpdate (runStateT m s)
+      (x, s') <- lift $ do disableDestructiveUpdate (runStateT m s)
          `catchError_` \ e ->
-           runStateT (h e) sErr
+           runStateT (h e) s
       put s'
       return x
 
-    inEmacs = liftCommandMT $ withEnv $ initEnv
-            { envHighlightingLevel  = highlighting
-            , envHighlightingMethod = highlightingMethod
-            }
-
     -- | Handle nasty errors like stack space overflow (issue 637)
     -- We assume that the input action handles other kind of errors.
     handleNastyErrors :: CommandM () -> CommandM ()
@@ -280,6 +254,37 @@ runInteraction (IOTCM current highlighting highlightingMethod cmd)
                                    , sShowImplicitArguments = x
                                    } ]
 
+-- | Run an 'IOTCM' value, catch the exceptions, emit output
+--
+--   If an error happens the state of 'CommandM' does not change,
+--   but stPersistent may change (which contains successfully
+--   loaded interfaces for example).
+
+runInteraction :: IOTCM -> CommandM ()
+runInteraction (IOTCM current highlighting highlightingMethod cmd) =
+  handleCommand inEmacs onFail $ do
+    current <- liftIO $ absolute current
+    -- Raises an error if the given file is not the one currently
+    -- loaded.
+    cf <- gets theCurrentFile
+    when (not (independent cmd) && Just current /= (fst <$> cf)) $
+        lift $ typeError $ GenericError "Error: First load the file."
+
+    withCurrentFile $ interpret cmd
+
+    cf <- gets theCurrentFile
+    when (Just current == (fst <$> cf)) $
+        putResponse . Resp_InteractionPoints =<< gets theInteractionPoints
+
+  where
+    inEmacs = liftCommandMT $ withEnv $ initEnv
+            { envHighlightingLevel  = highlighting
+            , envHighlightingMethod = highlightingMethod
+            }
+
+    -- If an independent command fails we should reset theCurrentFile (Issue853).
+    onFail | independent cmd = modify $ \ s -> s { theCurrentFile = Nothing }
+           | otherwise       = return ()
 
 ----------------------------------------------------------------------------
 -- | An interactive computation.
@@ -287,13 +292,13 @@ runInteraction (IOTCM current highlighting highlightingMethod cmd)
 type Interaction = Interaction' Range
 
 data Interaction' range
-    -- | @cmd_load m includes@ loads the module in file @m@, using
-    -- @includes@ as the include directories.
-  = Cmd_load            FilePath [FilePath]
+    -- | @cmd_load m argv@ loads the module in file @m@, using
+    -- @argv@ as the command-line options.
+  = Cmd_load            FilePath [String]
 
-    -- | @cmd_compile b m includes@ compiles the module in file @m@ using
-    -- the backend @b@, using @includes@ as the include directories.
-  | Cmd_compile         Backend FilePath [FilePath]
+    -- | @cmd_compile b m argv@ compiles the module in file @m@ using
+    -- the backend @b@, using @argv@ as the command-line options.
+  | Cmd_compile         Backend FilePath [String]
 
   | Cmd_constraints
 
@@ -307,6 +312,10 @@ data Interaction' range
                         B.Rewrite
                         String
 
+    -- | Shows all the top-level names in scope which mention all the given
+    -- identifiers in their type.
+  | Cmd_search_about_toplevel B.Rewrite String
+
   | Cmd_solveAll
 
     -- | Parse the given expression (as if it were defined at the
@@ -465,13 +474,15 @@ instance Read InteractionId where
     readsPrec = parseToReadsPrec $
         fmap InteractionId readParse
 
+-- | Note that the grammar implemented by this instance does not
+-- necessarily match the current representation of ranges.
+
 instance Read a => Read (Range' a) where
-    readsPrec = parseToReadsPrec $ do
-                exact "Range"
-                fmap Range readParse
-          `mplus` do
-                exact "noRange"
-                return noRange
+    readsPrec = parseToReadsPrec $
+      (exact "intervalsToRange" >>
+       liftM2 intervalsToRange readParse readParse)
+        `mplus`
+      (exact "noRange" >> return noRange)
 
 instance Read a => Read (Interval' a) where
     readsPrec = parseToReadsPrec $ do
@@ -504,16 +515,16 @@ independent _                               = False
 
 interpret :: Interaction -> CommandM ()
 
-interpret (Cmd_load m includes) =
-  cmd_load' m includes True $ \_ -> interpret Cmd_metas
+interpret (Cmd_load m argv) =
+  cmd_load' m argv True $ \_ -> interpret Cmd_metas
 
-interpret (Cmd_compile b file includes) =
-  cmd_load' file includes False $ \(i, mw) -> do
+interpret (Cmd_compile b file argv) =
+  cmd_load' file argv False $ \(i, mw) -> do
     case mw of
       Imp.NoWarnings -> do
         lift $ case b of
-          MAlonzo -> MAlonzo.compilerMain True i
-          MAlonzoNoMain -> MAlonzo.compilerMain False i
+          MAlonzo -> MAlonzo.compilerMain IsMain i
+          MAlonzoNoMain -> MAlonzo.compilerMain NotMain i
           Epic    -> Epic.compilerMain i
           JS      -> JS.compilerMain i
         display_info $ Info_CompilationOk
@@ -536,6 +547,9 @@ interpret Cmd_metas = do -- CL.showMetas []
 interpret (Cmd_show_module_contents_toplevel norm s) =
   liftCommandMT B.atTopLevel $ showModuleContents norm noRange s
 
+interpret (Cmd_search_about_toplevel norm s) =
+  liftCommandMT B.atTopLevel $ searchAbout norm noRange s
+
 interpret Cmd_solveAll = do
   out <- lift $ mapM lowr =<< B.getSolvedInteractionPoints False -- only solve metas which have a proper instantiation, i.e., not another meta
   putResponse $ Resp_SolveAll out
@@ -558,21 +572,21 @@ interpret (Cmd_compute_toplevel ignore s) =
 
 interpret (ShowImplicitArgs showImpl) = do
   opts <- lift commandLineOptions
-  setCommandLineOptions' $
+  setCommandLineOpts $
     opts { optPragmaOptions =
              (optPragmaOptions opts) { optShowImplicit = showImpl } }
 
 interpret ToggleImplicitArgs = do
   opts <- lift commandLineOptions
   let ps = optPragmaOptions opts
-  setCommandLineOptions' $
+  setCommandLineOpts $
     opts { optPragmaOptions =
              ps { optShowImplicit = not $ optShowImplicit ps } }
 
 interpret (Cmd_load_highlighting_info source) = do
     -- Make sure that the include directories have
     -- been set.
-    setCommandLineOptions' =<< lift commandLineOptions
+    setCommandLineOpts =<< lift commandLineOptions
 
     resp <- lift $ liftIO . tellToUpdateHighlighting =<< do
       ex <- liftIO $ doesFileExist source
@@ -625,8 +639,7 @@ interpret (Cmd_intro pmLambda ii rng _) = do
   liftCommandMT (B.withInteractionId ii) $ case ss of
     []    -> do
       display_info $ Info_Intro $ text "No introduction forms found."
-    [s]   -> do
-      interpret $ Cmd_refine ii rng s
+    [s]   -> give_gen ii rng s Intro
     _:_:_ -> do
       display_info $ Info_Intro $
         sep [ text "Don't know which constructor to introduce of"
@@ -737,11 +750,11 @@ interpret (Cmd_make_case ii rng s) = do
     -- Drops pattern added to extended lambda functions when lambda lifting them
     extlam_dropLLifted :: CaseContext -> Bool -> A.Clause -> A.Clause
     extlam_dropLLifted Nothing _ x = x
-    extlam_dropLLifted (Just (ExtLamInfo h nh)) hidden (A.Clause (A.LHS info A.LHSProj{} ps) rhs decl) = __IMPOSSIBLE__
-    extlam_dropLLifted (Just (ExtLamInfo h nh)) hidden (A.Clause (A.LHS info (A.LHSHead name nps) ps) rhs decl)
+    extlam_dropLLifted (Just (ExtLamInfo h nh)) hidden (A.Clause (A.LHS info A.LHSProj{} ps) rhs decl catchall) = __IMPOSSIBLE__
+    extlam_dropLLifted (Just (ExtLamInfo h nh)) hidden (A.Clause (A.LHS info (A.LHSHead name nps) ps) rhs decl catchall)
       = let n = if hidden then h + nh else nh
         in
-         (A.Clause (A.LHS info (A.LHSHead name (drop n nps)) ps) rhs decl)
+         (A.Clause (A.LHS info (A.LHSHead name (drop n nps)) ps) rhs decl catchall)
 
 interpret (Cmd_compute ignore ii rng s) = do
   e <- lift $ B.parseExprIn ii rng s
@@ -777,23 +790,23 @@ showOpenMetas = do
       return $ d ++ "  [ at " ++ show r ++ " ]"
 
 
--- | @cmd_load' file includes unsolvedOk cmd@
+-- | @cmd_load' file argv unsolvedOk cmd@
 --   loads the module in file @file@,
---   using @includes@ as the include directories.
+--   using @argv@ as the command-line options.
 --
 -- If type checking completes without any exceptions having been
 -- encountered then the command @cmd r@ is executed, where @r@ is the
 -- result of 'Imp.typeCheckMain'.
 
-cmd_load' :: FilePath -> [FilePath]
+cmd_load' :: FilePath -> [String]
           -> Bool -- ^ Allow unsolved meta-variables?
           -> ((Interface, Imp.MaybeWarnings) -> CommandM ())
           -> CommandM ()
-cmd_load' file includes unsolvedOK cmd = do
+cmd_load' file argv unsolvedOK cmd = do
     f <- liftIO $ absolute file
     ex <- liftIO $ doesFileExist $ filePath f
-    lift $ TM.setIncludeDirs includes $
-      if ex then ProjectRoot f else CurrentDir
+    let relativeTo | ex        = ProjectRoot f
+                   | otherwise = CurrentDir
 
     -- Forget the previous "current file" and interaction points.
     modify $ \st -> st { theInteractionPoints = []
@@ -803,15 +816,15 @@ cmd_load' file includes unsolvedOK cmd = do
     t <- liftIO $ getModificationTime file
 
     -- All options are reset when a file is reloaded, including the
-    -- choice of whether or not to display implicit arguments. (At
-    -- this point the include directories have already been set, so
-    -- they are preserved.)
-    opts <- lift $ commandLineOptions
-    defaultOptions <- gets optionsOnReload
-    setCommandLineOptions' $
-      Lenses.setIncludeDirs (optIncludeDirs opts) $
-      mapPragmaOptions (\ o -> o { optAllowUnsolved = unsolvedOK }) $
-      defaultOptions
+    -- choice of whether or not to display implicit arguments.
+    opts0 <- gets optionsOnReload
+    z <- liftIO $ runOptM $ parseStandardOptions' argv opts0
+    case z of
+      Left err   -> lift $ typeError $ GenericError err
+      Right opts -> do
+        lift $ TM.setCommandLineOptions' relativeTo $
+          mapPragmaOptions (\ o -> o { optAllowUnsolved = unsolvedOK }) opts
+        displayStatus
 
     -- Reset the state, preserving options and decoded modules. Note
     -- that if the include directories have changed, then the decoded
@@ -825,6 +838,9 @@ cmd_load' file includes unsolvedOK cmd = do
     -- Remove any prior syntax highlighting.
     putResponse Resp_ClearHighlighting
 
+    -- We activate the cache only when agda is used interactively
+    lift activateLoadedFileCache
+
     ok <- lift $ Imp.typeCheckMain f
 
     -- The module type checked. If the file was not changed while the
@@ -852,7 +868,7 @@ data Backend = MAlonzo
              | Epic | JS
     deriving (Show, Read)
 
-data GiveRefine = Give | Refine
+data GiveRefine = Give | Refine | Intro
   deriving (Eq, Show)
 
 -- | A "give"-like action (give, refine, etc).
@@ -879,17 +895,19 @@ give_gen ii rng s0 giveRefine = do
           case giveRefine of
             Give   -> B.give
             Refine -> B.refine
+            Intro  -> B.refine
     -- save scope of the interaction point (for printing the given expr. later)
     scope     <- lift $ getInteractionScope ii
     -- parse string and "give", obtaining an abstract expression
     -- and newly created interaction points
-    (ae, iis) <- lift $ do
-      mis  <- getInteractionPoints
-      reportSLn "interaction.give" 30 $ "interaction points before = " ++ show mis
-      ae   <- give_ref ii Nothing =<< B.parseExprIn ii rng s
-      mis' <- getInteractionPoints
-      reportSLn "interaction.give" 30 $ "interaction points after = " ++ show mis'
-      return (ae, mis' \\ mis)
+    (time, (ae, ae0, iis)) <- maybeTimed $ lift $ do
+        mis  <- getInteractionPoints
+        reportSLn "interaction.give" 30 $ "interaction points before = " ++ show mis
+        given <- B.parseExprIn ii rng s
+        ae    <- give_ref ii Nothing given
+        mis' <- getInteractionPoints
+        reportSLn "interaction.give" 30 $ "interaction points after = " ++ show mis'
+        return (ae, given, mis' \\ mis)
     -- favonia: backup the old scope for highlighting
     insertOldInteractionScope ii scope
     -- sort the new interaction points and put them into the state
@@ -906,7 +924,9 @@ give_gen ii rng s0 giveRefine = do
     -- we still cannot just `give' the user string (which may be empty).
     -- WRONG: also, if no interaction metas were created by @Refine@
     -- WRONG: let literally = (giveRefine == Give || null iis) && rng /= noRange
-    let literally = giveRefine == Give && rng /= noRange
+    -- Ulf, 2015-03-30, if we're doing intro we can't do literal give since
+    -- there is nothing in the hole (issue 1892).
+    let literally = giveRefine /= Intro && ae == ae0 && rng /= noRange
     -- Ulf, 2014-01-24: This works for give since we're highlighting the string
     -- that's already in the buffer. Doing it before the give action means that
     -- the highlighting is moved together with the text when the hole goes away.
@@ -916,8 +936,8 @@ give_gen ii rng s0 giveRefine = do
       highlightExpr ae
     putResponse $ Resp_GiveAction ii $ mkNewTxt literally ce
     lift $ reportSLn "interaction.give" 30 $ "putResponse GiveAction passed"
-    -- display new goal set
-    interpret Cmd_metas
+    -- display new goal set (if not measuring time)
+    maybe (interpret Cmd_metas) (display_info . Info_Time) time
     lift $ reportSLn "interaction.give" 30 $ "interpret Cmd_metas passed"
   where
     -- Substitutes xs for x in ys.
@@ -936,7 +956,7 @@ highlightExpr e =
     generateAndPrintSyntaxInfo decl Full
   where
     dummy = mkName_ (NameId 0 0) "dummy"
-    info  = mkDefInfo (nameConcrete dummy) defaultFixity' PublicAccess ConcreteDef (getRange e)
+    info  = mkDefInfo (nameConcrete dummy) noFixity' PublicAccess ConcreteDef (getRange e)
     decl  = A.Axiom NoFunSig info defaultArgInfo (qnameFromList [dummy]) e
 
 -- | Sorts interaction points based on their ranges.
@@ -1005,6 +1025,22 @@ showModuleContents norm rng s = do
     text "Names" $$
     nest 2 (align 10 types')
 
+-- | Shows all the top-level names in scope which mention all the given
+-- identifiers in their type.
+
+searchAbout :: B.Rewrite -> Range -> String -> CommandM ()
+searchAbout norm rg nm = do
+  let tnm = trim nm
+  unless (null tnm) $ do
+    fancy <- lift $ B.atTopLevel $ do
+       hits <- findMentions norm rg tnm
+       forM hits $ \ (x, t) -> do
+         t <- TCP.prettyTCM t
+         return (show x, text ":" <+> t)
+    display_info $ Info_SearchAbout $
+      text "Definitions about" <+> text (intercalate ", " $ words nm) $$
+      nest 2 (align 10 fancy)
+
 -- | Explain why something is in scope.
 
 whyInScope :: String -> CommandM ()
@@ -1048,6 +1084,7 @@ whyInScope s = do
         pKind ConName        = TCP.text "constructor"
         pKind FldName        = TCP.text "record field"
         pKind PatternSynName = TCP.text "pattern synonym"
+        pKind MacroName      = TCP.text "macro name"
         pKind QuotableName   = TCP.text "quotable name"
 
         pName :: AbstractName -> TCM Doc
@@ -1085,8 +1122,8 @@ whyInScope s = do
 
 -- | Sets the command line options and updates the status information.
 
-setCommandLineOptions' :: CommandLineOptions -> CommandM ()
-setCommandLineOptions' opts = do
+setCommandLineOpts :: CommandLineOptions -> CommandM ()
+setCommandLineOpts opts = do
     lift $ TM.setCommandLineOptions opts
     displayStatus
 
@@ -1132,17 +1169,6 @@ display_info info = do
   displayStatus
   putResponse $ Resp_DisplayInfo info
 
--- UNUSED
--- takenNameStr :: TCM [String]
--- takenNameStr = do
---   xss <- sequence [ List.map (fst . unDom) <$> getContext
---                   , Map.keys <$> asks envLetBindings
---                   , List.map qnameName . HMap.keys . sigDefinitions <$> getSignature
---                ]
---   return $ concat [ parts $ nameConcrete x | x <- concat xss]
---   where
---     parts x = [ s | Id s <- nameParts x ]
-
 refreshStr :: [String] -> String -> ([String], String)
 refreshStr taken s = go nameModifiers where
   go (m:mods) = let s' = s ++ m in
@@ -1184,13 +1210,18 @@ parseAndDoAtToplevel
      -- ^ The expression to parse.
   -> CommandM ()
 parseAndDoAtToplevel cmd title s = do
-  e      <- liftIO $ parse exprParser s
+  e   <- liftIO $ parse exprParser s
+  (time, res) <-
+    maybeTimed (lift $ B.atTopLevel $
+                prettyA =<< cmd =<< concreteToAbstract_ e)
+  display_info (title $ fromMaybe empty time $$ res)
+
+maybeTimed :: CommandM a -> CommandM (Maybe Doc, a)
+maybeTimed work = do
   doTime <- lift $ hasVerbosity "profile.interactive" 10
-  let work = lift (B.atTopLevel $ prettyA =<< cmd =<< concreteToAbstract_ e)
-  res <- if not doTime then work else do
-      (r, time) <- measureTime work
-      return $ text "Time:" <+> pretty time $$ r
-  display_info (title res)
+  if not doTime then (Nothing,) <$> work else do
+    (r, time) <- measureTime work
+    return (Just $ text "Time:" <+> pretty time, r)
 
 -- | Tell to highlight the code using the given highlighting
 -- info (unless it is @Nothing@).
@@ -1206,7 +1237,7 @@ tellToUpdateHighlighting (Just (info, modFile)) =
 tellEmacsToJumpToError :: Range -> [Response]
 tellEmacsToJumpToError r =
   case rStart r of
-    Nothing                                    -> []
-    Just (Pn { srcFile = Nothing })            -> []
-    Just (Pn { srcFile = Just f, posPos = p }) ->
+    Nothing                                           -> []
+    Just (Pn { srcFile = Strict.Nothing })            -> []
+    Just (Pn { srcFile = Strict.Just f, posPos = p }) ->
        [ Resp_JumpToError (filePath f) p ]
