@@ -1,6 +1,6 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP             #-}
 {-# LANGUAGE DoAndIfThenElse #-}
-{-# LANGUAGE TupleSections #-}
+{-# LANGUAGE TupleSections   #-}
 
 module Agda.Interaction.MakeCase where
 
@@ -16,6 +16,7 @@ import qualified Agda.Syntax.Concrete as C
 import qualified Agda.Syntax.Abstract as A
 import qualified Agda.Syntax.Info as A
 import Agda.Syntax.Internal
+import Agda.Syntax.Internal.Pattern
 import Agda.Syntax.Scope.Monad (resolveName, ResolvedName(..))
 import Agda.Syntax.Translation.ConcreteToAbstract
 import Agda.Syntax.Translation.InternalToAbstract
@@ -26,6 +27,7 @@ import Agda.TypeChecking.Pretty
 import Agda.TypeChecking.RecordPatterns
 import Agda.TypeChecking.Reduce
 import Agda.TypeChecking.Substitute
+import Agda.TypeChecking.Substitute.Pattern
 import Agda.TypeChecking.Irrelevance
 import Agda.TypeChecking.Rules.LHS.Implicit
 import Agda.TheTypeChecker
@@ -152,7 +154,8 @@ parseVariables ii rng ss = do
 makeCase :: InteractionId -> Range -> String -> TCM (CaseContext , [A.Clause])
 makeCase hole rng s = withInteractionId hole $ do
   meta <- lookupInteractionId hole
-  (casectxt, f, clause@(Clause{ clauseTel = tel, clausePerm = perm, namedClausePats = ps })) <- findClause meta
+  (casectxt, f, clause@(Clause{ clauseTel = tel, namedClausePats = ps })) <- findClause meta
+  let perm = clausePerm clause
   reportSDoc "interaction.case" 10 $ vcat
     [ text "splitting clause:"
     , nest 2 $ vcat
@@ -188,7 +191,12 @@ makeCase hole rng s = withInteractionId hole $ do
   else do
     -- split on variables
     vars <- parseVariables hole rng vars
-    (casectxt,) <$> do split f vars $ clauseToSplitClause clause
+    cs <- split f vars $ clauseToSplitClause clause
+    reportSDoc "interaction.case" 65 $ vcat
+      [ text "split result:"
+      , nest 2 $ vcat $ map (text . show) cs
+      ]
+    return (casectxt,cs)
   where
 
   failNoCop = typeError $ GenericError $
@@ -209,26 +217,27 @@ makeCase hole rng s = withInteractionId hole $ do
 
   -- Finds the new variable corresponding to an old one, if any.
   newVar :: SplitClause -> Nat -> Maybe Nat
-  newVar c x = case ignoreSharing $ applySubst (scSubst c) (var x) of
+  newVar c x = case ignoreSharing $ applyPatSubst (scSubst c) (var x) of
     Var y [] -> Just y
     _        -> Nothing
 
 
 makeAbsurdClause :: QName -> SplitClause -> TCM A.Clause
-makeAbsurdClause f (SClause tel perm ps _ t) = do
+makeAbsurdClause f (SClause tel ps _ t) = do
   reportSDoc "interaction.case" 10 $ vcat
     [ text "Interaction.MakeCase.makeCase: split clause:"
     , nest 2 $ vcat
       [ text "context =" <+> (prettyTCM =<< getContextTelescope)
       , text "tel =" <+> prettyTCM tel
-      , text "perm =" <+> text (show perm)
       , text "ps =" <+> text (show ps)
       ]
     ]
   withCurrentModule (qnameModule f) $ do
     -- Andreas, 2015-05-29 Issue 635
     -- Contract implicit record patterns before printing.
-    c <- translateRecordPatterns $ Clause noRange tel perm ps NoBody t
+    -- c <- translateRecordPatterns $ Clause noRange tel perm ps NoBody t False
+    -- Jesper, 2015-09-19 Don't contract, since we do on-demand splitting
+    let c = Clause noRange tel ps NoBody t False
     -- Normalise the dot patterns
     ps <- addCtxTel tel $ normalise $ namedClausePats c
     inTopContext $ reify $ QNamed f $ c { namedClausePats = ps }
@@ -236,10 +245,11 @@ makeAbsurdClause f (SClause tel perm ps _ t) = do
 -- | Make a clause with a question mark as rhs.
 makeAbstractClause :: QName -> SplitClause -> TCM A.Clause
 makeAbstractClause f cl = do
-  A.Clause lhs _ _ <- makeAbsurdClause f cl
-  let ii = __IMPOSSIBLE__  -- No interaction point since we never type check this
-  let info = A.emptyMetaInfo -- metaNumber = Nothing in order to print as ?, not ?n
-  return $ A.Clause lhs (A.RHS $ A.QuestionMark info ii) []
+  A.Clause lhs _ _ _ <- makeAbsurdClause f cl
+  let ii = InteractionId (-1)  -- Dummy interaction point since we never type check this.
+                               -- Can end up in verbose output though (#1842), hence not __IMPOSSIBLE__.
+  let info = A.emptyMetaInfo   -- metaNumber = Nothing in order to print as ?, not ?n
+  return $ A.Clause lhs (A.RHS $ A.QuestionMark info ii) [] False
 
 deBruijnIndex :: A.Expr -> TCM Nat
 deBruijnIndex e = do
