@@ -57,6 +57,7 @@ data DisplayInfo
     = Info_CompilationOk
     | Info_Constraints String
     | Info_AllGoals String
+    | Info_Time Doc
     | Info_Error String
         -- ^ When an error message is displayed this constructor should be
         -- used, if appropriate.
@@ -67,6 +68,7 @@ data DisplayInfo
         -- ^ 'Info_Auto' denotes either an error or a success (when 'Resp_GiveAction' is present)
         --   TODO: split these into separate constructors
     | Info_ModuleContents Doc
+    | Info_SearchAbout Doc
     | Info_WhyInScope Doc
     | Info_NormalForm Doc
     | Info_GoalType Doc
