@@ -30,9 +30,11 @@ import qualified Agda.TypeChecking.Monad.Benchmark as Bench
 import Agda.TypeChecking.Errors
 import Agda.TypeChecking.Pretty
 
+import Agda.Compiler.Common (IsMain (..))
 import Agda.Compiler.MAlonzo.Compiler as MAlonzo
 import Agda.Compiler.Epic.Compiler as Epic
 import Agda.Compiler.JS.Compiler as JS
+import Agda.Compiler.UHC.Compiler as UHC
 
 import Agda.Utils.Lens
 import Agda.Utils.Monad
@@ -51,7 +53,7 @@ runAgda :: TCM ()
 runAgda = do
   progName <- liftIO getProgName
   argv     <- liftIO getArgs
-  let opts = parseStandardOptions argv
+  opts     <- liftIO $ runOptM $ parseStandardOptions argv
   case opts of
     Left err -> liftIO $ optionError err
     Right opts -> runAgdaWithOptions generateHTML progName opts
@@ -73,45 +75,47 @@ runAgdaWithOptions generateHTML progName opts
           && not (optGHCiInteraction opts)
                             = liftIO printUsage
       | otherwise           = do
-          setCommandLineOptions opts
           -- Main function.
           -- Bill everything to root of Benchmark trie.
-          Bench.billTo [] $ checkFile
-
-          -- Print benchmarks.
-          Bench.print
-
-          -- Print accumulated statistics.
-          printStatistics 20 Nothing =<< use lensAccumStatistics
+          Bench.billTo [] (checkFile opts) `finally_` do
+            -- Print benchmarks.
+            Bench.print
 
+            -- Print accumulated statistics.
+            printStatistics 20 Nothing =<< use lensAccumStatistics
   where
-    checkFile :: TCM ()
-    checkFile = do
-      i             <- optInteractive     <$> liftTCM commandLineOptions
-      ghci          <- optGHCiInteraction <$> liftTCM commandLineOptions
-      compile       <- optCompile         <$> liftTCM commandLineOptions
-      compileNoMain <- optCompileNoMain   <$> liftTCM commandLineOptions
-      epic          <- optEpicCompile     <$> liftTCM commandLineOptions
-      js            <- optJSCompile       <$> liftTCM commandLineOptions
+    checkFile :: CommandLineOptions -> TCM ()
+    checkFile opts = do
+      let i             = optInteractive     opts
+          ghci          = optGHCiInteraction opts
+          ghc           = optGhcCompile      opts
+          compileNoMain = optCompileNoMain   opts
+          epic          = optEpicCompile     opts
+          js            = optJSCompile       opts
+          uhc           = optUHCCompile      opts
       when i $ liftIO $ putStr splashScreen
       let failIfNoInt (Just i) = return i
           -- The allowed combinations of command-line
           -- options should rule out Nothing here.
           failIfNoInt Nothing  = __IMPOSSIBLE__
 
-          failIfInt x Nothing  = x
-          failIfInt _ (Just _) = __IMPOSSIBLE__
+          failIfInt Nothing  = return ()
+          failIfInt (Just _) = __IMPOSSIBLE__
 
           interaction :: TCM (Maybe Interface) -> TCM ()
           interaction | i             = runIM . interactionLoop
-                      | ghci          = (failIfInt mimicGHCi =<<)
-                      | compile && compileNoMain
-                                      = (MAlonzo.compilerMain False =<<) . (failIfNoInt =<<)
-                      | compile       = (MAlonzo.compilerMain True =<<) . (failIfNoInt =<<)
+                      | ghci          = mimicGHCi . (failIfInt =<<)
+                      | ghc && compileNoMain
+                                      = (MAlonzo.compilerMain NotMain =<<) . (failIfNoInt =<<)
+                      | ghc           = (MAlonzo.compilerMain IsMain =<<) . (failIfNoInt =<<)
                       | epic          = (Epic.compilerMain    =<<) . (failIfNoInt =<<)
                       | js            = (JS.compilerMain      =<<) . (failIfNoInt =<<)
+                      | uhc && compileNoMain
+                                      = (UHC.compilerMain NotMain =<<) . (failIfNoInt =<<)
+                      | uhc           = (UHC.compilerMain IsMain =<<)  . (failIfNoInt =<<)
                       | otherwise     = (() <$)
       interaction $ do
+        setCommandLineOptions opts
         hasFile <- hasInputFile
         -- Andreas, 2013-10-30 The following 'resetState' kills the
         -- verbosity options.  That does not make sense (see fail/Issue641).
@@ -156,7 +160,7 @@ runAgdaWithOptions generateHTML progName opts
 printUsage :: IO ()
 printUsage = do
   progName <- getProgName
-  putStr $ usage standardOptions_ [] progName
+  putStr $ usage standardOptions_ progName
 
 -- | Print version information.
 printVersion :: IO ()
@@ -166,8 +170,7 @@ printVersion =
 -- | What to do for bad options.
 optionError :: String -> IO ()
 optionError err = do
-  putStrLn $ "Error: " ++ err
-  printUsage
+  putStrLn $ "Error: " ++ err ++ "\nRun 'agda --help' for help on command line options."
   exitFailure
 
 -- | Run a TCM action in IO; catch and pretty print errors.
