@@ -17,7 +17,7 @@ module Agda.Syntax.Abstract
     ) where
 
 import Prelude hiding (foldl, foldr)
-import Control.Arrow ((***), first, second)
+import Control.Arrow (first)
 import Control.Applicative
 
 import Data.Foldable as Fold
@@ -27,13 +27,13 @@ import Data.Sequence (Seq, (<|), (><))
 import qualified Data.Sequence as Seq
 import Data.Traversable
 import Data.Typeable (Typeable)
+import Data.Void
 
+import Agda.Syntax.Concrete (FieldAssignment'(..), exprFieldA)
 import qualified Agda.Syntax.Concrete as C
 import Agda.Syntax.Concrete.Pretty ()
 import Agda.Syntax.Info
-import Agda.Syntax.Common hiding (Arg, Dom, NamedArg, ArgInfo)
-import qualified Agda.Syntax.Common as Common
-import Agda.Syntax.Fixity
+import Agda.Syntax.Common
 import Agda.Syntax.Position
 import Agda.Syntax.Abstract.Name
 import Agda.Syntax.Abstract.Name as A (QNamed)
@@ -41,29 +41,12 @@ import Agda.Syntax.Literal
 import Agda.Syntax.Scope.Base
 
 import Agda.Utils.Geniplate
-import Agda.Utils.Tuple
+import Agda.Utils.Lens
 
 #include "undefined.h"
 import Agda.Utils.Impossible
 
-type Color      = Expr
-type Arg a      = Common.Arg Color a
-type Dom a      = Common.Dom Color a
-type NamedArg a = Common.NamedArg Color a
-type ArgInfo    = Common.ArgInfo Color
-type Args       = [NamedArg Expr]
-
-instance Eq Color where
-  Var x == Var y = x == y
-  Def x == Def y = x == y
-  -- TODO guilhem:
-  _ == _         = __IMPOSSIBLE__
-
-instance Ord Color where
-  Var x <= Var y = x <= y
-  Def x <= Def y = x <= y
-  -- TODO guilhem:
-  _ <= _         = __IMPOSSIBLE__
+type Args = [NamedArg Expr]
 
 -- | Expressions after scope checking (operators parsed, names resolved).
 data Expr
@@ -72,6 +55,7 @@ data Expr
   | Proj QName                         -- ^ Projection.
   | Con  AmbiguousQName                -- ^ Constructor.
   | PatternSyn QName                   -- ^ Pattern synonym.
+  | Macro QName                        -- ^ Macro.
   | Lit Literal                        -- ^ Literal.
   | QuestionMark MetaInfo InteractionId
     -- ^ Meta variable for interaction.
@@ -93,20 +77,24 @@ data Expr
   | Prop ExprInfo                      -- ^ @Prop@ (no longer supported, used as dummy type).
   | Let  ExprInfo [LetBinding] Expr    -- ^ @let bs in e@.
   | ETel Telescope                     -- ^ Only used when printing telescopes.
-  | Rec  ExprInfo Assigns              -- ^ Record construction.
+  | Rec  ExprInfo RecordAssigns        -- ^ Record construction.
   | RecUpdate ExprInfo Expr Assigns    -- ^ Record update.
   | ScopedExpr ScopeInfo Expr          -- ^ Scope annotation.
   | QuoteGoal ExprInfo Name Expr       -- ^ Binds @Name@ to current type in @Expr@.
-  | QuoteContext ExprInfo Name Expr    -- ^ Binds @Name@ to current context in @Expr@.
+  | QuoteContext ExprInfo              -- ^ Returns the current context.
   | Quote ExprInfo                     -- ^ Quote an identifier 'QName'.
   | QuoteTerm ExprInfo                 -- ^ Quote a term.
   | Unquote ExprInfo                   -- ^ The splicing construct: unquote ...
+  | Tactic ExprInfo Expr [NamedArg Expr] [NamedArg Expr]
+                                       -- ^ @tactic e x1 .. xn | y1 | .. | yn@
   | DontCare Expr                      -- ^ For printing @DontCare@ from @Syntax.Internal@.
   deriving (Typeable, Show)
 
 -- | Record field assignment @f = e@.
-type Assign  = (C.Name, Expr)
+type Assign  = FieldAssignment' Expr
 type Assigns = [Assign]
+type RecordAssign  = Either Assign ModuleName
+type RecordAssigns = [RecordAssign]
 
 -- | Is a type signature a `postulate' or a function signature?
 data Axiom
@@ -124,10 +112,12 @@ data Declaration
   | Primitive  DefInfo QName Expr                    -- ^ primitive function
   | Mutual     MutualInfo [Declaration]              -- ^ a bunch of mutually recursive definitions
   | Section    ModuleInfo ModuleName [TypedBindings] [Declaration]
-  | Apply      ModuleInfo ModuleName ModuleApplication (Ren QName) (Ren ModuleName)
-  | Import     ModuleInfo ModuleName
+  | Apply      ModuleInfo ModuleName ModuleApplication (Ren QName) (Ren ModuleName) ImportDirective
+    -- ^ The @ImportDirective@ is for highlighting purposes.
+  | Import     ModuleInfo ModuleName ImportDirective
+    -- ^ The @ImportDirective@ is for highlighting purposes.
   | Pragma     Range      Pragma
-  | Open       ModuleInfo ModuleName
+  | Open       ModuleInfo ModuleName ImportDirective
     -- ^ only retained for highlighting purposes
   | FunDef     DefInfo QName Delayed [Clause] -- ^ sequence of function clauses
   | DataSig    DefInfo QName Telescope Expr -- ^ lone data signature
@@ -135,12 +125,13 @@ data Declaration
   | DataDef    DefInfo QName [LamBinding] [Constructor]
       -- ^ the 'LamBinding's are 'DomainFree' and binds the parameters of the datatype.
   | RecSig     DefInfo QName Telescope Expr -- ^ lone record signature
-  | RecDef     DefInfo QName (Maybe (Ranged Induction)) (Maybe QName) [LamBinding] Expr [Declaration]
+  | RecDef     DefInfo QName (Maybe (Ranged Induction)) (Maybe Bool) (Maybe QName) [LamBinding] Expr [Declaration]
       -- ^ The 'Expr' gives the constructor type telescope, @(x1 : A1)..(xn : An) -> Prop@,
       --   and the optional name is the constructor's name.
-  | PatternSynDef QName [Arg Name] Pattern
+  | PatternSynDef QName [Arg Name] (Pattern' Void)
       -- ^ Only for highlighting purposes
-  | UnquoteDecl MutualInfo DefInfo QName Expr
+  | UnquoteDecl MutualInfo [DefInfo] [QName] Expr
+  | UnquoteDef  [DefInfo] [QName] Expr
   | ScopedDecl ScopeInfo [Declaration]  -- ^ scope annotation
   deriving (Typeable, Show)
 
@@ -156,15 +147,19 @@ instance GetDefInfo Declaration where
   getDefInfo (DataSig i _ _ _)      = Just i
   getDefInfo (DataDef i _ _ _)      = Just i
   getDefInfo (RecSig i _ _ _)       = Just i
-  getDefInfo (RecDef i _ _ _ _ _ _) = Just i
+  getDefInfo (RecDef i _ _ _ _ _ _ _) = Just i
   getDefInfo _ = Nothing
 
+type ImportDirective = ImportDirective' QName ModuleName
+type Renaming        = Renaming'        QName ModuleName
+type ImportedName    = ImportedName'    QName ModuleName
+
 data ModuleApplication
     = SectionApp Telescope ModuleName [NamedArg Expr]
       -- ^ @tel. M args@:  applies @M@ to @args@ and abstracts @tel@.
     | RecordModuleIFS ModuleName
       -- ^ @M {{...}}@
-  deriving (Typeable, Show)
+  deriving (Typeable, Show, Eq)
 
 data Pragma
   = OptionsPragma [String]
@@ -175,13 +170,18 @@ data Pragma
   | RewritePragma QName
   | CompiledPragma QName String
   | CompiledExportPragma QName String
+  | CompiledDeclareDataPragma QName String
   | CompiledTypePragma QName String
   | CompiledDataPragma QName String [String]
   | CompiledEpicPragma QName String
   | CompiledJSPragma QName String
+  | CompiledUHCPragma QName String
+  | CompiledDataUHCPragma QName String [String]
+  | NoSmashingPragma QName
   | StaticPragma QName
-  | EtaPragma QName
-  deriving (Typeable, Show)
+  | InlinePragma QName
+  | DisplayPragma QName [NamedArg Pattern] Expr
+  deriving (Typeable, Show, Eq)
 
 -- | Bindings that are valid in a @let@.
 data LetBinding
@@ -189,11 +189,15 @@ data LetBinding
     -- ^ @LetBind info rel name type defn@
   | LetPatBind LetInfo Pattern Expr
     -- ^ Irrefutable pattern binding.
-  | LetApply ModuleInfo ModuleName ModuleApplication (Ren QName) (Ren ModuleName)
-    -- ^ @LetApply mi newM (oldM args) renaming moduleRenaming@.
-  | LetOpen ModuleInfo ModuleName
+  | LetApply ModuleInfo ModuleName ModuleApplication (Ren QName) (Ren ModuleName) ImportDirective
+    -- ^ @LetApply mi newM (oldM args) renaming moduleRenaming dir@.
+    -- The @ImportDirective@ is for highlighting purposes.
+  | LetOpen ModuleInfo ModuleName ImportDirective
     -- ^ only for highlighting and abstractToConcrete
-  deriving (Typeable, Show)
+  | LetDeclaredVariable Name
+    -- ^ Only used for highlighting. Refers to the first occurrence of
+    -- @x@ in @let x : A; x = e@.
+  deriving (Typeable, Show, Eq)
 
 -- | Only 'Axiom's.
 type TypeSignature  = Declaration
@@ -204,12 +208,12 @@ type Field          = TypeSignature
 data LamBinding
   = DomainFree ArgInfo Name   -- ^ . @x@ or @{x}@ or @.x@ or @.{x}@
   | DomainFull TypedBindings  -- ^ . @(xs:e)@ or @{xs:e}@ or @(let Ds)@
-  deriving (Typeable, Show)
+  deriving (Typeable, Show, Eq)
 
 -- | Typed bindings with hiding information.
 data TypedBindings = TypedBindings Range (Arg TypedBinding)
             -- ^ . @(xs : e)@ or @{xs : e}@
-  deriving (Typeable, Show)
+  deriving (Typeable, Show, Eq)
 
 -- | A typed binding.  Appears in dependent function spaces, typed lambdas, and
 --   telescopes.  It might be tempting to simplify this to only bind a single
@@ -230,7 +234,7 @@ data TypedBinding
     -- ^ As in telescope @(x y z : A)@ or type @(x y z : A) -> B@.
   | TLet Range [LetBinding]
     -- ^ E.g. @(let x = e)@ or @(let open M)@.
-  deriving (Typeable, Show)
+  deriving (Typeable, Show, Eq)
 
 type Telescope  = [TypedBindings]
 
@@ -241,7 +245,8 @@ data Clause' lhs = Clause
   { clauseLHS        :: lhs
   , clauseRHS        :: RHS
   , clauseWhereDecls :: [Declaration]
-  } deriving (Typeable, Show, Functor, Foldable, Traversable)
+  , clauseCatchall   :: Bool
+  } deriving (Typeable, Show, Functor, Foldable, Traversable, Eq)
 
 type Clause = Clause' LHS
 type SpineClause = Clause' SpineLHS
@@ -255,7 +260,7 @@ data RHS
       -- ^ The 'QName's are the names of the generated with functions.
       --   One for each 'Expr'.
       --   The RHS shouldn't be another @RewriteRHS@.
-  deriving (Typeable, Show)
+  deriving (Typeable, Show, Eq)
 
 -- | The lhs of a clause in spine view (inside-out).
 --   Projection patterns are contained in @spLhsPats@,
@@ -266,7 +271,11 @@ data SpineLHS = SpineLHS
   , spLhsPats     :: [NamedArg Pattern]  -- ^ Function parameters (patterns).
   , spLhsWithPats :: [Pattern]           -- ^ @with@ patterns (after @|@).
   }
-  deriving (Typeable, Show)
+  deriving (Typeable, Show, Eq)
+
+
+instance Eq LHS where
+  (LHS _ core wps) == (LHS _ core' wps') = core == core' && wps == wps'
 
 -- | The lhs of a clause in focused (projection-application) view (outside-in).
 --   Projection patters are represented as 'LHSProj's.
@@ -295,7 +304,7 @@ data LHSCore' e
              , lhsPatsRight  :: [NamedArg (Pattern' e)]
                -- ^ Further applied to patterns.
              }
-  deriving (Eq, Typeable, Show, Functor, Foldable, Traversable)
+  deriving (Typeable, Show, Functor, Foldable, Traversable, Eq)
 
 type LHSCore = LHSCore' Expr
 
@@ -336,8 +345,8 @@ lhsCoreApp (LHSProj d ps1 h ps2) ps' = LHSProj d ps1 h $ ps2 ++ ps'
 -- | Add projection and applicative patterns to the right.
 lhsCoreAddSpine :: LHSCore' e -> [NamedArg (Pattern' e)] -> LHSCore' e
 lhsCoreAddSpine core ps = case ps2 of
-    (Common.Arg info (Named n (DefP i d ps0)) : ps2') ->
-       LHSProj d ps0 (Common.Arg info $ Named n $ lhsCoreApp core ps1) []
+    (Arg info (Named n (DefP i d ps0)) : ps2') ->
+       LHSProj d ps0 (Arg info $ Named n $ lhsCoreApp core ps1) []
          `lhsCoreAddSpine` ps2'
     [] -> lhsCoreApp core ps
     _ -> __IMPOSSIBLE__
@@ -382,7 +391,8 @@ data Pattern' e
   | AbsurdP PatInfo
   | LitP Literal
   | PatternSynP PatInfo QName [NamedArg (Pattern' e)]
-  deriving (Typeable, Show, Functor, Foldable, Traversable)
+  | RecP PatInfo [FieldAssignment' (Pattern' e)]
+  deriving (Typeable, Show, Functor, Foldable, Traversable, Eq)
 
 type Pattern  = Pattern' Expr
 type Patterns = [NamedArg Pattern]
@@ -395,7 +405,7 @@ instance IsProjP (Pattern' e) where
   isProjP (DefP _ d []) = Just d
   isProjP _             = Nothing
 
-instance IsProjP a => IsProjP (Common.Arg c a) where
+instance IsProjP a => IsProjP (Arg a) where
   isProjP = isProjP . unArg
 
 instance IsProjP a => IsProjP (Named n a) where
@@ -405,26 +415,67 @@ instance IsProjP a => IsProjP (Named n a) where
     Instances
  --------------------------------------------------------------------------}
 
--- | Literal equality of patterns, ignoring dot patterns
-instance Eq (Pattern' e) where
-  p == p' =
-    case (p,p') of
-      ((VarP x)             , (VarP x')             ) -> x === x'
-      ((ConP _ x ps)        , (ConP _ x' ps')       ) -> x == x' && ps == ps'
-      ((DefP _ x ps)        , (DefP _ x' ps')       ) -> x == x' && ps == ps'
-      ((WildP _)            , (WildP _)             ) -> True
-      ((AsP _ x p)          , (AsP _ x' p')         ) -> x === x' && p == p'
-      ((DotP _ _)           , (DotP _ _)            ) -> True
-      (AbsurdP{}            , AbsurdP{}             ) -> True
-      ((LitP l)             , (LitP l')             ) -> l == l'
-      ((PatternSynP _ x ps) , (PatternSynP _ x' ps')) -> x == x' && ps == ps'
-      (_                    , _                     ) -> False
-    where (Name _ (C.Name _ x) _ _) === (Name _ (C.Name _ x') _ _) = True
-          (Name _ C.NoName{}   _ _) === (Name _ C.NoName{}    _ _) = True
-          _                         === _                          = False
+-- | Does not compare 'ScopeInfo' fields.
 
-instance Eq LHS where
-  (LHS _ core wps) == (LHS _ core' wps') = core == core' && wps == wps'
+instance Eq Expr where
+  ScopedExpr _ a1         == ScopedExpr _ a2         = a1 == a2
+
+  Var a1                  == Var a2                  = a1 == a2
+  Def a1                  == Def a2                  = a1 == a2
+  Proj a1                 == Proj a2                 = a1 == a2
+  Con a1                  == Con a2                  = a1 == a2
+  PatternSyn a1           == PatternSyn a2           = a1 == a2
+  Macro a1                == Macro a2                = a1 == a2
+  Lit a1                  == Lit a2                  = a1 == a2
+  QuestionMark a1 b1      == QuestionMark a2 b2      = (a1, b1) == (a2, b2)
+  Underscore a1           == Underscore a2           = a1 == a2
+  App a1 b1 c1            == App a2 b2 c2            = (a1, b1, c1) == (a2, b2, c2)
+  WithApp a1 b1 c1        == WithApp a2 b2 c2        = (a1, b1, c1) == (a2, b2, c2)
+  Lam a1 b1 c1            == Lam a2 b2 c2            = (a1, b1, c1) == (a2, b2, c2)
+  AbsurdLam a1 b1         == AbsurdLam a2 b2         = (a1, b1) == (a2, b2)
+  ExtendedLam a1 b1 c1 d1 == ExtendedLam a2 b2 c2 d2 = (a1, b1, c1, d1) == (a2, b2, c2, d2)
+  Pi a1 b1 c1             == Pi a2 b2 c2             = (a1, b1, c1) == (a2, b2, c2)
+  Fun a1 b1 c1            == Fun a2 b2 c2            = (a1, b1, c1) == (a2, b2, c2)
+  Set a1 b1               == Set a2 b2               = (a1, b1) == (a2, b2)
+  Prop a1                 == Prop a2                 = a1 == a2
+  Let a1 b1 c1            == Let a2 b2 c2            = (a1, b1, c1) == (a2, b2, c2)
+  ETel a1                 == ETel a2                 = a1 == a2
+  Rec a1 b1               == Rec a2 b2               = (a1, b1) == (a2, b2)
+  RecUpdate a1 b1 c1      == RecUpdate a2 b2 c2      = (a1, b1, c1) == (a2, b2, c2)
+  QuoteGoal a1 b1 c1      == QuoteGoal a2 b2 c2      = (a1, b1, c1) == (a2, b2, c2)
+  QuoteContext a1         == QuoteContext a2         = a1 == a2
+  Quote a1                == Quote a2                = a1 == a2
+  QuoteTerm a1            == QuoteTerm a2            = a1 == a2
+  Unquote a1              == Unquote a2              = a1 == a2
+  Tactic a1 b1 c1 d1      == Tactic a2 b2 c2 d2      = (a1, b1, c1, d1) == (a2, b2, c2, d2)
+  DontCare a1             == DontCare a2             = a1 == a2
+
+  _                       == _                       = False
+
+-- | Does not compare 'ScopeInfo' fields.
+
+instance Eq Declaration where
+  ScopedDecl _ a1                == ScopedDecl _ a2                = a1 == a2
+
+  Axiom a1 b1 c1 d1 e1           == Axiom a2 b2 c2 d2 e2           = (a1, b1, c1, d1, e1) == (a2, b2, c2, d2, e2)
+  Field a1 b1 c1                 == Field a2 b2 c2                 = (a1, b1, c1) == (a2, b2, c2)
+  Primitive a1 b1 c1             == Primitive a2 b2 c2             = (a1, b1, c1) == (a2, b2, c2)
+  Mutual a1 b1                   == Mutual a2 b2                   = (a1, b1) == (a2, b2)
+  Section a1 b1 c1 d1            == Section a2 b2 c2 d2            = (a1, b1, c1, d1) == (a2, b2, c2, d2)
+  Apply a1 b1 c1 d1 e1 f1        == Apply a2 b2 c2 d2 e2 f2        = (a1, b1, c1, d1, e1, f1) == (a2, b2, c2, d2, e2, f2)
+  Import a1 b1 c1                == Import a2 b2 c2                = (a1, b1, c1) == (a2, b2, c2)
+  Pragma a1 b1                   == Pragma a2 b2                   = (a1, b1) == (a2, b2)
+  Open a1 b1 c1                  == Open a2 b2 c2                  = (a1, b1, c1) == (a2, b2, c2)
+  FunDef a1 b1 c1 d1             == FunDef a2 b2 c2 d2             = (a1, b1, c1, d1) == (a2, b2, c2, d2)
+  DataSig a1 b1 c1 d1            == DataSig a2 b2 c2 d2            = (a1, b1, c1, d1) == (a2, b2, c2, d2)
+  DataDef a1 b1 c1 d1            == DataDef a2 b2 c2 d2            = (a1, b1, c1, d1) == (a2, b2, c2, d2)
+  RecSig a1 b1 c1 d1             == RecSig a2 b2 c2 d2             = (a1, b1, c1, d1) == (a2, b2, c2, d2)
+  RecDef a1 b1 c1 d1 e1 f1 g1 h1 == RecDef a2 b2 c2 d2 e2 f2 g2 h2 = (a1, b1, c1, d1, e1, f1, g1, h1) == (a2, b2, c2, d2, e2, f2, g2, h2)
+  PatternSynDef a1 b1 c1         == PatternSynDef a2 b2 c2         = (a1, b1, c1) == (a2, b2, c2)
+  UnquoteDecl a1 b1 c1 d1        == UnquoteDecl a2 b2 c2 d2        = (a1, b1, c1, d1) == (a2, b2, c2, d2)
+  UnquoteDef a1 b1 c1            == UnquoteDef a2 b2 c2            = (a1, b1, c1) == (a2, b2, c2)
+
+  _                              == _                              = False
 
 instance Underscore Expr where
   underscore   = Underscore emptyMetaInfo
@@ -474,31 +525,34 @@ instance HasRange Expr where
     getRange (ETel tel)            = getRange tel
     getRange (ScopedExpr _ e)      = getRange e
     getRange (QuoteGoal _ _ e)     = getRange e
-    getRange (QuoteContext _ _ e)  = getRange e
+    getRange (QuoteContext i)      = getRange i
     getRange (Quote i)             = getRange i
     getRange (QuoteTerm i)         = getRange i
     getRange (Unquote i)           = getRange i
+    getRange (Tactic i _ _ _)      = getRange i
     getRange (DontCare{})          = noRange
     getRange (PatternSyn x)        = getRange x
+    getRange (Macro x)             = getRange x
 
 instance HasRange Declaration where
     getRange (Axiom    _ i _ _ _    ) = getRange i
     getRange (Field      i _ _      ) = getRange i
     getRange (Mutual     i _        ) = getRange i
     getRange (Section    i _ _ _    ) = getRange i
-    getRange (Apply      i _ _ _ _  ) = getRange i
-    getRange (Import     i _        ) = getRange i
+    getRange (Apply      i _ _ _ _ _) = getRange i
+    getRange (Import     i _ _      ) = getRange i
     getRange (Primitive  i _ _      ) = getRange i
     getRange (Pragma     i _        ) = getRange i
-    getRange (Open       i _        ) = getRange i
+    getRange (Open       i _ _      ) = getRange i
     getRange (ScopedDecl _ d        ) = getRange d
     getRange (FunDef     i _ _ _    ) = getRange i
     getRange (DataSig    i _ _ _    ) = getRange i
     getRange (DataDef    i _ _ _    ) = getRange i
     getRange (RecSig     i _ _ _    ) = getRange i
-    getRange (RecDef   i _ _ _ _ _ _) = getRange i
+    getRange (RecDef   i _ _ _ _ _ _ _) = getRange i
     getRange (PatternSynDef x _ _   ) = getRange x
     getRange (UnquoteDecl _ i _ _)    = getRange i
+    getRange (UnquoteDef i _ _)       = getRange i
 
 instance HasRange (Pattern' e) where
     getRange (VarP x)            = getRange x
@@ -510,6 +564,7 @@ instance HasRange (Pattern' e) where
     getRange (AbsurdP i)         = getRange i
     getRange (LitP l)            = getRange l
     getRange (PatternSynP i _ _) = getRange i
+    getRange (RecP i _)          = getRange i
 
 instance HasRange SpineLHS where
     getRange (SpineLHS i _ _ _)  = getRange i
@@ -522,7 +577,7 @@ instance HasRange (LHSCore' e) where
     getRange (LHSProj d ps1 lhscore ps2) = d `fuseRange` ps1 `fuseRange` lhscore `fuseRange` ps2
 
 instance HasRange a => HasRange (Clause' a) where
-    getRange (Clause lhs rhs ds) = getRange (lhs,rhs,ds)
+    getRange (Clause lhs rhs ds catchall) = getRange (lhs,rhs,ds)
 
 instance HasRange RHS where
     getRange AbsurdRHS                = noRange
@@ -533,8 +588,9 @@ instance HasRange RHS where
 instance HasRange LetBinding where
     getRange (LetBind  i _ _ _ _     ) = getRange i
     getRange (LetPatBind  i _ _      ) = getRange i
-    getRange (LetApply i _ _ _ _     ) = getRange i
-    getRange (LetOpen  i _           ) = getRange i
+    getRange (LetApply i _ _ _ _ _   ) = getRange i
+    getRange (LetOpen  i _ _         ) = getRange i
+    getRange (LetDeclaredVariable x)   = getRange x
 
 -- setRange for patterns applies the range to the outermost pattern constructor
 instance SetRange (Pattern' a) where
@@ -547,6 +603,7 @@ instance SetRange (Pattern' a) where
     setRange r (AbsurdP _)          = AbsurdP (PatRange r)
     setRange r (LitP l)             = LitP (setRange r l)
     setRange r (PatternSynP _ n as) = PatternSynP (PatRange r) (setRange r n) as
+    setRange r (RecP i as)          = RecP (PatRange r) as
 
 instance KillRange LamBinding where
   killRange (DomainFree info x) = killRange1 (DomainFree info) x
@@ -582,32 +639,36 @@ instance KillRange Expr where
   killRange (ETel tel)             = killRange1 ETel tel
   killRange (ScopedExpr s e)       = killRange1 (ScopedExpr s) e
   killRange (QuoteGoal i x e)      = killRange3 QuoteGoal i x e
-  killRange (QuoteContext i x e)   = killRange3 QuoteContext i x e
+  killRange (QuoteContext i)       = killRange1 QuoteContext i
   killRange (Quote i)              = killRange1 Quote i
   killRange (QuoteTerm i)          = killRange1 QuoteTerm i
   killRange (Unquote i)            = killRange1 Unquote i
+  killRange (Tactic i e xs ys)     = killRange4 Tactic i e xs ys
   killRange (DontCare e)           = killRange1 DontCare e
   killRange (PatternSyn x)         = killRange1 PatternSyn x
+  killRange (Macro x)              = killRange1 Macro x
 
 instance KillRange Declaration where
   killRange (Axiom    p i rel a b     ) = killRange4 (Axiom p)  i rel a b
   killRange (Field      i a b         ) = killRange3 Field      i a b
   killRange (Mutual     i a           ) = killRange2 Mutual     i a
   killRange (Section    i a b c       ) = killRange4 Section    i a b c
-  killRange (Apply      i a b c d     ) = killRange3 Apply      i a b c d
-   -- the last two arguments of Apply are name maps, so nothing to kill
-  killRange (Import     i a           ) = killRange2 Import     i a
+  killRange (Apply      i a b c d e   ) = killRange3 Apply      i a b c d (killRange e)
+   -- the arguments c and d of Apply are name maps, so nothing to kill
+   -- Andreas, 2016-01-24 really?
+  killRange (Import     i a b         ) = killRange3 Import     i a b
   killRange (Primitive  i a b         ) = killRange3 Primitive  i a b
   killRange (Pragma     i a           ) = Pragma (killRange i) a
-  killRange (Open       i x           ) = killRange2 Open       i x
+  killRange (Open       i x dir       ) = killRange3 Open       i x dir
   killRange (ScopedDecl a d           ) = killRange1 (ScopedDecl a) d
   killRange (FunDef  i a b c          ) = killRange4 FunDef  i a b c
   killRange (DataSig i a b c          ) = killRange4 DataSig i a b c
   killRange (DataDef i a b c          ) = killRange4 DataDef i a b c
   killRange (RecSig  i a b c          ) = killRange4 RecSig  i a b c
-  killRange (RecDef  i a b c d e f    ) = killRange7 RecDef  i a b c d e f
+  killRange (RecDef  i a b c d e f g  ) = killRange8 RecDef  i a b c d e f g
   killRange (PatternSynDef x xs p     ) = killRange3 PatternSynDef x xs p
   killRange (UnquoteDecl mi i x e     ) = killRange4 UnquoteDecl mi i x e
+  killRange (UnquoteDef i x e         ) = killRange3 UnquoteDef i x e
 
 instance KillRange ModuleApplication where
   killRange (SectionApp a b c  ) = killRange3 SectionApp a b c
@@ -623,6 +684,7 @@ instance KillRange e => KillRange (Pattern' e) where
   killRange (AbsurdP i)         = killRange1 AbsurdP i
   killRange (LitP l)            = killRange1 LitP l
   killRange (PatternSynP i a p) = killRange3 PatternSynP i a p
+  killRange (RecP i as)         = killRange2 RecP i as
 
 instance KillRange SpineLHS where
   killRange (SpineLHS i a b c)  = killRange4 SpineLHS i a b c
@@ -635,7 +697,7 @@ instance KillRange e => KillRange (LHSCore' e) where
   killRange (LHSProj a b c d) = killRange4 LHSProj a b c d
 
 instance KillRange a => KillRange (Clause' a) where
-  killRange (Clause lhs rhs ds) = killRange3 Clause lhs rhs ds
+  killRange (Clause lhs rhs ds catchall) = killRange4 Clause lhs rhs ds catchall
 
 instance KillRange RHS where
   killRange AbsurdRHS                = AbsurdRHS
@@ -646,8 +708,12 @@ instance KillRange RHS where
 instance KillRange LetBinding where
   killRange (LetBind    i info a b c) = killRange5 LetBind  i info a b c
   killRange (LetPatBind i a b       ) = killRange3 LetPatBind i a b
-  killRange (LetApply   i a b c d   ) = killRange3 LetApply i a b c d
-  killRange (LetOpen    i x         ) = killRange2 LetOpen  i x
+  killRange (LetApply   i a b c d e ) = killRange3 LetApply i a b c d (killRange e)
+  killRange (LetOpen    i x dir     ) = killRange3 LetOpen  i x dir
+  killRange (LetDeclaredVariable x)   = killRange1 LetDeclaredVariable x
+
+-- See Agda.Utils.GeniPlate:
+-- Does not descend into ScopeInfo and renaming maps, for instance.
 
 instanceUniverseBiT' [] [t| (Declaration, QName)          |]
 instanceUniverseBiT' [] [t| (Declaration, AmbiguousQName) |]
@@ -656,6 +722,7 @@ instanceUniverseBiT' [] [t| (Declaration, LetBinding)     |]
 instanceUniverseBiT' [] [t| (Declaration, LamBinding)     |]
 instanceUniverseBiT' [] [t| (Declaration, TypedBinding)   |]
 instanceUniverseBiT' [] [t| (Declaration, Pattern)        |]
+instanceUniverseBiT' [] [t| (Declaration, Pattern' Void)  |]
 instanceUniverseBiT' [] [t| (Declaration, Declaration)    |]
 instanceUniverseBiT' [] [t| (Declaration, ModuleName)     |]
 instanceUniverseBiT' [] [t| (Declaration, ModuleInfo)     |]
@@ -700,9 +767,10 @@ instance AllNames Declaration where
   allNames (DataSig _ q _ _)          = Seq.singleton q
   allNames (DataDef _ q _ decls)      = q <| allNames decls
   allNames (RecSig _ q _ _)           = Seq.singleton q
-  allNames (RecDef _ q _ c _ _ decls) = q <| allNames c >< allNames decls
+  allNames (RecDef _ q _ _ c _ _ decls) = q <| allNames c >< allNames decls
   allNames (PatternSynDef q _ _)      = Seq.singleton q
-  allNames (UnquoteDecl _ _ q _)      = Seq.singleton q
+  allNames (UnquoteDecl _ _ qs _)     = Seq.fromList qs
+  allNames (UnquoteDef _ qs _)        = Seq.fromList qs
   allNames (FunDef _ q _ cls)         = q <| allNames cls
   allNames (Section _ _ _ decls)      = allNames decls
   allNames Apply{}                    = Seq.empty
@@ -712,7 +780,7 @@ instance AllNames Declaration where
   allNames (ScopedDecl _ decls)       = allNames decls
 
 instance AllNames Clause where
-  allNames (Clause _ rhs decls) = allNames rhs >< allNames decls
+  allNames (Clause _ rhs decls _) = allNames rhs >< allNames decls
 
 instance AllNames RHS where
   allNames (RHS e)                   = allNames e
@@ -739,16 +807,18 @@ instance AllNames Expr where
   allNames Prop{}                  = Seq.empty
   allNames (Let _ lbs e)           = allNames lbs >< allNames e
   allNames ETel{}                  = __IMPOSSIBLE__
-  allNames (Rec _ fields)          = allNames $ map snd fields
-  allNames (RecUpdate _ e fs)      = allNames e >< allNames (map snd fs)
+  allNames (Rec _ fields)          = allNames [ a ^. exprFieldA | Left a <- fields ]
+  allNames (RecUpdate _ e fs)      = allNames e >< allNames (map (view exprFieldA) fs)
   allNames (ScopedExpr _ e)        = allNames e
   allNames (QuoteGoal _ _ e)       = allNames e
-  allNames (QuoteContext _ _ e)    = allNames e
+  allNames (QuoteContext _)        = Seq.empty
   allNames Quote{}                 = Seq.empty
   allNames QuoteTerm{}             = Seq.empty
   allNames Unquote{}               = Seq.empty
+  allNames (Tactic _ e xs ys)      = allNames e >< allNames xs >< allNames ys
   allNames DontCare{}              = Seq.empty
-  allNames (PatternSyn x)          = Seq.empty
+  allNames PatternSyn{}            = Seq.empty
+  allNames Macro{}                 = Seq.empty
 
 instance AllNames LamBinding where
   allNames DomainFree{}       = Seq.empty
@@ -762,10 +832,11 @@ instance AllNames TypedBinding where
   allNames (TLet _ lbs)  = allNames lbs
 
 instance AllNames LetBinding where
-  allNames (LetBind _ _ _ e1 e2)  = allNames e1 >< allNames e2
-  allNames (LetPatBind _ _ e)     = allNames e
-  allNames (LetApply _ _ app _ _) = allNames app
-  allNames LetOpen{}              = Seq.empty
+  allNames (LetBind _ _ _ e1 e2)    = allNames e1 >< allNames e2
+  allNames (LetPatBind _ _ e)       = allNames e
+  allNames (LetApply _ _ app _ _ _) = allNames app
+  allNames LetOpen{}                = Seq.empty
+  allNames (LetDeclaredVariable _)  = Seq.empty
 
 instance AllNames ModuleApplication where
   allNames (SectionApp bindss _ es) = allNames bindss >< allNames es
@@ -797,14 +868,30 @@ instance AnyAbstract Declaration where
   anyAbstract (Section _ _ _ ds)     = anyAbstract ds
   anyAbstract (FunDef i _ _ _)       = defAbstract i == AbstractDef
   anyAbstract (DataDef i _ _ _)      = defAbstract i == AbstractDef
-  anyAbstract (RecDef i _ _ _ _ _ _) = defAbstract i == AbstractDef
+  anyAbstract (RecDef i _ _ _ _ _ _ _) = defAbstract i == AbstractDef
   anyAbstract (DataSig i _ _ _)      = defAbstract i == AbstractDef
   anyAbstract (RecSig i _ _ _)       = defAbstract i == AbstractDef
   anyAbstract _                      = __IMPOSSIBLE__
 
+-- | Turn an 'AbstractName' to an expression.
+nameExpr :: AbstractName -> Expr
+nameExpr d = mk (anameKind d) $ anameName d
+  where
+    mk DefName        x = Def x
+    mk FldName        x = Proj x
+    mk ConName        x = Con $ AmbQ [x]
+    mk PatternSynName x = PatternSyn x
+    mk MacroName      x = Macro x
+    mk QuotableName   x = App i (Quote i) (defaultNamedArg $ Def x)
+      where i = ExprRange (getRange x)
+
 app :: Expr -> [NamedArg Expr] -> Expr
 app = foldl (App (ExprRange noRange))
 
+mkLet :: ExprInfo -> [LetBinding] -> Expr -> Expr
+mkLet i [] e = e
+mkLet i ds e = Let i ds e
+
 patternToExpr :: Pattern -> Expr
 patternToExpr (VarP x)            = Var x
 patternToExpr (ConP _ c ps)       =
@@ -817,8 +904,9 @@ patternToExpr (DotP _ e)          = e
 patternToExpr (AbsurdP _)         = Underscore emptyMetaInfo  -- TODO: could this happen?
 patternToExpr (LitP l)            = Lit l
 patternToExpr (PatternSynP _ _ _) = __IMPOSSIBLE__
+patternToExpr (RecP _ as)         = Rec exprNoRange $ map (Left . fmap patternToExpr) as
 
-type PatternSynDefn = ([Arg Name], Pattern)
+type PatternSynDefn = ([Arg Name], Pattern' Void)
 type PatternSynDefns = Map QName PatternSynDefn
 
 lambdaLiftExpr :: [Name] -> Expr -> Expr
@@ -828,14 +916,16 @@ lambdaLiftExpr (n:ns) e = Lam (ExprRange noRange) (DomainFree defaultArgInfo n)
 
 substPattern :: [(Name, Pattern)] -> Pattern -> Pattern
 substPattern s p = case p of
-  VarP z      -> fromMaybe p (lookup z s)
-  ConP i q ps -> ConP i q (fmap (fmap (fmap (substPattern s))) ps)
-  WildP i     -> p
-  DotP i e    -> DotP i (substExpr (map (fmap patternToExpr) s) e)
-  AbsurdP i   -> p
-  LitP l      -> p
-  _           -> __IMPOSSIBLE__ -- pattern synonyms (already gone) and
-                                -- @-patterns (not supported anyways).
+  VarP z        -> fromMaybe p (lookup z s)
+  ConP i q ps   -> ConP i q (map (fmap (fmap (substPattern s))) ps)
+  RecP i ps     -> RecP i (map (fmap (substPattern s)) ps)
+  WildP i       -> p
+  DotP i e      -> DotP i (substExpr (map (fmap patternToExpr) s) e)
+  AbsurdP i     -> p
+  LitP l        -> p
+  DefP{}        -> p              -- destructor pattern
+  AsP{}         -> __IMPOSSIBLE__ -- @-patterns (not supported anyways)
+  PatternSynP{} -> __IMPOSSIBLE__ -- pattern synonyms (already gone)
 
 class SubstExpr a where
   substExpr :: [(Name, Expr)] -> a -> a
@@ -846,15 +936,25 @@ instance SubstExpr a => SubstExpr [a] where
 instance SubstExpr a => SubstExpr (Arg a) where
   substExpr = fmap . substExpr
 
-instance SubstExpr a => SubstExpr (Common.Named name a) where
+instance SubstExpr a => SubstExpr (Named name a) where
   substExpr = fmap . substExpr
 
 instance (SubstExpr a, SubstExpr b) => SubstExpr (a, b) where
   substExpr s (x, y) = (substExpr s x, substExpr s y)
 
+instance (SubstExpr a, SubstExpr b) => SubstExpr (Either a b) where
+  substExpr s (Left x)  = Left (substExpr s x)
+  substExpr s (Right y) = Right (substExpr s y)
+
 instance SubstExpr C.Name where
   substExpr _ = id
 
+instance SubstExpr ModuleName where
+  substExpr _ = id
+
+instance SubstExpr Assign where
+  substExpr s (FieldAssignment n x) = FieldAssignment n (substExpr s x)
+
 instance SubstExpr Expr where
   substExpr s e = case e of
     Var n                 -> fromMaybe e (lookup n s)
@@ -880,12 +980,14 @@ instance SubstExpr Expr where
     -- XXX: Do we need to do more with ScopedExprs?
     ScopedExpr si e       -> ScopedExpr si (substExpr s e)
     QuoteGoal i n e       -> QuoteGoal i n (substExpr s e)
-    QuoteContext i n e    -> QuoteContext i n (substExpr s e)
+    QuoteContext i        -> e
     Quote i               -> e
     QuoteTerm i           -> e
     Unquote i             -> e
+    Tactic i e xs ys      -> Tactic i (substExpr s e) (substExpr s xs) (substExpr s ys)
     DontCare e            -> DontCare (substExpr s e)
-    PatternSyn x          -> e
+    PatternSyn{}          -> e
+    Macro{}               -> e
 
 instance SubstExpr LetBinding where
   substExpr s lb = case lb of
