@@ -21,15 +21,15 @@ import Data.List
 import Data.Traversable as T
 
 import Agda.Syntax.Abstract
-import Agda.Syntax.Abstract.Name as A
-import Agda.Syntax.Common hiding (Arg, Dom, NamedArg)
-import qualified Agda.Syntax.Common as Common
+import Agda.Syntax.Common
 import qualified Agda.Syntax.Concrete.Name as C
+import Agda.Syntax.Concrete (FieldAssignment'(..))
 import Agda.Syntax.Info
 import Agda.Syntax.Position
 import Agda.Syntax.Scope.Monad
 
 import Agda.TypeChecking.Monad.Base (TypeError(..), typeError)
+import Agda.Utils.Either
 import Agda.Utils.Tuple
 
 #include "undefined.h"
@@ -191,10 +191,10 @@ groupClauses (pc@(Path p c) : pcs) = (c, Path p (rhs c) : grp) : groupClauses re
     rhsExpr _       = __IMPOSSIBLE__
 
 clauseToPath :: Clause -> ScopeM (ProjPath Clause)
-clauseToPath (Clause (LHS i lhs wps) (RHS e) []) =
-  fmap (\ lhs -> Clause (LHS i lhs wps) (RHS e) []) <$> lhsToPath [] lhs
-clauseToPath (Clause lhs (RHS e) (_:_)) = typeError $ NotImplemented $ "copattern clauses with where declarations"
-clauseToPath (Clause lhs _ wheredecls) = typeError $ NotImplemented $ "copattern clauses with absurd, with or rewrite right hand side"
+clauseToPath (Clause (LHS i lhs wps) (RHS e) [] catchall) =
+  fmap (\ lhs -> Clause (LHS i lhs wps) (RHS e) [] catchall) <$> lhsToPath [] lhs
+clauseToPath (Clause lhs (RHS e) (_:_) _) = typeError $ NotImplemented $ "copattern clauses with where declarations"
+clauseToPath (Clause lhs _ wheredecls _) = typeError $ NotImplemented $ "copattern clauses with absurd, with or rewrite right hand side"
 
 lhsToPath :: [ProjEntry] -> LHSCore -> ScopeM (ProjPath LHSCore)
 lhsToPath acc lhs@LHSHead{}         = return $ Path acc lhs
@@ -219,14 +219,14 @@ pathToRecord pps =
       Rec ei <$> mapM abstractions pes
 
         where
-          abstractions :: (ProjEntry, Expr) -> ScopeM (C.Name, Expr)
-          abstractions (ProjEntry p xs, e) = (C.unqualify $ qnameToConcrete p,) <$>
+          abstractions :: (ProjEntry, Expr) -> ScopeM RecordAssign
+          abstractions (ProjEntry p xs, e) = Left . FieldAssignment (C.unqualify $ qnameToConcrete p) <$>
             foldr abstract (return e) xs
 
           abstract :: NamedArg Name -> ScopeM Expr -> ScopeM Expr
-          abstract (Common.Arg info (Named Nothing x)) me =
+          abstract (Arg info (Named Nothing x)) me =
             Lam (ExprRange noRange) (DomainFree info x) <$> me
-          abstract (Common.Arg _ (Named Just{} _)) me = typeError $ NotImplemented $
+          abstract (Arg _ (Named Just{} _)) me = typeError $ NotImplemented $
             "named arguments in projection patterns"
 
 -- | Similar to 'groupClauses'.
@@ -266,10 +266,13 @@ class Rename e where
 instance Rename QName where
   rename _ q = q
 
+instance Rename Name where
+  rename rho x = maybe x id (rho x)
+
 instance Rename Expr where
   rename rho e =
     case e of
-      Var x                 -> Var $ maybe x id (rho x)
+      Var x                 -> Var (rename rho x)
       Def f                 -> e
       Proj f                -> e
       Con c                 -> e
@@ -287,24 +290,33 @@ instance Rename Expr where
       Prop{}                -> e
       Let i bs e            -> Let i (rename rho bs) (rename rho e)
       ETel tel              -> ETel (rename rho tel)
-      Rec i fes             -> Rec i $ map (id -*- rename rho) fes
-      RecUpdate i e fes     -> RecUpdate i (rename rho e) $ map (id -*- rename rho) fes
+      Rec i fes             -> Rec i $ rename rho fes
+      RecUpdate i e fes     -> RecUpdate i (rename rho e) (rename rho fes)
       ScopedExpr i e        -> ScopedExpr i (rename rho e)
       QuoteGoal i n e       -> QuoteGoal i n (rename rho e)
-      QuoteContext i n e    -> QuoteContext i n (rename rho e)
+      QuoteContext i        -> e
       Quote i               -> e
       QuoteTerm i           -> e
       Unquote i             -> e
+      Tactic i e xs ys      -> Tactic i (rename rho e) (rename rho xs) (rename rho ys)
       DontCare e            -> DontCare (rename rho e)
-      PatternSyn n          -> e
+      PatternSyn{}          -> e
+      Macro{}               -> e
+
+instance Rename ModuleName where
+  rename rho x = x
+
+instance Rename a => Rename (FieldAssignment' a) where
+  rename rho = fmap (rename rho)
 
 instance Rename LetBinding where
   rename rho e =
       case e of
-        LetBind i r n e e' -> LetBind i r n (rename rho e) (rename rho e')
-        LetPatBind i p e   -> LetPatBind i (rename rho p) (rename rho e)
-        LetApply{}         -> e
-        LetOpen{}          -> e
+        LetBind i r n e e'    -> LetBind i r n (rename rho e) (rename rho e')
+        LetPatBind i p e      -> LetPatBind i (rename rho p) (rename rho e)
+        LetApply{}            -> e
+        LetOpen{}             -> e
+        LetDeclaredVariable x -> LetDeclaredVariable (rename rho x)
 
 instance Rename LamBinding where
   rename rho e =
@@ -320,8 +332,8 @@ instance Rename TypedBinding where
   rename rho (TLet r lbs)   = TLet r (rename rho lbs)
 
 instance Rename Clause where
-  rename rho (Clause lhs rhs wheredecls) =
-    Clause (rename rho lhs) (rename rho rhs) (rename rho wheredecls)
+  rename rho (Clause lhs rhs wheredecls catchall) =
+    Clause (rename rho lhs) (rename rho rhs) (rename rho wheredecls) catchall
 
 instance Rename RHS where
   rename rho e = case e of
@@ -351,11 +363,13 @@ instance Rename a => Rename (Named n a) where
 instance Rename a => Rename [a] where
   rename rho = map (rename rho)
 
+instance (Rename a, Rename b) => Rename (Either a b) where
+  rename rho = mapEither (rename rho) (rename rho)
+
 instance (Rename a, Rename b) => Rename (a, b) where
   rename rho (a,b) = (rename rho a, rename rho b)
 
 
-
 -- | Alpha-Equivalence of patterns, ignoring dot patterns
 class Alpha t where
   alpha :: t -> t -> Maybe NameMap
