@@ -15,7 +15,7 @@ module Agda.Syntax.Abstract.Name
   , IsNoName(..)
   ) where
 
-import Control.Monad.State
+import Control.DeepSeq
 
 import Data.Foldable (Foldable)
 import Data.Traversable (Traversable)
@@ -29,7 +29,7 @@ import Test.QuickCheck
 import Agda.Syntax.Position
 import Agda.Syntax.Common
 import {-# SOURCE #-} Agda.Syntax.Fixity
-import Agda.Syntax.Concrete.Name (IsNoName(..))
+import Agda.Syntax.Concrete.Name (IsNoName(..), NumHoles(..))
 import qualified Agda.Syntax.Concrete.Name as C
 
 import Agda.Utils.List
@@ -44,7 +44,7 @@ import Agda.Utils.Impossible
 -- | A name is a unique identifier and a suggestion for a concrete name. The
 --   concrete name contains the source location (if any) of the name. The
 --   source location of the binding site is also recorded.
-data Name = Name { nameId          :: NameId
+data Name = Name { nameId          :: !NameId
                  , nameConcrete    :: C.Name
                  , nameBindingSite :: Range
                  , nameFixity      :: Fixity'
@@ -137,7 +137,7 @@ class MkName a where
   mkName_ = mkName noRange
 
 instance MkName String where
-  mkName r i s = Name i (C.Name noRange (C.stringNameParts s)) r defaultFixity'
+  mkName r i s = Name i (C.Name noRange (C.stringNameParts s)) r noFixity'
 
 
 qnameToList :: QName -> [Name]
@@ -256,6 +256,12 @@ instance Hashable QName where
 instance IsNoName Name where
   isNoName = isNoName . nameConcrete
 
+instance NumHoles Name where
+  numHoles = numHoles . nameConcrete
+
+instance NumHoles QName where
+  numHoles = numHoles . qnameName
+
 ------------------------------------------------------------------------
 -- * Show instances
 ------------------------------------------------------------------------
@@ -397,3 +403,18 @@ instance Arbitrary QName where
 
 instance CoArbitrary QName where
   coarbitrary = coarbitrary . qnameName
+
+------------------------------------------------------------------------
+-- * NFData instances
+------------------------------------------------------------------------
+
+-- | The range is not forced.
+
+instance NFData Name where
+  rnf (Name _ a _ b) = rnf a `seq` rnf b
+
+instance NFData QName where
+  rnf (QName a b) = rnf a `seq` rnf b
+
+instance NFData ModuleName where
+  rnf (MName a) = rnf a
