@@ -14,9 +14,11 @@ import Data.Traversable
 import Data.Monoid
 import Data.Foldable
 
-import Agda.Syntax.Common hiding (Arg)
+import Agda.Syntax.Common
 import Agda.Syntax.Concrete
 
+import Agda.Utils.Either
+
 #include "undefined.h"
 import Agda.Utils.Impossible
 
@@ -24,11 +26,19 @@ import Agda.Utils.Impossible
 --
 --   Note: does not go into patterns!
 class ExprLike a where
-  mapExpr      :: (Expr -> Expr) -> a -> a
+  mapExpr :: (Expr -> Expr) -> a -> a
   -- ^ This corresponds to 'map'.
-  traverseExpr :: (Monad m, Applicative m) => (Expr -> m Expr) -> a -> m a
+
+  traverseExpr
+#if __GLASGOW_HASKELL__ <= 708
+    :: (Applicative m, Monad m)
+#else
+    :: Monad m
+#endif
+    => (Expr -> m Expr) -> a -> m a
   -- ^ This corresponds to 'mapM'.
-  foldExpr     :: Monoid m => (Expr -> m) -> a -> m
+
+  foldExpr :: Monoid m => (Expr -> m) -> a -> m
   -- ^ This is a reduce.
 
   traverseExpr = __IMPOSSIBLE__  -- TODO: implement!
@@ -42,6 +52,9 @@ instance ExprLike Name where
 instance ExprLike QName where
   mapExpr f = id
 
+instance ExprLike Bool where
+  mapExpr f = id
+
 -- * Instances for functors.
 
 instance ExprLike a => ExprLike (Named name a) where
@@ -64,6 +77,16 @@ instance ExprLike a => ExprLike (Maybe a) where
   traverseExpr = traverse . traverseExpr
   foldExpr     = foldMap  . foldExpr
 
+instance ExprLike a => ExprLike (MaybePlaceholder a) where
+  mapExpr      = fmap     . mapExpr
+  traverseExpr = traverse . traverseExpr
+  foldExpr     = foldMap  . foldExpr
+
+instance (ExprLike a, ExprLike b) => ExprLike (Either a b) where
+  mapExpr f      = mapEither (mapExpr f) (mapExpr f)
+  traverseExpr f = traverseEither (traverseExpr f) (traverseExpr f)
+  foldExpr f     = either (foldExpr f) (foldExpr f)
+
 instance ExprLike a => ExprLike (TypedBinding' a) where
   mapExpr      = fmap     . mapExpr
   traverseExpr = traverse . traverseExpr
@@ -89,6 +112,11 @@ instance (ExprLike a, ExprLike b, ExprLike c) => ExprLike (a, b, c) where
   traverseExpr f (x, y, z) = (,,) <$> traverseExpr f x <*> traverseExpr f y <*> traverseExpr f z
   foldExpr     f (x, y, z) = foldExpr f x `mappend` foldExpr f y `mappend` foldExpr f z
 
+instance (ExprLike a, ExprLike b, ExprLike c, ExprLike d) => ExprLike (a, b, c, d) where
+  mapExpr      f (x, y, z, w) = (mapExpr f x, mapExpr f y, mapExpr f z, mapExpr f w)
+  traverseExpr f (x, y, z, w) = (,,,) <$> traverseExpr f x <*> traverseExpr f y <*> traverseExpr f z <*> traverseExpr f w
+  foldExpr     f (x, y, z, w) = foldExpr f x `mappend` foldExpr f y `mappend` foldExpr f z `mappend` foldExpr f w
+
 -- * Interesting instances
 
 instance ExprLike Expr where
@@ -120,7 +148,7 @@ instance ExprLike Expr where
      Dot r e            -> f $ Dot r                  $ mapE e
      ETel tel           -> f $ ETel                   $ mapE tel
      QuoteGoal r x e    -> f $ QuoteGoal r x          $ mapE e
-     QuoteContext r x e -> f $ QuoteContext r x       $ mapE e
+     QuoteContext r     -> f $ e0
      Tactic r e es      -> f $ Tactic r     (mapE e)  $ mapE es
      Quote{}            -> f $ e0
      QuoteTerm{}        -> f $ e0
@@ -129,6 +157,16 @@ instance ExprLike Expr where
      Equal{}            -> f $ e0
    where mapE e = mapExpr f e
 
+instance ExprLike FieldAssignment where
+  mapExpr      f (FieldAssignment x e) = FieldAssignment x (mapExpr f e)
+  traverseExpr f (FieldAssignment x e) = (\e' -> FieldAssignment x e') <$> traverseExpr f e
+  foldExpr     f (FieldAssignment _ e) = foldExpr f e
+
+instance ExprLike ModuleAssignment where
+  mapExpr      f (ModuleAssignment m es i) = ModuleAssignment m (mapExpr f es) i
+  traverseExpr f (ModuleAssignment m es i) = (\es' -> ModuleAssignment m es' i) <$> traverseExpr f es
+  foldExpr     f (ModuleAssignment m es i) = foldExpr f es
+
 instance ExprLike a => ExprLike (OpApp a) where
   mapExpr f e0 = case e0 of
      SyntaxBindingLambda r bs e -> SyntaxBindingLambda r (mapE bs) $ mapE e
@@ -161,12 +199,12 @@ instance ExprLike ModuleApplication where
 instance ExprLike Declaration where
   mapExpr f e0 = case e0 of
      TypeSig ai x e            -> TypeSig ai x                         $ mapE e
-     Field x e                 -> Field x                              $ mapE e
-     FunClause lhs rhs wh      -> FunClause (mapE lhs) (mapE rhs)      $ mapE wh
+     Field i x e               -> Field i x                            $ mapE e
+     FunClause lhs rhs wh ca   -> FunClause (mapE lhs) (mapE rhs) (mapE wh) (mapE ca)
      DataSig r ind x bs e      -> DataSig r ind x (mapE bs)            $ mapE e
      Data r ind n bs e cs      -> Data r ind n (mapE bs) (mapE e)      $ mapE cs
      RecordSig r ind bs e      -> RecordSig r ind (mapE bs)            $ mapE e
-     Record r n ind c tel e ds -> Record r n ind c (mapE tel) (mapE e) $ mapE ds
+     Record r n ind eta c tel e ds -> Record r n ind eta c (mapE tel) (mapE e) $ mapE ds
      Infix{}                   -> e0
      Syntax{}                  -> e0
      PatternSyn{}              -> e0
@@ -174,6 +212,7 @@ instance ExprLike Declaration where
      Abstract  r ds            -> Abstract  r                          $ mapE ds
      Private   r ds            -> Private   r                          $ mapE ds
      InstanceB r ds            -> InstanceB r                          $ mapE ds
+     Macro     r ds            -> Macro     r                          $ mapE ds
      Postulate r ds            -> Postulate r                          $ mapE ds
      Primitive r ds            -> Primitive r                          $ mapE ds
      Open{}                    -> e0
@@ -181,6 +220,7 @@ instance ExprLike Declaration where
      ModuleMacro r n es op dir -> ModuleMacro r n (mapE es) op dir
      Module r n tel ds         -> Module r n (mapE tel)                $ mapE ds
      UnquoteDecl r x e         -> UnquoteDecl r x (mapE e)
+     UnquoteDef r x e          -> UnquoteDef r x (mapE e)
      Pragma{}                  -> e0
    where mapE e = mapExpr f e
 
