@@ -9,6 +9,10 @@
 {-# LANGUAGE MultiParamTypeClasses      #-}
 {-# LANGUAGE StandaloneDeriving         #-}
 {-# LANGUAGE TemplateHaskell            #-}
+{-# LANGUAGE TupleSections              #-}
+#if __GLASGOW_HASKELL__ <= 708
+{-# LANGUAGE OverlappingInstances #-}
+#endif
 
 module Agda.Syntax.Internal
     ( module Agda.Syntax.Internal
@@ -19,11 +23,8 @@ module Agda.Syntax.Internal
 
 import Prelude hiding (foldr, mapM, null)
 
-import Control.Arrow ((***))
 import Control.Applicative hiding (empty)
 import Control.Monad.Identity hiding (mapM)
-import Control.Monad.State hiding (mapM)
-import Control.Parallel
 
 import Data.Foldable ( Foldable, foldMap )
 import Data.Function
@@ -40,8 +41,7 @@ import Data.Traversable
 import Data.Typeable (Typeable)
 
 import Agda.Syntax.Position
-import Agda.Syntax.Common hiding (Arg, Dom, NamedArg, ArgInfo)
-import qualified Agda.Syntax.Common as Common
+import Agda.Syntax.Common
 import Agda.Syntax.Literal
 import Agda.Syntax.Abstract (IsProjP(..))
 import Agda.Syntax.Abstract.Name
@@ -63,16 +63,11 @@ import Agda.Utils.Permutation
 import Agda.Utils.Pointer
 import Agda.Utils.Size
 import Agda.Utils.Pretty as P
+import Agda.Utils.Tuple
 
 #include "undefined.h"
 import Agda.Utils.Impossible
 
-type Color      = Term
-type ArgInfo    = Common.ArgInfo Color
-type Arg a      = Common.Arg Color a
-type Dom a      = Common.Dom Color a
-type NamedArg a = Common.NamedArg Color a
-
 -- | Type of argument lists.
 --
 type Args       = [Arg Term]
@@ -126,7 +121,6 @@ instance LensConName ConHead where
 --
 data Term = Var {-# UNPACK #-} !Int Elims -- ^ @x es@ neutral
           | Lam ArgInfo (Abs Term)        -- ^ Terms are beta normal. Relevance is ignored
-          | ExtLam [Clause] Args          -- ^ Only used by unquote --> reify. Should never appear elsewhere.
           | Lit Literal
           | Def QName Elims               -- ^ @f es@, possibly a delta/iota-redex
           | Con ConHead Args              -- ^ @c vs@
@@ -199,7 +193,7 @@ instance LensSort (Type' a) where
 
 -- General instance leads to overlapping instances.
 -- instance (Decoration f, LensSort a) => LensSort (f a) where
-instance LensSort a => LensSort (Common.Dom c a) where
+instance LensSort a => LensSort (Dom a) where
   lensSort = traverseF . lensSort
 
 instance LensSort a => LensSort (Abs a) where
@@ -273,6 +267,10 @@ data LevelAtom
     -- ^ Introduced by 'instantiate', removed by 'reduce'.
   deriving (Show, Typeable)
 
+---------------------------------------------------------------------------
+-- * Blocked Terms
+---------------------------------------------------------------------------
+
 -- | Even if we are not stuck on a meta during reduction
 --   we can fail to reduce a definition by pattern matching
 --   for another reason.
@@ -337,10 +335,10 @@ instance Monoid Blocked_ where
   _              `mappend` b@Blocked{}    = b
   NotBlocked x _ `mappend` NotBlocked y _ = NotBlocked (x `mappend` y) ()
 
--- See Issue 1573.
+-- See issues 1573 and 1674.
 #if !MIN_VERSION_transformers(0,4,1)
 instance Error Blocked_ where
-  noMsg = __IMPOSSIBLE__
+  noMsg = mempty
 #endif
 
 -- | When trying to reduce @f es@, on match failed on one
@@ -384,7 +382,7 @@ stuckOn e r =
 -- | A clause is a list of patterns and the clause body should @Bind@.
 --
 --  The telescope contains the types of the pattern variables and the
---  permutation is how to get from the order the variables occur in
+--  de Bruijn indices say how to get from the order the variables occur in
 --  the patterns to the order they occur in the telescope. The body
 --  binds the variables in the order they appear in the patterns.
 --
@@ -398,9 +396,7 @@ data Clause = Clause
     { clauseRange     :: Range
     , clauseTel       :: Telescope
       -- ^ @Δ@: The types of the pattern variables.
-    , clausePerm      :: Permutation
-      -- ^ @π@ with @Γ ⊢ renamingR π : Δ@, which means @Δ ⊢ renaming π : Γ@.
-    , namedClausePats :: [NamedArg Pattern]
+    , namedClausePats :: [NamedArg DeBruijnPattern]
       -- ^ @let Γ = patternVars namedClausePats@
     , clauseBody      :: ClauseBody
       -- ^ @λΓ.v@
@@ -409,10 +405,11 @@ data Clause = Clause
       --   Used, e.g., by @TermCheck@.
       --   Can be 'Irrelevant' if we encountered an irrelevant projection
       --   pattern on the lhs.
+    , clauseCatchall  :: Bool
     }
   deriving (Typeable, Show)
 
-clausePats :: Clause -> [Arg Pattern]
+clausePats :: Clause -> [Arg DeBruijnPattern]
 clausePats = map (fmap namedThing) . namedClausePats
 
 data ClauseBodyF a = Body a
@@ -471,6 +468,9 @@ namedVarP :: PatVarName -> Named (Ranged PatVarName) Pattern
 namedVarP x = Named named $ VarP x
   where named = if isUnderscore x then Nothing else Just $ unranged x
 
+namedDBVarP :: Int -> PatVarName -> Named (Ranged PatVarName) DeBruijnPattern
+namedDBVarP m = (fmap . fmap) (m,) . namedVarP
+
 -- | The @ConPatternInfo@ states whether the constructor belongs to
 --   a record type (@Just@) or data type (@Nothing@).
 --   In the former case, the @Bool@ says whether the record pattern
@@ -497,15 +497,15 @@ noConPatternInfo = ConPatternInfo Nothing Nothing
 
 -- | Extract pattern variables in left-to-right order.
 --   A 'DotP' is also treated as variable (see docu for 'Clause').
-patternVars :: Arg Pattern -> [Arg (Either PatVarName Term)]
-patternVars (Common.Arg i (VarP x)     ) = [Common.Arg i $ Left x]
-patternVars (Common.Arg i (DotP t)     ) = [Common.Arg i $ Right t]
-patternVars (Common.Arg i (ConP _ _ ps)) = List.concat $ map (patternVars . fmap namedThing) ps
-patternVars (Common.Arg i (LitP l)     ) = []
-patternVars (Common.Arg i ProjP{}      ) = []
+patternVars :: Arg (Pattern' a) -> [Arg (Either a Term)]
+patternVars (Arg i (VarP x)     ) = [Arg i $ Left x]
+patternVars (Arg i (DotP t)     ) = [Arg i $ Right t]
+patternVars (Arg i (ConP _ _ ps)) = List.concat $ map (patternVars . fmap namedThing) ps
+patternVars (Arg i (LitP l)     ) = []
+patternVars (Arg i ProjP{}      ) = []
 
 -- | Does the pattern perform a match that could fail?
-properlyMatching :: Pattern -> Bool
+properlyMatching :: Pattern' a -> Bool
 properlyMatching VarP{} = False
 properlyMatching DotP{} = False
 properlyMatching LitP{} = True
@@ -513,7 +513,7 @@ properlyMatching (ConP _ ci ps) = isNothing (conPRecord ci) || -- not a record c
   List.any (properlyMatching . namedArg) ps  -- or one of subpatterns is a proper m
 properlyMatching ProjP{} = True
 
-instance IsProjP Pattern where
+instance IsProjP (Pattern' a) where
   isProjP (ProjP d) = Just d
   isProjP _         = Nothing
 
@@ -523,7 +523,7 @@ instance IsProjP Pattern where
 
 -- | Substitutions.
 
-data Substitution
+data Substitution' a
 
   = IdS
     -- ^ Identity substitution.
@@ -534,7 +534,7 @@ data Substitution
     --   Apply this to closed terms you want to use in a non-empty context.
     --   @Γ ⊢ EmptyS : ()@
 
-  | Term :# Substitution
+  | a :# Substitution' a
     -- ^ Substitution extension, ``cons''.
     --   @
     --     Γ ⊢ u : Aρ   Γ ⊢ ρ : Δ
@@ -542,7 +542,7 @@ data Substitution
     --     Γ ⊢ u :# ρ : Δ, A
     --   @
 
-  | Strengthen Empty Substitution
+  | Strengthen Empty (Substitution' a)
     -- ^ Strengthening substitution.  First argument is @__IMPOSSIBLE__@.
     --   Apply this to a term which does not contain variable 0
     --   to lower all de Bruijn indices by one.
@@ -552,7 +552,7 @@ data Substitution
     --     Γ ⊢ Strengthen ρ : Δ, A
     --   @
 
-  | Wk !Int Substitution
+  | Wk !Int (Substitution' a)
     -- ^ Weakning substitution, lifts to an extended context.
     --   @
     --         Γ ⊢ ρ : Δ
@@ -561,7 +561,7 @@ data Substitution
     --   @
 
 
-  | Lift !Int Substitution
+  | Lift !Int (Substitution' a)
     -- ^ Lifting substitution.  Use this to go under a binder.
     --   @Lift 1 ρ == var 0 :# Wk 1 ρ@.
     --   @
@@ -569,10 +569,36 @@ data Substitution
     --     -------------------------
     --     Γ, Ψρ ⊢ Lift |Ψ| ρ : Δ, Ψ
     --   @
-  deriving (Show)
+
+  deriving (Show, Functor, Foldable, Traversable)
+
+type Substitution = Substitution' Term
+type PatternSubstitution = Substitution' DeBruijnPattern
 
 infixr 4 :#
 
+
+---------------------------------------------------------------------------
+-- * Views
+---------------------------------------------------------------------------
+
+-- | View type as equality type.
+
+data EqualityView
+  = EqualityType
+    { eqtSort  :: Sort     -- ^ Sort of this type.
+    , eqtName  :: QName    -- ^ Builtin EQUALITY.
+    , eqtLevel :: Arg Term -- ^ Hidden
+    , eqtType  :: Arg Term -- ^ Hidden
+    , eqtLhs   :: Arg Term -- ^ NotHidden
+    , eqtRhs   :: Arg Term -- ^ NotHidden
+    }
+  | OtherType Type -- ^ reduced
+
+isEqualityType :: EqualityView -> Bool
+isEqualityType EqualityType{} = True
+isEqualityType OtherType{}    = False
+
 ---------------------------------------------------------------------------
 -- * Absurd Lambda
 ---------------------------------------------------------------------------
@@ -596,33 +622,36 @@ isAbsurdPatternName x = x == absurdPatternName
 -- * Pointers and Sharing
 ---------------------------------------------------------------------------
 
+-- | Remove top-level @Shared@ data constructors.
 ignoreSharing :: Term -> Term
--- ignoreSharing (Shared p) = ignoreSharing $ derefPtr p
+ignoreSharing (Shared p) = ignoreSharing $ derefPtr p
 ignoreSharing v          = v
 
 ignoreSharingType :: Type -> Type
--- ignoreSharingType (El s v) = El s (ignoreSharing v)
-ignoreSharingType v = v
+ignoreSharingType (El s v) = El s (ignoreSharing v)
+-- ignoreSharingType v = v
 
 -- | Introduce sharing.
-shared :: Term -> Term
--- shared v@Shared{}   = v
--- shared v@(Var _ []) = v
--- shared v            = Shared (newPtr v)
-shared v = v
-
-sharedType :: Type -> Type
--- sharedType (El s v) = El s (shared v)
-sharedType v = v
+shared_ :: Term -> Term
+shared_ v@Shared{}   = v
+shared_ v@(Var _ []) = v
+shared_ v@(Con _ []) = v -- Issue 1691: sharing (zero : Nat) destroys constructorForm
+shared_ v            = Shared (newPtr v)
 
 -- | Typically m would be TCM and f would be Blocked.
-updateSharedFM :: (Monad m, Applicative m, Traversable f) => (Term -> m (f Term)) -> Term -> m (f Term)
+updateSharedFM
+#if __GLASGOW_HASKELL__ <= 708
+  :: (Applicative m, Monad m, Traversable f)
+#else
+  :: (Monad m, Traversable f)
+#endif
+  => (Term -> m (f Term)) -> Term -> m (f Term)
 updateSharedFM f v0@(Shared p) = do
   fv <- f (derefPtr p)
   flip traverse fv $ \v ->
     case derefPtr (setPtr v p) of
       Var _ [] -> return v
-      _        -> compressPointerChain v0 `pseq` return v0
+      _        -> return $! compressPointerChain v0
 updateSharedFM f v = f v
 
 updateSharedM :: Monad m => (Term -> m Term) -> Term -> m Term
@@ -630,14 +659,14 @@ updateSharedM f v0@(Shared p) = do
   v <- f (derefPtr p)
   case derefPtr (setPtr v p) of
     Var _ [] -> return v
-    _        -> compressPointerChain v0 `pseq` return v0
+    _        -> return $! compressPointerChain v0
 updateSharedM f v = f v
 
 updateShared :: (Term -> Term) -> Term -> Term
 updateShared f v0@(Shared p) =
   case derefPtr (setPtr (f $ derefPtr p) p) of
     v@(Var _ []) -> v
-    _            -> compressPointerChain v0 `pseq` v0
+    _            -> compressPointerChain v0
 updateShared f v = f v
 
 pointerChain :: Term -> [Ptr Term]
@@ -645,14 +674,16 @@ pointerChain (Shared p) = p : pointerChain (derefPtr p)
 pointerChain _          = []
 
 -- Redirect all top-level pointers to point to the last pointer. So, after
--- compression there are at most two top-level indirections.
+-- compression there are at most two top-level indirections. Then return the
+-- inner-most pointer so we have only one pointer for the result.
 compressPointerChain :: Term -> Term
 compressPointerChain v =
   case reverse $ pointerChain v of
     p:_:ps@(_:_) -> setPointers (Shared p) ps
+    p:_:_        -> (Shared p)
     _            -> v
   where
-    setPointers _ [] = v
+    setPointers u [] = u
     setPointers u (p : ps) =
       setPtr u p `seq` setPointers u ps
 
@@ -717,16 +748,16 @@ instance SgTel (ArgName, Dom Type) where
   sgTel (x, dom) = ExtendTel dom $ Abs x EmptyTel
 
 instance SgTel (Dom (ArgName, Type)) where
-  sgTel (Common.Dom ai (x, t)) = ExtendTel (Common.Dom ai t) $ Abs x EmptyTel
+  sgTel (Dom ai (x, t)) = ExtendTel (Dom ai t) $ Abs x EmptyTel
 
 instance SgTel (Dom Type) where
   sgTel dom = sgTel (stringToArgName "_", dom)
 
 hackReifyToMeta :: Term
-hackReifyToMeta = DontCare $ Lit $ LitInt noRange (-42)
+hackReifyToMeta = DontCare $ Lit $ LitNat noRange (-42)
 
 isHackReifyToMeta :: Term -> Bool
-isHackReifyToMeta (DontCare (Lit (LitInt r (-42)))) = r == noRange
+isHackReifyToMeta (DontCare (Lit (LitNat r (-42)))) = r == noRange
 isHackReifyToMeta _ = False
 
 ---------------------------------------------------------------------------
@@ -809,12 +840,15 @@ hasElims v =
     MetaV x es -> Just (MetaV x, es)
     Con{}      -> Nothing
     Lit{}      -> Nothing
+    -- Andreas, 2016-04-13, Issue 1932: We convert λ x → x .f  into f
+    Lam _ (Abs _ v)  -> case ignoreSharing v of
+      Var 0 [Proj f] -> Just (Def f, [])
+      _              -> Nothing
     Lam{}      -> Nothing
     Pi{}       -> Nothing
     Sort{}     -> Nothing
     Level{}    -> Nothing
     DontCare{} -> Nothing
-    ExtLam{}   -> Nothing
     Shared{}   -> __IMPOSSIBLE__
 
 {- PROBABLY USELESS
@@ -840,7 +874,7 @@ allApplyElims = mapM isApplyElim
 
 -- | Split at first non-'Apply'
 splitApplyElims :: Elims -> (Args, Elims)
-splitApplyElims (Apply u : es) = (u :) *** id $ splitApplyElims es
+splitApplyElims (Apply u : es) = mapFst (u :) $ splitApplyElims es
 splitApplyElims es             = ([], es)
 
 class IsProjElim e where
@@ -858,17 +892,21 @@ dropProjElims = filter (isNothing . isProjElim)
 argsFromElims :: Elims -> Args
 argsFromElims = map argFromElim . dropProjElims
 
+-- | Drop 'Proj' constructors. (Safe)
+allProjElims :: Elims -> Maybe [QName]
+allProjElims = mapM isProjElim
+
 {- NOTE: Elim' already contains Arg.
 
 -- | Commute functors 'Arg' and 'Elim\''.
-swapArgElim :: Common.Arg c (Elim' a) -> Elim' (Common.Arg c a)
+swapArgElim :: Arg (Elim' a) -> Elim' (Arg a)
 
-swapArgElim (Common.Arg ai (Apply a)) = Apply (Common.Arg ai a)
-swapArgElim (Common.Arg ai (Proj  d)) = Proj  d
+swapArgElim (Arg ai (Apply a)) = Apply (Arg ai a)
+swapArgElim (Arg ai (Proj  d)) = Proj  d
 
 -- IMPOSSIBLE TO DEFINE
-swapElimArg :: Elim' (Common.Arg c a) -> Common.Arg c (Elim' a)
-swapElimArg (Apply (Common.Arg ai a)) = Common.Arg ai (Apply a)
+swapElimArg :: Elim' (Arg a) -> Arg (Elim' a)
+swapElimArg (Apply (Arg ai a)) = Arg ai (Apply a)
 swapElimArg (Proj  d) = defaultArg (Proj  d)
 -}
 
@@ -889,8 +927,8 @@ instance Null ClauseBody where
 -- | A 'null' clause is one with no patterns and no rhs.
 --   Should not exist in practice.
 instance Null Clause where
-  empty = Clause empty empty empty empty empty empty
-  null (Clause r tel perm pats body t)
+  empty = Clause empty empty empty empty empty False
+  null (Clause r tel pats body t catchall)
     =  null tel
     && null pats
     && null body
@@ -939,7 +977,11 @@ class TermSize a where
 
   tsize :: a -> Sum Int
 
+#if __GLASGOW_HASKELL__ >= 710
+instance {-# OVERLAPPABLE #-} (Foldable t, TermSize a) => TermSize (t a) where
+#else
 instance (Foldable t, TermSize a) => TermSize (t a) where
+#endif
   tsize = foldMap tsize
 
 instance TermSize Term where
@@ -955,7 +997,6 @@ instance TermSize Term where
     Sort s      -> tsize s
     DontCare mv -> tsize mv
     Shared p    -> tsize (derefPtr p)
-    ExtLam{}    -> __IMPOSSIBLE__
 
 instance TermSize Sort where
   tsize s = case s of
@@ -978,7 +1019,7 @@ instance TermSize LevelAtom where
   tsize (NeutralLevel _ v) = tsize v
   tsize (UnreducedLevel v) = tsize v
 
-instance TermSize Substitution where
+instance TermSize a => TermSize (Substitution' a) where
   tsize IdS                = 1
   tsize EmptyS             = 1
   tsize (Wk _ rho)         = 1 + tsize rho
@@ -1006,7 +1047,6 @@ instance KillRange Term where
     Sort s      -> killRange1 Sort s
     DontCare mv -> killRange1 DontCare mv
     Shared p    -> Shared $ updatePtr killRange p
-    ExtLam c vs -> killRange2 ExtLam c vs
 
 instance KillRange Level where
   killRange (Max as) = killRange1 Max as
@@ -1043,17 +1083,17 @@ instance KillRange Substitution where
 instance KillRange ConPatternInfo where
   killRange (ConPatternInfo mr mt) = killRange1 (ConPatternInfo mr) mt
 
-instance KillRange Pattern where
+instance KillRange a => KillRange (Pattern' a) where
   killRange p =
     case p of
-      VarP{}           -> p
+      VarP x           -> killRange1 VarP x
       DotP v           -> killRange1 DotP v
       ConP con info ps -> killRange3 ConP con info ps
       LitP l           -> killRange1 LitP l
       ProjP q          -> killRange1 ProjP q
 
 instance KillRange Clause where
-  killRange (Clause r tel perm ps body t) = killRange6 Clause r tel perm ps body t
+  killRange (Clause r tel ps body t catchall) = killRange6 Clause r tel ps body t catchall
 
 instance KillRange a => KillRange (ClauseBodyF a) where
   killRange = fmap killRange
@@ -1119,7 +1159,6 @@ instance Pretty Term where
       MetaV x els -> pretty x `pApp` els
       DontCare v  -> pretty v
       Shared{}    -> __IMPOSSIBLE__
-      ExtLam{}    -> __IMPOSSIBLE__
     where
       pApp d els = mparens (not (null els) && p > 9) $
                    d <+> fsep (map (prettyPrec 10) els)
@@ -1175,14 +1214,6 @@ instance Pretty Elim where
   prettyPrec p (Apply v) = prettyPrec p v
   prettyPrec _ (Proj x)  = text ("." ++ show x)
 
-instance Pretty a => Pretty (Arg a) where
-  prettyPrec p a =
-    ($ unArg a) $
-    case getHiding a of
-      NotHidden -> prettyPrec p
-      Hidden    -> braces . pretty
-      Instance  -> braces . braces . pretty
-
 instance Pretty a => Pretty (Pattern' a) where
   prettyPrec n (VarP x)      = prettyPrec n x
   prettyPrec _ (DotP t)      = text "." P.<> prettyPrec 10 t
