@@ -7,15 +7,19 @@ import Control.Applicative
 import Data.Traversable
 import Data.Monoid
 import Data.Foldable
+import Agda.Syntax.Common
 import Agda.Syntax.Internal
 
-import Agda.Utils.Impossible
-#include "undefined.h"
-
 class TermLike a where
-  traverseTerm  :: (Term -> Term) -> a -> a
-  traverseTermM :: (Monad m, Applicative m) => (Term -> m Term) -> a -> m a
-  foldTerm      :: Monoid m => (Term -> m) -> a -> m
+  traverseTerm :: (Term -> Term) -> a -> a
+  traverseTermM
+#if __GLASGOW_HASKELL__ <= 708
+    :: (Applicative m, Monad m)
+#else
+    :: Monad m
+#endif
+    => (Term -> m Term) -> a -> m a
+  foldTerm :: Monoid m => (Term -> m) -> a -> m
 
 -- * Constants
 
@@ -111,7 +115,6 @@ instance TermLike Term where
     Sort _      -> f t
     DontCare mv -> f $ DontCare $ traverseTerm f mv
     Shared p    -> f $ Shared $ traverseTerm f p
-    ExtLam{}    -> __IMPOSSIBLE__
 
   traverseTermM f t = case t of
     Var i xs    -> f =<< Var i <$> traverseTermM f xs
@@ -125,7 +128,6 @@ instance TermLike Term where
     Sort _      -> f t
     DontCare mv -> f =<< DontCare <$> traverseTermM f mv
     Shared p    -> f =<< Shared <$> traverseTermM f p
-    ExtLam{}    -> __IMPOSSIBLE__
 
   foldTerm f t = f t `mappend` case t of
     Var i xs    -> foldTerm f xs
@@ -139,7 +141,6 @@ instance TermLike Term where
     Sort _      -> mempty
     DontCare mv -> foldTerm f mv
     Shared p    -> foldTerm f p
-    ExtLam{}    -> __IMPOSSIBLE__
 
 instance TermLike Level where
   traverseTerm f  (Max as) = Max $ traverseTerm f as
@@ -178,6 +179,36 @@ instance TermLike Type where
   traverseTermM f (El s t) = El s <$> traverseTermM f t
   foldTerm f (El s t) = foldTerm f t
 
+instance TermLike EqualityView where
+
+  traverseTerm f v = case v of
+    OtherType t -> OtherType
+      (traverseTerm f t)
+    EqualityType s eq l t a b -> EqualityType s eq
+      (traverseTerm f l)
+      (traverseTerm f t)
+      (traverseTerm f a)
+      (traverseTerm f b)
+
+  traverseTermM f v = case v of
+    OtherType t -> OtherType
+      <$> traverseTermM f t
+    EqualityType s eq l t a b -> EqualityType s eq
+      <$> traverseTermM f l
+      <*> traverseTermM f t
+      <*> traverseTermM f a
+      <*> traverseTermM f b
+
+  foldTerm f v = case v of
+    OtherType t -> foldTerm f t
+    EqualityType s eq l t a b -> foldTerm f [l, t, a, b]
+
 -- | Put it in a monad to make it possible to do strictly.
-copyTerm :: (TermLike a, Applicative m, Monad m) => a -> m a
+copyTerm
+#if __GLASGOW_HASKELL__ <= 708
+  :: (TermLike a, Applicative m, Monad m)
+#else
+  :: (TermLike a, Monad m)
+#endif
+  => a -> m a
 copyTerm = traverseTermM return
