@@ -1,5 +1,5 @@
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
-{-# LANGUAGE Rank2Types #-}
+{-# LANGUAGE Rank2Types                 #-}
 
 {-| When lexing by hands (for instance string literals) we need to do some
     looking ahead. The 'LookAhead' monad keeps track of the position we are
@@ -47,7 +47,7 @@ newtype ErrorFunction =
  --------------------------------------------------------------------------}
 
 instance Monad LookAhead where
-    return  = LookAhead . return
+    return  = pure
     m >>= k = LookAhead $ unLookAhead m >>= unLookAhead . k
     fail s  =
         do  err <- LookAhead ask
