@@ -1,22 +1,27 @@
-{-# LANGUAGE CPP #-}
-{-# LANGUAGE DeriveDataTypeable #-}
-{-# LANGUAGE GADTs #-}
+{-# LANGUAGE CPP                 #-}
+{-# LANGUAGE DeriveDataTypeable  #-}
+{-# LANGUAGE GADTs               #-}
+{-# LANGUAGE LambdaCase          #-}
+{-# LANGUAGE RankNTypes          #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE RankNTypes #-}
-{-# LANGUAGE TupleSections #-}
+{-# LANGUAGE TupleSections       #-}
 
 {-| This module defines the notion of a scope and operations on scopes.
 -}
 module Agda.Syntax.Scope.Base where
 
-import Control.Arrow ((***), first, second)
+import Control.Arrow (first, second, (***))
 import Control.Applicative
 import Control.DeepSeq
+import Control.Monad
 
+import Data.Either (partitionEithers)
 import Data.Function
 import Data.List as List
 import Data.Map (Map)
 import qualified Data.Map as Map
+import Data.Set (Set)
+import qualified Data.Set as Set
 import Data.Maybe
 import Data.Typeable (Typeable)
 
@@ -29,14 +34,14 @@ import Agda.Syntax.Common
 import Agda.Syntax.Fixity
 import Agda.Syntax.Abstract.Name as A
 import Agda.Syntax.Concrete.Name as C
-import Agda.Syntax.Concrete
-  (ImportDirective(..), UsingOrHiding(..), ImportedName(..), Renaming(..))
+import qualified Agda.Syntax.Concrete as C
 
 import Agda.Utils.AssocList (AssocList)
 import qualified Agda.Utils.AssocList as AssocList
 import Agda.Utils.Functor
 import Agda.Utils.Lens
 import Agda.Utils.List
+import Agda.Utils.Pretty
 import qualified Agda.Utils.Map as Map
 
 #include "undefined.h"
@@ -53,7 +58,7 @@ data Scope = Scope
       , scopeImports        :: Map C.QName A.ModuleName
       , scopeDatatypeModule :: Bool
       }
-  deriving (Typeable)
+  deriving (Typeable, Eq)
 
 -- | See 'Agda.Syntax.Common.Access'.
 data NameSpaceId
@@ -93,12 +98,19 @@ updateScopeNameSpacesM f s = for (f $ scopeNameSpaces s) $ \ x ->
 -- | The complete information about the scope at a particular program point
 --   includes the scope stack, the local variables, and the context precedence.
 data ScopeInfo = ScopeInfo
-      { scopeCurrent    :: A.ModuleName
-      , scopeModules    :: Map A.ModuleName Scope
-      , scopeLocals     :: LocalVars
-      , scopePrecedence :: Precedence
+      { scopeCurrent       :: A.ModuleName
+      , scopeModules       :: Map A.ModuleName Scope
+      , scopeLocals        :: LocalVars
+      , scopePrecedence    :: Precedence
+      , scopeInverseName   :: Map A.QName [C.QName]
+      , scopeInverseModule :: Map A.ModuleName [C.QName]
+      , scopeInScope       :: InScopeSet
       }
-  deriving (Typeable)
+  deriving Typeable
+
+instance Eq ScopeInfo where
+  ScopeInfo c1 m1 l1 p1 _ _ _ == ScopeInfo c2 m2 l2 p2 _ _ _ =
+    c1 == c2 && m1 == m2 && l1 == l2 && p1 == p2
 
 -- | Local variables.
 type LocalVars = AssocList C.Name LocalVar
@@ -114,8 +126,6 @@ data LocalVar
     --   (List not empty).
   deriving (Typeable)
 
-instance NFData LocalVar where rnf x = seq x ()
-
 instance Eq LocalVar where
   (==) = (==) `on` localVar
 
@@ -138,7 +148,7 @@ notShadowedLocal :: LocalVar -> Maybe A.Name
 notShadowedLocal (LocalVar x) = Just x
 notShadowedLocal ShadowedVar{} = Nothing
 
--- | Get all locals that are not shadowed.
+-- | Get all locals that are not shadowed __by imports__.
 notShadowedLocals :: LocalVars -> AssocList C.Name A.Name
 notShadowedLocals = mapMaybe $ \ (c,x) -> (c,) <$> notShadowedLocal x
 
@@ -149,6 +159,9 @@ updateScopeLocals f sc = sc { scopeLocals = f (scopeLocals sc) }
 setScopeLocals :: LocalVars -> ScopeInfo -> ScopeInfo
 setScopeLocals vars = updateScopeLocals (const vars)
 
+mapScopeInfo :: (Scope -> Scope) -> ScopeInfo -> ScopeInfo
+mapScopeInfo f i = i{ scopeModules = f <$> scopeModules i }
+
 ------------------------------------------------------------------------
 -- * Name spaces
 --
@@ -163,12 +176,14 @@ data NameSpace = NameSpace
         -- ^ Maps concrete names to a list of abstract names.
       , nsModules :: ModulesInScope
         -- ^ Maps concrete module names to a list of abstract module names.
+      , nsInScope :: InScopeSet
       }
-  deriving (Typeable)
+  deriving (Typeable, Eq)
 
 type ThingsInScope a = Map C.Name [a]
 type NamesInScope    = ThingsInScope AbstractName
 type ModulesInScope  = ThingsInScope AbstractModule
+type InScopeSet      = Set A.QName
 
 -- | Set of types consisting of exactly 'AbstractName' and 'AbstractModule'.
 --
@@ -209,7 +224,8 @@ data KindOfName
   | FldName        -- ^ Record field name.
   | DefName        -- ^ Ordinary defined name.
   | PatternSynName -- ^ Name of a pattern synonym.
-  | QuotableName   -- ^ A name that can only quoted.
+  | MacroName      -- ^ Name of a macro
+  | QuotableName   -- ^ A name that can only be quoted.
   deriving (Eq, Show, Typeable, Enum, Bounded)
 
 -- | A list containing all name kinds.
@@ -280,35 +296,40 @@ mergeNames = Map.unionWith union
 
 -- | The empty name space.
 emptyNameSpace :: NameSpace
-emptyNameSpace = NameSpace Map.empty Map.empty
+emptyNameSpace = NameSpace Map.empty Map.empty Set.empty
 
 
 -- | Map functions over the names and modules in a name space.
 mapNameSpace :: (NamesInScope   -> NamesInScope  ) ->
                 (ModulesInScope -> ModulesInScope) ->
+                (InScopeSet     -> InScopeSet    ) ->
                 NameSpace -> NameSpace
-mapNameSpace fd fm ns =
-  ns { nsNames   = fd $ nsNames ns
-     , nsModules = fm $ nsModules  ns
+mapNameSpace fd fm fs ns =
+  ns { nsNames   = fd $ nsNames   ns
+     , nsModules = fm $ nsModules ns
+     , nsInScope = fs $ nsInScope ns
      }
 
 -- | Zip together two name spaces.
 zipNameSpace :: (NamesInScope   -> NamesInScope   -> NamesInScope  ) ->
                 (ModulesInScope -> ModulesInScope -> ModulesInScope) ->
+                (InScopeSet     -> InScopeSet     -> InScopeSet    ) ->
                 NameSpace -> NameSpace -> NameSpace
-zipNameSpace fd fm ns1 ns2 =
+zipNameSpace fd fm fs ns1 ns2 =
   ns1 { nsNames   = nsNames   ns1 `fd` nsNames   ns2
       , nsModules = nsModules ns1 `fm` nsModules ns2
+      , nsInScope = nsInScope ns1 `fs` nsInScope ns2
       }
 
 -- | Map monadic function over a namespace.
 mapNameSpaceM :: Applicative m =>
   (NamesInScope   -> m NamesInScope  ) ->
   (ModulesInScope -> m ModulesInScope) ->
+  (InScopeSet     -> m InScopeSet    ) ->
   NameSpace -> m NameSpace
-mapNameSpaceM fd fm ns = update ns <$> fd (nsNames ns) <*> fm (nsModules ns)
+mapNameSpaceM fd fm fs ns = update ns <$> fd (nsNames ns) <*> fm (nsModules ns) <*> fs (nsInScope ns)
   where
-    update ns ds ms = ns { nsNames = ds, nsModules = ms }
+    update ns ds ms is = ns { nsNames = ds, nsModules = ms, nsInScope = is }
 
 ------------------------------------------------------------------------
 -- * General operations on scopes
@@ -327,49 +348,66 @@ emptyScope = Scope
 -- | The empty scope info.
 emptyScopeInfo :: ScopeInfo
 emptyScopeInfo = ScopeInfo
-  { scopeCurrent    = noModuleName
-  , scopeModules    = Map.singleton noModuleName emptyScope
-  , scopeLocals     = []
-  , scopePrecedence = TopCtx
+  { scopeCurrent       = noModuleName
+  , scopeModules       = Map.singleton noModuleName emptyScope
+  , scopeLocals        = []
+  , scopePrecedence    = TopCtx
+  , scopeInverseName   = Map.empty
+  , scopeInverseModule = Map.empty
+  , scopeInScope       = Set.empty
   }
 
 -- | Map functions over the names and modules in a scope.
 mapScope :: (NameSpaceId -> NamesInScope   -> NamesInScope  ) ->
             (NameSpaceId -> ModulesInScope -> ModulesInScope) ->
+            (NameSpaceId -> InScopeSet    -> InScopeSet     ) ->
             Scope -> Scope
-mapScope fd fm = updateScopeNameSpaces $ AssocList.mapWithKey mapNS
+mapScope fd fm fs = updateScopeNameSpaces $ AssocList.mapWithKey mapNS
   where
-    mapNS acc = mapNameSpace (fd acc) (fm acc)
+    mapNS acc = mapNameSpace (fd acc) (fm acc) (fs acc)
 
 -- | Same as 'mapScope' but applies the same function to all name spaces.
 mapScope_ :: (NamesInScope   -> NamesInScope  ) ->
              (ModulesInScope -> ModulesInScope) ->
+             (InScopeSet     -> InScopeSet    ) ->
              Scope -> Scope
-mapScope_ fd fm = mapScope (const fd) (const fm)
+mapScope_ fd fm fs = mapScope (const fd) (const fm) (const fs)
+
+-- | Same as 'mapScope' but applies the function only on the given name space.
+mapScope' :: NameSpaceId -> (NamesInScope   -> NamesInScope  ) ->
+                            (ModulesInScope -> ModulesInScope) ->
+                            (InScopeSet    -> InScopeSet     ) ->
+                            Scope -> Scope
+mapScope' i fd fm fs = mapScope (\ j -> if i == j then fd else id)
+                                (\ j -> if i == j then fm else id)
+                                (\ j -> if i == j then fs else id)
 
 -- | Map monadic functions over the names and modules in a scope.
-mapScopeM :: (Functor m, Applicative m) =>
+mapScopeM :: Applicative m =>
   (NameSpaceId -> NamesInScope   -> m NamesInScope  ) ->
   (NameSpaceId -> ModulesInScope -> m ModulesInScope) ->
+  (NameSpaceId -> InScopeSet     -> m InScopeSet    ) ->
   Scope -> m Scope
-mapScopeM fd fm = updateScopeNameSpacesM $ AssocList.mapWithKeyM mapNS
+mapScopeM fd fm fs = updateScopeNameSpacesM $ AssocList.mapWithKeyM mapNS
   where
-    mapNS acc = mapNameSpaceM (fd acc) (fm acc)
+    mapNS acc = mapNameSpaceM (fd acc) (fm acc) (fs acc)
 
 -- | Same as 'mapScopeM' but applies the same function to both the public and
 --   private name spaces.
-mapScopeM_ :: (Functor m, Applicative m) =>
+mapScopeM_ :: Applicative m =>
   (NamesInScope   -> m NamesInScope  ) ->
   (ModulesInScope -> m ModulesInScope) ->
+  (InScopeSet     -> m InScopeSet    ) ->
   Scope -> m Scope
-mapScopeM_ fd fm = mapScopeM (const fd) (const fm)
+mapScopeM_ fd fm fs = mapScopeM (const fd) (const fm) (const fs)
 
 -- | Zip together two scopes. The resulting scope has the same name as the
 --   first scope.
 zipScope :: (NameSpaceId -> NamesInScope   -> NamesInScope   -> NamesInScope  ) ->
             (NameSpaceId -> ModulesInScope -> ModulesInScope -> ModulesInScope) ->
+            (NameSpaceId -> InScopeSet     -> InScopeSet     -> InScopeSet    ) ->
             Scope -> Scope -> Scope
-zipScope fd fm s1 s2 =
+zipScope fd fm fs s1 s2 =
   s1 { scopeNameSpaces = [ (nsid, zipNS nsid ns1 ns2)
                          | ((nsid, ns1), (nsid', ns2)) <- zipWith' (,) (scopeNameSpaces s1) (scopeNameSpaces s2)
                          , assert (nsid == nsid')
@@ -379,19 +417,29 @@ zipScope fd fm s1 s2 =
   where
     assert True  = True
     assert False = __IMPOSSIBLE__
-    zipNS acc = zipNameSpace (fd acc) (fm acc)
+    zipNS acc = zipNameSpace (fd acc) (fm acc) (fs acc)
 
 -- | Same as 'zipScope' but applies the same function to both the public and
 --   private name spaces.
 zipScope_ :: (NamesInScope   -> NamesInScope   -> NamesInScope  ) ->
              (ModulesInScope -> ModulesInScope -> ModulesInScope) ->
+             (InScopeSet     -> InScopeSet     -> InScopeSet    ) ->
              Scope -> Scope -> Scope
-zipScope_ fd fm = zipScope (const fd) (const fm)
+zipScope_ fd fm fs = zipScope (const fd) (const fm) (const fs)
+
+-- | Recompute the inScope sets of a scope.
+recomputeInScopeSets :: Scope -> Scope
+recomputeInScopeSets = updateScopeNameSpaces (map $ second recomputeInScope)
+  where
+    recomputeInScope ns = ns { nsInScope = Set.unions $ map (Set.fromList . map anameName)
+                                         $ Map.elems $ nsNames ns }
 
 -- | Filter a scope keeping only concrete names matching the predicates.
 --   The first predicate is applied to the names and the second to the modules.
 filterScope :: (C.Name -> Bool) -> (C.Name -> Bool) -> Scope -> Scope
-filterScope pd pm = mapScope_ (Map.filterKeys pd) (Map.filterKeys pm)
+filterScope pd pm = recomputeInScopeSets .  mapScope_ (Map.filterKeys pd) (Map.filterKeys pm) id
+  -- We don't have enough information in the in scope set to do an
+  -- incremental update here, so just recompute it from the name map.
 
 -- | Return all names in a scope.
 allNamesInScope :: InScope a => Scope -> ThingsInScope a
@@ -417,11 +465,12 @@ thingsInScope :: [NameSpaceId] -> Scope -> NameSpace
 thingsInScope fs s =
   NameSpace { nsNames   = namesInScope fs s
             , nsModules = namesInScope fs s
+            , nsInScope = Set.unions [ nsInScope $ scopeNameSpace nsid s | nsid <- fs ]
             }
 
 -- | Merge two scopes. The result has the name of the first scope.
 mergeScope :: Scope -> Scope -> Scope
-mergeScope = zipScope_ mergeNames mergeNames
+mergeScope = zipScope_ mergeNames mergeNames Set.union
 
 -- | Merge a non-empty list of scopes. The result has the name of the first
 --   scope in the list.
@@ -460,15 +509,18 @@ addNamesToScope :: NameSpaceId -> C.Name -> [AbstractName] -> Scope -> Scope
 addNamesToScope acc x ys s = mergeScope s s1
   where
     s1 = setScopeAccess acc $ setNameSpace PublicNS ns emptyScope
-    ns = emptyNameSpace { nsNames = Map.singleton x ys }
+    ns = emptyNameSpace { nsNames   = Map.singleton x ys
+                        , nsInScope = Set.fromList (map anameName ys) }
 
 -- | Add a name to a scope.
 addNameToScope :: NameSpaceId -> C.Name -> AbstractName -> Scope -> Scope
 addNameToScope acc x y s = addNamesToScope acc x [y] s
 
--- | Remove a name from a scope.
+-- | Remove a name from a scope. Caution: does not update the nsInScope set.
+--   This is only used by rebindName and in that case we add the name right
+--   back (but with a different kind).
 removeNameFromScope :: NameSpaceId -> C.Name -> Scope -> Scope
-removeNameFromScope ns x s = mapScope remove (const id) s
+removeNameFromScope ns x s = mapScope remove (const id) (const id) s
   where
     remove ns' | ns' /= ns = id
                | otherwise = Map.delete x
@@ -480,58 +532,82 @@ addModuleToScope acc x m s = mergeScope s s1
     s1 = setScopeAccess acc $ setNameSpace PublicNS ns emptyScope
     ns = emptyNameSpace { nsModules = Map.singleton x [m] }
 
+-- When we get here we cannot have both using and hiding
+type UsingOrHiding = Either C.Using [C.ImportedName]
+
+usingOrHiding :: C.ImportDirective -> UsingOrHiding
+usingOrHiding i =
+  case (using i, hiding i) of
+    (UseEverything, xs) -> Right xs
+    (u, [])             -> Left u
+    _                   -> __IMPOSSIBLE__
+
 -- | Apply an 'ImportDirective' to a scope.
-applyImportDirective :: ImportDirective -> Scope -> Scope
+applyImportDirective :: C.ImportDirective -> Scope -> Scope
 applyImportDirective dir s = mergeScope usedOrHidden renamed
   where
-    usedOrHidden = useOrHide (hideLHS (renaming dir) $ usingOrHiding dir) s
-    renamed      = rename (renaming dir) $ useOrHide useRenamedThings s
+    usedOrHidden = useOrHide (hideLHS (impRenaming dir) $ usingOrHiding dir) s
+    renamed      = rename (impRenaming dir) $ useOrHide useRenamedThings s
 
-    useRenamedThings = Using $ map renFrom $ renaming dir
+    useRenamedThings = Left $ Using $ map renFrom $ impRenaming dir
 
-    hideLHS :: [Renaming] -> UsingOrHiding -> UsingOrHiding
-    hideLHS _   i@(Using _) = i
-    hideLHS ren (Hiding xs) = Hiding $ xs ++ map renFrom ren
+    hideLHS :: [C.Renaming] -> UsingOrHiding -> UsingOrHiding
+    hideLHS _   i@(Left _) = i
+    hideLHS ren (Right xs) = Right $ xs ++ map renFrom ren
 
     useOrHide :: UsingOrHiding -> Scope -> Scope
-    useOrHide (Hiding xs) s = filterNames notElem notElem xs s
-    useOrHide (Using  xs) s = filterNames elem    elem    xs s
+    useOrHide (Right xs)        s = filterNames notElem notElem xs s
+    useOrHide (Left (Using xs)) s = filterNames elem    elem    xs s
+    useOrHide _                 _ = __IMPOSSIBLE__
 
-    filterNames :: (C.Name -> [C.Name] -> Bool) -> (C.Name -> [C.Name] -> Bool) ->
-                   [ImportedName] -> Scope -> Scope
-    filterNames pd pm xs = filterScope' (flip pd ds) (flip pm ms)
+    filterNames :: (C.Name -> [C.Name] -> Bool) ->
+                   (C.Name -> [C.Name] -> Bool) ->
+                   [C.ImportedName] -> Scope -> Scope
+    filterNames pd pm xs = filterScope (`pd` ds) (`pm` ms)
       where
         ds = [ x | ImportedName   x <- xs ]
         ms = [ m | ImportedModule m <- xs ]
 
-    filterScope' pd pm = filterScope pd pm
-
     -- Renaming
-    rename :: [Renaming] -> Scope -> Scope
+    rename :: [C.Renaming] -> Scope -> Scope
     rename rho = mapScope_ (Map.mapKeys $ ren drho)
                            (Map.mapKeys $ ren mrho)
+                           id
       where
-        mrho = [ (x, y) | Renaming { renFrom = ImportedModule x, renTo = y } <- rho ]
-        drho = [ (x, y) | Renaming { renFrom = ImportedName   x, renTo = y } <- rho ]
+        (drho, mrho) = partitionEithers $ for rho $ \case
+          Renaming (ImportedName   x) (ImportedName   y) _ -> Left  (x,y)
+          Renaming (ImportedModule x) (ImportedModule y) _ -> Right (x,y)
+          _ -> __IMPOSSIBLE__
 
         ren r x = fromMaybe x $ lookup x r
 
 -- | Rename the abstract names in a scope.
 renameCanonicalNames :: Map A.QName A.QName -> Map A.ModuleName A.ModuleName ->
                         Scope -> Scope
-renameCanonicalNames renD renM = mapScope_ renameD renameM
+renameCanonicalNames renD renM = mapScope_ renameD renameM (Set.map newName)
   where
-    renameD = Map.map $ map $ over lensAnameName $ \ x -> Map.findWithDefault x x renD
-    renameM = Map.map $ map $ over lensAmodName  $ \ x -> Map.findWithDefault x x renM
+    newName x = Map.findWithDefault x x renD
+    newMod  x = Map.findWithDefault x x renM
+
+    renameD = Map.map $ map $ over lensAnameName newName
+    renameM = Map.map $ map $ over lensAmodName  newMod
 
 -- | Remove private name space of a scope.
 --
 --   Should be a right identity for 'exportedNamesInScope'.
 --   @exportedNamesInScope . restrictPrivate == exportedNamesInScope@.
 restrictPrivate :: Scope -> Scope
-restrictPrivate s
-  = setNameSpace PrivateNS  emptyNameSpace
-  $ s { scopeImports = Map.empty }
+restrictPrivate s = setNameSpace PrivateNS emptyNameSpace
+                  $ s { scopeImports = Map.empty }
+
+-- | Remove private things from the given module from a scope.
+restrictLocalPrivate :: ModuleName -> Scope -> Scope
+restrictLocalPrivate m = mapScope' PrivateNS (Map.mapMaybe rName) (Map.mapMaybe rMod)
+                                             (Set.filter (not . (`isInModule` m)))
+  where
+    check p x = x <$ guard (p x)
+    rName as = check (not . null) $ filter (not . (`isInModule`    m) . anameName) as
+    rMod  as = check (not . null) $ filter (not . (`isSubModuleOf` m) . amodName)  as
 
 -- | Remove names that can only be used qualified (when opening a scope)
 removeOnlyQualified :: Scope -> Scope
@@ -539,7 +615,7 @@ removeOnlyQualified s = setNameSpace OnlyQualifiedNS emptyNameSpace s
 
 -- | Add an explanation to why things are in scope.
 inScopeBecause :: (WhyInScope -> WhyInScope) -> Scope -> Scope
-inScopeBecause f = mapScope_ mapName mapMod
+inScopeBecause f = mapScope_ mapName mapMod id
   where
     mapName = fmap . map $ \a -> a { anameLineage = f $ anameLineage a }
     mapMod  = fmap . map $ \a -> a { amodLineage  = f $ amodLineage a  }
@@ -559,6 +635,11 @@ publicModules scope = Map.filterWithKey (\ m _ -> reachable m) allMods
 
     reachable = (`elem` chase root)
 
+publicNames :: ScopeInfo -> Set AbstractName
+publicNames scope =
+  Set.fromList $ concat $ Map.elems $
+  exportedNamesInScope $ mergeScopes $ Map.elems $ publicModules scope
+
 everythingInScope :: ScopeInfo -> NameSpace
 everythingInScope scope = allThingsInScope $ mergeScopes $
     (s0 :) $ map look $ scopeParents s0
@@ -601,6 +682,36 @@ flattenScope ms scope =
     moduleScope :: A.ModuleName -> Scope
     moduleScope m = fromMaybe __IMPOSSIBLE__ $ Map.lookup m $ scopeModules scope
 
+-- | Get all concrete names in scope. Includes bound variables.
+concreteNamesInScope :: ScopeInfo -> Set C.QName
+concreteNamesInScope scope =
+  Set.unions [ build allNamesInScope root, imported, locals ]
+  where
+    current = moduleScope $ scopeCurrent scope
+    root    = mergeScopes $ current : map moduleScope (scopeParents current)
+
+    locals  = Set.fromList [ C.QName x | (x, _) <- scopeLocals scope ]
+
+    imported = Set.unions
+               [ qual c (build exportedNamesInScope $ moduleScope a)
+               | (c, a) <- Map.toList $ scopeImports root ]
+    qual c = Set.map (q c)
+      where
+        q (C.QName x)  = C.Qual x
+        q (C.Qual m x) = C.Qual m . q x
+
+    build :: (forall a. InScope a => Scope -> ThingsInScope a) -> Scope -> Set C.QName
+    build getNames s = Set.unions $
+        (Set.fromList $ map C.QName $ Map.keys (getNames s :: ThingsInScope AbstractName)) :
+          [ Set.mapMonotonic (\ y -> C.Qual x y) $
+              build exportedNamesInScope $ moduleScope m
+          | (x, mods) <- Map.toList (getNames s)
+          , prettyShow x /= "_"
+          , AbsModule m _ <- mods ]
+
+    moduleScope :: A.ModuleName -> Scope
+    moduleScope m = fromMaybe __IMPOSSIBLE__ $ Map.lookup m $ scopeModules scope
+
 -- | Look up a name in the scope
 scopeLookup :: InScope a => C.QName -> ScopeInfo -> [a]
 scopeLookup q scope = map fst $ scopeLookup' q scope
@@ -637,10 +748,10 @@ scopeLookup' q scope = nubBy ((==) `on` fst) $ findName q root ++ maybeToList to
         -- that constructors can also be qualified by their datatype
         -- and projections by their record type.  This feature is off
         -- if we just consider the modules:
-        -- m <- mods
+        m <- mods
         -- The feature is on if we consider also the data and record types:
         -- trace ("mods ++ defs = " ++ show (mods ++ defs)) $ do
-        m <- nub $ mods ++ defs -- record types will appear both as a mod and a def
+        -- m <- nub $ mods ++ defs -- record types will appear both as a mod and a def
         -- Get the scope of module m, if any, and remove its private definitions.
         let ss  = Map.lookup m $ scopeModules scope
             ss' = restrictPrivate <$> ss
@@ -680,22 +791,53 @@ scopeLookup' q scope = nubBy ((==) `on` fst) $ findName q root ++ maybeToList to
 
 -- * Inverse look-up
 
-data AllowAmbiguousConstructors = AllowAmbiguousConstructors | NoAmbiguousConstructors
+data AllowAmbiguousNames = AmbiguousAnything | AmbiguousConstructors | AmbiguousNothing
   deriving (Eq)
 
+isNameInScope :: A.QName -> ScopeInfo -> Bool
+isNameInScope q scope =
+  billToPure [ Scoping, InverseScopeLookup ] $
+  Set.member q (scopeInScope scope)
+
 -- | Find the concrete names that map (uniquely) to a given abstract name.
 --   Sort by length, shortest first.
 
 inverseScopeLookup :: Either A.ModuleName A.QName -> ScopeInfo -> [C.QName]
-inverseScopeLookup = inverseScopeLookup' AllowAmbiguousConstructors
+inverseScopeLookup = inverseScopeLookup' AmbiguousConstructors
 
-inverseScopeLookup' :: AllowAmbiguousConstructors -> Either A.ModuleName A.QName -> ScopeInfo -> [C.QName]
-inverseScopeLookup' ambCon name scope = billToPure [ Scoping , InverseScopeLookup ] $
+inverseScopeLookup' :: AllowAmbiguousNames -> Either A.ModuleName A.QName -> ScopeInfo -> [C.QName]
+inverseScopeLookup' amb name scope = billToPure [ Scoping , InverseScopeLookup ] $
   -- trace ("importMap = " ++ show importMap) $
   -- trace ("moduleMap = " ++ show moduleMap) $
   case name of
     Left  m -> best $ filter unambiguousModule $ findModule m
-    Right q -> best $ filter unambiguousName   $ findName nameMap q
+    Right q -> best $ filter unambiguousName   $ findName q
+  where
+    findName   x = fromMaybe [] $ Map.lookup x (scopeInverseName scope)
+    findModule x = fromMaybe [] $ Map.lookup x (scopeInverseModule scope)
+
+    len :: C.QName -> Int
+    len (C.QName _)  = 1
+    len (C.Qual _ x) = 1 + len x
+
+    best :: [C.QName] -> [C.QName]
+    best = sortBy (compare `on` len)
+
+    unique :: forall a . [a] -> Bool
+    unique []      = __IMPOSSIBLE__
+    unique [_]     = True
+    unique (_:_:_) = False
+
+    unambiguousModule q = amb == AmbiguousAnything || unique (scopeLookup q scope :: [AbstractModule])
+    unambiguousName   q = amb == AmbiguousAnything || unique xs || amb == AmbiguousConstructors && all ((ConName ==) . anameKind) xs
+      where xs = scopeLookup q scope
+
+recomputeInverseScopeMaps :: ScopeInfo -> ScopeInfo
+recomputeInverseScopeMaps scope = billToPure [ Scoping , InverseScopeLookup ] $
+  scope { scopeInverseName   = nameMap
+        , scopeInverseModule = Map.fromList [ (x, findModule x) | x <- Map.keys moduleMap ++ Map.keys importMap ]
+        , scopeInScope       = Set.unions $ map (scopeSet . snd) scopes
+        }
   where
     this = scopeCurrent scope
     current = this : scopeParents (moduleScope this)
@@ -707,12 +849,8 @@ inverseScopeLookup' ambCon name scope = billToPure [ Scoping , InverseScopeLooku
     restrict m s | m `elem` current = s
                  | otherwise = restrictPrivate s
 
-    len :: C.QName -> Int
-    len (C.QName _)  = 1
-    len (C.Qual _ x) = 1 + len x
-
-    best :: [C.QName] -> [C.QName]
-    best = sortBy (compare `on` len) . filter (not . internalName)
+    scopeSet     s = Set.unions $ map (namespaceSet . snd) $ scopeNameSpaces s
+    namespaceSet s = nsInScope s
 
     internalName :: C.QName -> Bool
     internalName C.QName{} = False
@@ -721,14 +859,6 @@ inverseScopeLookup' ambCon name scope = billToPure [ Scoping , InverseScopeLooku
       -- Recognize fresh names created Parser.y
       intern (C.Name _ [ C.Id ('.' : '#' : _) ]) = True
       intern _ = False
-    unique :: forall a . [a] -> Bool
-    unique []      = __IMPOSSIBLE__
-    unique [_]     = True
-    unique (_:_:_) = False
-
-    unambiguousModule q = unique (scopeLookup q scope :: [AbstractModule])
-    unambiguousName   q = unique xs || AllowAmbiguousConstructors == ambCon && all ((ConName ==) . anameKind) xs
-      where xs = scopeLookup q scope
 
     findName :: Ord a => Map a [(A.ModuleName, C.Name)] -> a -> [C.QName]
     findName table q = do
@@ -737,7 +867,9 @@ inverseScopeLookup' ambCon name scope = billToPure [ Scoping , InverseScopeLooku
         then return (C.QName x)
         else do
           y <- findModule m
-          return $ C.qualify y x
+          let z = C.qualify y x
+          guard $ not $ internalName z
+          return z
 
     findModule :: A.ModuleName -> [C.QName]
     findModule q = findName moduleMap q ++
@@ -758,14 +890,20 @@ inverseScopeLookup' ambCon name scope = billToPure [ Scoping , InverseScopeLooku
       (m, s)  <- scopes
       (x, ms) <- Map.toList (allNamesInScope s)
       q       <- anameName <$> ms
-      return (q, [(m, x)])
+      if elem m current
+        then return (q, [C.QName x])
+        else do
+          y <- findModule m
+          let z = C.qualify y x
+          guard $ not $ internalName z
+          return (q, [z])
 
 -- | Find the concrete names that map (uniquely) to a given abstract qualified name.
 --   Sort by length, shortest first.
 inverseScopeLookupName :: A.QName -> ScopeInfo -> [C.QName]
 inverseScopeLookupName x = inverseScopeLookup (Right x)
 
-inverseScopeLookupName' :: AllowAmbiguousConstructors -> A.QName -> ScopeInfo -> [C.QName]
+inverseScopeLookupName' :: AllowAmbiguousNames -> A.QName -> ScopeInfo -> [C.QName]
 inverseScopeLookupName' ambCon x = inverseScopeLookup' ambCon (Right x)
 
 -- | Find the concrete names that map (uniquely) to a given abstract module name.
@@ -791,7 +929,7 @@ instance Show NameSpaceId where
     OnlyQualifiedNS -> "only-qualified"
 
 instance Show NameSpace where
-  show (NameSpace names mods) =
+  show (NameSpace names mods _) =
     unlines $
       blockOfLines "names"   (map pr $ Map.toList names) ++
       blockOfLines "modules" (map pr $ Map.toList mods)
@@ -818,7 +956,7 @@ blockOfLines _  [] = []
 blockOfLines hd ss = hd : map ("  " ++) ss
 
 instance Show ScopeInfo where
-  show (ScopeInfo this mods locals ctx) =
+  show (ScopeInfo this mods locals ctx _ _ _) =
     unlines $
       [ "ScopeInfo"
       , "  current = " ++ show this
