@@ -1,5 +1,7 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP                      #-}
+{-# LANGUAGE LambdaCase               #-}
 {-# LANGUAGE NondecreasingIndentation #-}
+{-# LANGUAGE TupleSections            #-}
 
 #if __GLASGOW_HASKELL__ >= 710
 {-# LANGUAGE FlexibleContexts #-}
@@ -11,11 +13,11 @@
 module Agda.Syntax.Scope.Monad where
 
 import Prelude hiding (mapM)
-import Control.Arrow ((***), first, second)
+import Control.Arrow (first, second)
 import Control.Applicative
-import Control.Monad hiding (mapM)
-import Control.Monad.Writer hiding (mapM)
-import Control.Monad.State hiding (mapM)
+import Control.Monad hiding (mapM, forM)
+import Control.Monad.Writer hiding (mapM, forM)
+import Control.Monad.State hiding (mapM, forM)
 
 import Data.List as List
 import Data.Map (Map)
@@ -23,7 +25,7 @@ import qualified Data.Map as Map
 import Data.Maybe
 import Data.Set (Set)
 import qualified Data.Set as Set
-import Data.Traversable
+import Data.Traversable hiding (for)
 
 import Agda.Syntax.Common
 import Agda.Syntax.Position
@@ -39,11 +41,12 @@ import Agda.TypeChecking.Monad.Options
 
 import qualified Agda.Utils.AssocList as AssocList
 import Agda.Utils.Function
+import Agda.Utils.Functor
 import Agda.Utils.List
 import Agda.Utils.Maybe
 import Agda.Utils.Null (unlessNull)
-import Agda.Utils.Size
 import Agda.Utils.Pretty
+import Agda.Utils.Size
 import Agda.Utils.Tuple
 
 #include "undefined.h"
@@ -68,7 +71,7 @@ getCurrentModule :: ScopeM A.ModuleName
 getCurrentModule = setRange noRange . scopeCurrent <$> getScope
 
 setCurrentModule :: A.ModuleName -> ScopeM ()
-setCurrentModule m = modifyScopeInfo $ \s -> s { scopeCurrent = m }
+setCurrentModule m = modifyScope $ \s -> s { scopeCurrent = m }
 
 withCurrentModule :: A.ModuleName -> ScopeM a -> ScopeM a
 withCurrentModule new action = do
@@ -107,16 +110,13 @@ createModule b m = do
       sm = emptyScope { scopeName           = m
                       , scopeParents        = parents
                       , scopeDatatypeModule = b }
-  -- Andreas, 2015-07-02: internall error if module is not new.
-  modifyScopes $ Map.insertWith __IMPOSSIBLE__ m sm
-
--- | Apply a function to the scope info.
-modifyScopeInfo :: (ScopeInfo -> ScopeInfo) -> ScopeM ()
-modifyScopeInfo = modifyScope
+  -- Andreas, 2015-07-02: internal error if module is not new.
+  -- Ulf, 2016-02-15: It's not new if multiple imports (#1770).
+  modifyScopes $ Map.insertWith const m sm
 
 -- | Apply a function to the scope map.
 modifyScopes :: (Map A.ModuleName Scope -> Map A.ModuleName Scope) -> ScopeM ()
-modifyScopes f = modifyScopeInfo $ \s -> s { scopeModules = f $ scopeModules s }
+modifyScopes f = modifyScope $ \s -> s { scopeModules = f $ scopeModules s }
 
 -- | Apply a function to the given scope.
 modifyNamedScope :: A.ModuleName -> (Scope -> Scope) -> ScopeM ()
@@ -126,14 +126,17 @@ setNamedScope :: A.ModuleName -> Scope -> ScopeM ()
 setNamedScope m s = modifyNamedScope m $ const s
 
 -- | Apply a monadic function to the top scope.
-modifyNamedScopeM :: A.ModuleName -> (Scope -> ScopeM Scope) -> ScopeM ()
-modifyNamedScopeM m f = setNamedScope m =<< f =<< getNamedScope m
+modifyNamedScopeM :: A.ModuleName -> (Scope -> ScopeM (a, Scope)) -> ScopeM a
+modifyNamedScopeM m f = do
+  (a, s) <- f =<< getNamedScope m
+  setNamedScope m s
+  return a
 
 -- | Apply a function to the current scope.
 modifyCurrentScope :: (Scope -> Scope) -> ScopeM ()
 modifyCurrentScope f = getCurrentModule >>= (`modifyNamedScope` f)
 
-modifyCurrentScopeM :: (Scope -> ScopeM Scope) -> ScopeM ()
+modifyCurrentScopeM :: (Scope -> ScopeM (a, Scope)) -> ScopeM a
 modifyCurrentScopeM f = getCurrentModule >>= (`modifyNamedScopeM` f)
 
 -- | Apply a function to the public or private name space.
@@ -142,7 +145,7 @@ modifyCurrentNameSpace acc f = modifyCurrentScope $ updateScopeNameSpaces $
   AssocList.updateAt acc f
 
 setContextPrecedence :: Precedence -> ScopeM ()
-setContextPrecedence p = modifyScopeInfo $ \s -> s { scopePrecedence = p }
+setContextPrecedence p = modifyScope_ $ \s -> s { scopePrecedence = p }
 
 getContextPrecedence :: ScopeM Precedence
 getContextPrecedence = scopePrecedence <$> getScope
@@ -159,7 +162,7 @@ getLocalVars :: ScopeM LocalVars
 getLocalVars = scopeLocals <$> getScope
 
 modifyLocalVars :: (LocalVars -> LocalVars) -> ScopeM ()
-modifyLocalVars = modifyScope . updateScopeLocals
+modifyLocalVars = modifyScope_ . updateScopeLocals
 
 setLocalVars :: LocalVars -> ScopeM ()
 setLocalVars vars = modifyLocalVars $ const vars
@@ -189,9 +192,9 @@ freshAbstractName fx x = do
     , nameFixity      = fx
     }
 
--- | @freshAbstractName_ = freshAbstractName defaultFixity@
+-- | @freshAbstractName_ = freshAbstractName noFixity'@
 freshAbstractName_ :: C.Name -> ScopeM A.Name
-freshAbstractName_ = freshAbstractName defaultFixity'
+freshAbstractName_ = freshAbstractName noFixity'
 
 -- | Create a fresh abstract qualified name.
 freshAbstractQName :: Fixity' -> C.Name -> ScopeM A.QName
@@ -300,7 +303,7 @@ getNotation x ns = do
 
 -- | Bind a variable. The abstract name is supplied as the second argument.
 bindVariable :: C.Name -> A.Name -> ScopeM ()
-bindVariable x y = modifyScope $ updateScopeLocals $ AssocList.insert x $ LocalVar y
+bindVariable x y = modifyScope_ $ updateScopeLocals $ AssocList.insert x $ LocalVar y
 
 -- | Bind a defined name. Must not shadow anything.
 bindName :: Access -> KindOfName -> C.Name -> A.QName -> ScopeM ()
@@ -341,7 +344,7 @@ bindQModule acc q m = modifyCurrentScope $ \s ->
 
 -- | Clear the scope of any no names.
 stripNoNames :: ScopeM ()
-stripNoNames = modifyScopes $ Map.map $ mapScope_ stripN stripN
+stripNoNames = modifyScopes $ Map.map $ mapScope_ stripN stripN id
   where
     stripN = Map.filterWithKey $ const . not . isNoName
 
@@ -351,23 +354,17 @@ type WSM = StateT Out ScopeM
 -- | Create a new scope with the given name from an old scope. Renames
 --   public names in the old scope to match the new name and returns the
 --   renamings.
---
---   Data and record types share a common abstract name with their module.
---   This invariant needs to be preserved by @copyScope@, since constructors
---   (fields) can be qualified by their data (record) type name (as an
---   alternative to qualification by their module).
---   (See Issue 836).
 copyScope :: C.QName -> A.ModuleName -> Scope -> ScopeM (Scope, (A.Ren A.ModuleName, A.Ren A.QName))
 copyScope oldc new s = first (inScopeBecause $ Applied oldc) <$> runStateT (copy new s) ([], [])
   where
-    -- | A memoizing algorithm, the renamings serving as memo structure.
     copy :: A.ModuleName -> Scope -> StateT (A.Ren A.ModuleName, A.Ren A.QName) ScopeM Scope
     copy new s = do
       lift $ reportSLn "scope.copy" 20 $ "Copying scope " ++ show old ++ " to " ++ show new
       lift $ reportSLn "scope.copy" 50 $ show s
       s0 <- lift $ getNamedScope new
-      -- Delete private names, then copy names and modules.
-      s' <- mapScopeM_ copyD copyM $ setNameSpace PrivateNS emptyNameSpace s
+      -- Delete private names, then copy names and modules. Recompute inScope
+      -- set rather than trying to copy it.
+      s' <- recomputeInScopeSets <$> mapScopeM_ copyD copyM return (setNameSpace PrivateNS emptyNameSpace s)
       -- Fix name and parent.
       return $ s' { scopeName    = scopeName s0
                   , scopeParents = scopeParents s0
@@ -398,15 +395,16 @@ copyScope oldc new s = first (inScopeBecause $ Applied oldc) <$> runStateT (copy
         findName x = lookup x <$> gets snd
         findMod  x = lookup x <$> gets fst
 
+        refresh :: A.Name -> WSM A.Name
+        refresh x = do
+          i <- lift fresh
+          return $ x { nameId = i }
+
         -- Change a binding M.x -> old.M'.y to M.x -> new.M'.y
         renName :: A.QName -> WSM A.QName
         renName x = do
-          -- Check whether we have already seen a module of the same name.
-          -- If yes, use its copy as @y@.
-          y <- ifJustM (findMod $ qnameToMName x) (return . mnameToQName) $ {- else -} do
-            -- First time, generate a fresh name for it.
-            i <- lift fresh
-            return $ A.qualify new' $ (qnameName x) { nameId = i }
+          -- Generate a fresh name for the target.
+          y <- A.qualify new' <$> refresh (qnameName x)
           lift $ reportSLn "scope.copy" 50 $ "  Copying " ++ show x ++ " to " ++ show y
           -- Andreas, 2015-08-11 Issue 1619:
           -- Names copied by a module macro should get the module macro's
@@ -425,19 +423,22 @@ copyScope oldc new s = first (inScopeBecause $ Applied oldc) <$> runStateT (copy
           -- If we have already copied this module, return the copy.
           ifJustM (findMod x) return $ {- else -} do
 
-          -- Check whether we have seen it already, yet as  name.
-          -- If yes, use its copy as @y@.
-          y <- ifJustM (findName $ mnameToQName x) (return . qnameToMName) $ {- else -} do
+          y <- do
              -- Andreas, Jesper, 2015-07-02: Issue 1597
              -- Don't blindly drop a prefix of length of the old qualifier.
              -- If things are imported by open public they do not have the old qualifier
              -- as prefix.  Those need just to be linked, not copied.
              -- return $ A.mnameFromList $ (newL ++) $ drop (size old) $ A.mnameToList x
-             caseMaybe (maybePrefixMatch (A.mnameToList old) (A.mnameToList x)) (return x) $ \ suffix -> do
-               return $ A.mnameFromList $ newL ++ suffix
+             -- caseMaybe (maybePrefixMatch (A.mnameToList old) (A.mnameToList x)) (return x) $ \ suffix -> do
+             --   return $ A.mnameFromList $ newL ++ suffix
+             -- Ulf, 2016-02-22: #1726
+             -- We still need to copy modules from 'open public'. Same as in renName.
+             y <- refresh (last $ A.mnameToList x)
+             return $ A.mnameFromList $ newL ++ [y]
           -- Andreas, Jesper, 2015-07-02: Issue 1597
           -- Don't copy a module over itself, it will just be emptied of its contents.
           if (x == y) then return x else do
+          lift $ reportSLn "scope.copy" 50 $ "  Copying module " ++ show x ++ " to " ++ show y
           addMod x y
           -- We need to copy the contents of included modules recursively
           lift $ createModule False y
@@ -448,48 +449,176 @@ copyScope oldc new s = first (inScopeBecause $ Applied oldc) <$> runStateT (copy
 
 -- | Apply an import directive and check that all the names mentioned actually
 --   exist.
-applyImportDirectiveM :: C.QName -> ImportDirective -> Scope -> ScopeM Scope
-applyImportDirectiveM m dir scope = do
-    let xs = filter doesntExist names
-    reportSLn "scope.import.apply" 20 $ "non existing names: " ++ show xs
-    unless (null xs)  $ typeError $ ModuleDoesntExport m xs
+applyImportDirectiveM :: C.QName -> C.ImportDirective -> Scope -> ScopeM (A.ImportDirective, Scope)
+applyImportDirectiveM m dir@ImportDirective{ impRenaming = ren, using = u, hiding = h } scope = do
+    -- Translate exported names to abstract syntax.
+    -- Raise error if unsuccessful.
+    caseMaybe mNamesA doesntExport $ \ namesA -> do
 
+    let extraModules =
+          [ x | ImportedName x <- names,
+                let mx = ImportedModule x,
+                 not $ doesntExist mx,
+                 notElem mx names ]
+        dir' = addExtraModules extraModules dir
+
+    dir' <- sanityCheck dir'
+
+    -- Check for duplicate imports in a single import directive.
+    -- @dup@ : To be imported names that are mentioned more than once.
     let dup = targetNames \\ nub targetNames
     unless (null dup) $ typeError $ DuplicateImports m dup
-    return $ applyImportDirective dir scope
+
+    -- Apply the import directive.
+    let scope' = applyImportDirective dir' scope
+
+    -- Look up the defined names in the new scope.
+    let namesInScope'   = (allNamesInScope scope' :: ThingsInScope AbstractName)
+    let modulesInScope' = (allNamesInScope scope' :: ThingsInScope AbstractModule)
+    let look x = head . Map.findWithDefault __IMPOSSIBLE__ x
+    -- We set the ranges to the ranges of the concrete names in order to get
+    -- highlighting for the names in the import directive.
+    let definedA = for definedNames $ \case
+          ImportedName   x -> ImportedName   . (x,) . setRange (getRange x) . anameName $ look x namesInScope'
+          ImportedModule x -> ImportedModule . (x,) . setRange (getRange x) . amodName  $ look x modulesInScope'
+
+    let adir = mapImportDir namesA definedA dir
+    return (adir, scope') -- TODO Issue 1714: adir
 
   where
+    -- | Names in the @using@ directive.
+    usingNames :: [ImportedName]
+    usingNames = case u of
+      Using  xs     -> xs
+      UseEverything -> []
+
+    -- | All names from the imported module mentioned in the import directive.
     names :: [ImportedName]
-    names = map renFrom (renaming dir) ++ case usingOrHiding dir of
-      Using  xs -> xs
-      Hiding xs -> xs
+    names = map renFrom ren ++ h ++ usingNames
 
+    -- | Names defined by the import (targets of renaming).
+    definedNames :: [ImportedName]
+    definedNames = map renTo ren
+
+    -- | Names to be in scope after import.
     targetNames :: [ImportedName]
-    targetNames = map renName (renaming dir) ++ case usingOrHiding dir of
-      Using xs -> xs
-      Hiding{} -> []
-      where
-        renName r = (renFrom r) { importedName = renTo r }
+    targetNames = definedNames ++ usingNames
 
-    doesntExist (ImportedName   x) = isNothing $
-      Map.lookup x (allNamesInScope scope :: ThingsInScope AbstractName)
-    doesntExist (ImportedModule x) = isNothing $
-      Map.lookup x (allNamesInScope scope :: ThingsInScope AbstractModule)
+    sanityCheck dir =
+      case (using dir, hiding dir) of
+        (Using xs, ys) -> do
+          let uselessHiding = [ x | x@ImportedName{} <- ys ] ++
+                              [ x | x@(ImportedModule y) <- ys, ImportedName y `notElem` names ]
+          unless (null uselessHiding) $ typeError $ GenericError $ "Hiding " ++ intercalate ", " (map show uselessHiding)
+                                                                ++ " has no effect"
+          return dir{ hiding = [] }
+        _ -> return dir
+
+    addExtraModules :: [C.Name] -> C.ImportDirective -> C.ImportDirective
+    addExtraModules extra dir =
+      dir{ using =
+              case using dir of
+                Using xs      -> Using $ concatMap addExtra xs
+                UseEverything -> UseEverything
+         , hiding      = concatMap addExtra      (hiding dir)
+         , impRenaming = concatMap extraRenaming (impRenaming dir)
+         }
+      where
+        addExtra f@(ImportedName y) | elem y extra = [f, ImportedModule y]
+        addExtra m = [m]
+
+        extraRenaming r@(Renaming from to rng) =
+          case (from, to) of
+            (ImportedName y, ImportedName z) | elem y extra ->
+              [r, Renaming (ImportedModule y) (ImportedModule z) rng]
+            _ -> [r]
+
+    -- | Names and modules (abstract) in scope before the import.
+    namesInScope   = (allNamesInScope scope :: ThingsInScope AbstractName)
+    modulesInScope = (allNamesInScope scope :: ThingsInScope AbstractModule)
+
+    -- | AST versions of the concrete names from the imported module.
+    --   @Nothing@ is one of the names is not exported.
+    mNamesA = forM names $ \case
+      ImportedName x   -> ImportedName   . (x,) . setRange (getRange x) . anameName . head <$> Map.lookup x namesInScope
+      ImportedModule x -> ImportedModule . (x,) . setRange (getRange x) . amodName  . head <$> Map.lookup x modulesInScope
+
+    head = headWithDefault __IMPOSSIBLE__
+
+    -- For the sake of the error message, we (re)compute the list of unresolved names.
+    doesntExport = do
+      -- Names @xs@ mentioned in the import directive @dir@ but not in the @scope@.
+      let xs = filter doesntExist names
+      reportSLn "scope.import.apply" 20 $ "non existing names: " ++ show xs
+      typeError $ ModuleDoesntExport m xs
+
+    doesntExist (ImportedName   x) = isNothing $ Map.lookup x namesInScope
+    doesntExist (ImportedModule x) = isNothing $ Map.lookup x modulesInScope
+
+-- | A finite map for @ImportedName@s.
+lookupImportedName
+  :: (Eq a, Eq b)
+  => ImportedName' a b
+  -> [ImportedName' (a,c) (b,d)]
+  -> ImportedName' c d
+lookupImportedName (ImportedName x) = loop where
+  loop [] = __IMPOSSIBLE__
+  loop (ImportedName (y,z) : _) | x == y = ImportedName z
+  loop (_ : ns) = loop ns
+lookupImportedName (ImportedModule x) = loop where
+  loop [] = __IMPOSSIBLE__
+  loop (ImportedModule (y,z) : _) | x == y = ImportedModule z
+  loop (_ : ns) = loop ns
+
+-- | Translation of @ImportDirective@.
+mapImportDir
+  :: (Eq a, Eq b)
+  => [ImportedName' (a,c) (b,d)]  -- ^ Translation of imported names.
+  -> [ImportedName' (a,c) (b,d)]  -- ^ Translation of names defined by this import.
+  -> ImportDirective' a b
+  -> ImportDirective' c d
+mapImportDir src tgt (ImportDirective r u h ren open) =
+  ImportDirective r
+    (mapUsing src u)
+    (map (`lookupImportedName` src) h)
+    (map (mapRenaming src tgt) ren) open
+
+-- | Translation of @Using or Hiding@.
+mapUsing
+  :: (Eq a, Eq b)
+  => [ImportedName' (a,c) (b,d)] -- ^ Translation of names in @using@ or @hiding@ list.
+  -> Using' a b
+  -> Using' c d
+mapUsing src UseEverything = UseEverything
+mapUsing src (Using  xs) = Using $ map (`lookupImportedName` src) xs
+
+-- | Translation of @Renaming@.
+mapRenaming
+  ::  (Eq a, Eq b)
+  => [ImportedName' (a,c) (b,d)]  -- ^ Translation of 'renFrom' names.
+  -> [ImportedName' (a,c) (b,d)]  -- ^ Translation of 'rento' names.
+  -> Renaming' a b
+  -> Renaming' c d
+mapRenaming src tgt (Renaming from to r) =
+  Renaming (lookupImportedName from src) (lookupImportedName to tgt) r
 
 -- | Open a module.
-openModule_ :: C.QName -> ImportDirective -> ScopeM ()
+openModule_ :: C.QName -> C.ImportDirective -> ScopeM A.ImportDirective
 openModule_ cm dir = do
   current <- getCurrentModule
   m <- amodName <$> resolveModule cm
   let acc | not (publicOpen dir)      = PrivateNS
           | m `isSubModuleOf` current = PublicNS
           | otherwise                 = ImportedNS
+
   -- Get the scope exported by module to be opened.
-  s <- setScopeAccess acc <$>
-        (applyImportDirectiveM cm dir . inScopeBecause (Opened cm) . removeOnlyQualified . restrictPrivate =<< getNamedScope m)
+  (adir, s') <- applyImportDirectiveM cm dir . inScopeBecause (Opened cm) . removeOnlyQualified . restrictPrivate =<< getNamedScope m
+  let s  = setScopeAccess acc s'
   let ns = scopeNameSpace acc s
   checkForClashes ns
   modifyCurrentScope (`mergeScope` s)
+
+  -- Importing names might shadow existing locals.
   verboseS "scope.locals" 10 $ do
     locals <- mapMaybe (\ (c,x) -> c <$ notShadowedLocal x) <$> getLocalVars
     let newdefs = Map.keys $ nsNames ns
@@ -500,6 +629,9 @@ openModule_ cm dir = do
     case Map.lookup c $ nsNames ns of
       Nothing -> x
       Just ys -> shadowLocal ys x
+
+  return adir
+
   where
     -- Only checks for clashes that would lead to the same
     -- name being exported twice from the module.
