@@ -914,7 +914,7 @@ data Hd = HdVar A.Name | HdCon A.QName | HdDef A.QName
 cOpApp :: Range -> C.QName -> A.Name -> [C.Expr] -> C.Expr
 cOpApp r x n es =
   C.OpApp r x (Set.singleton n)
-          (map (defaultNamedArg . NoPlaceholder . Ordinary) es)
+          (map (defaultNamedArg . noPlaceholder . Ordinary) es)
 
 tryToRecoverOpApp :: A.Expr -> AbsToCon C.Expr -> AbsToCon C.Expr
 tryToRecoverOpApp e def = caseMaybeM (recoverOpApp bracket cOpApp view e) def return
