@@ -1,8 +1,8 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP                        #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
-{-# LANGUAGE ImplicitParams #-}
-{-# LANGUAGE StandaloneDeriving #-}
-{-# LANGUAGE TupleSections #-}
+{-# LANGUAGE ImplicitParams             #-}
+{-# LANGUAGE StandaloneDeriving         #-}
+{-# LANGUAGE TupleSections              #-}
 
 -- | Call graphs and related concepts, more or less as defined in
 --     \"A Predicative Analysis of Structural Recursion\" by
@@ -30,35 +30,21 @@ module Agda.Termination.CallGraph
 
 import Prelude hiding (null)
 
-import Data.Foldable (Foldable)
-import qualified Data.Foldable as Fold
-import Data.Function
 import qualified Data.List as List
-import Data.Map (Map, (!))
-import qualified Data.Map as Map
 import Data.Monoid
 import Data.Set (Set)
-import qualified Data.Set as Set
-import Data.Traversable (Traversable)
-import qualified Data.Traversable as Trav
 
 import Agda.Termination.CallMatrix (CallMatrix, callMatrix, CallMatrixAug(..), CMSet(..), CallComb(..))
 import qualified Agda.Termination.CallMatrix as CMSet
 import Agda.Termination.CutOff
-import Agda.Termination.Order
 import Agda.Termination.SparseMatrix as Matrix hiding (tests)
-import Agda.Termination.Semiring (HasZero(..), Semiring)
-import qualified Agda.Termination.Semiring as Semiring
 
-import Agda.Utils.Favorites (Favorites(Favorites))
+import Agda.Utils.Favorites (Favorites)
 import qualified Agda.Utils.Favorites as Fav
 import Agda.Utils.Graph.AdjacencyMap.Unidirectional (Edge(..),Graph(..))
 import qualified Agda.Utils.Graph.AdjacencyMap.Unidirectional as Graph
 
 import Agda.Utils.Function
-import Agda.Utils.List hiding (tests)
-import Agda.Utils.Map
-import Agda.Utils.Maybe
 import Agda.Utils.Monad
 import Agda.Utils.Null
 import Agda.Utils.PartialOrd
@@ -68,9 +54,6 @@ import Agda.Utils.Singleton
 import Agda.Utils.TestHelpers
 import Agda.Utils.Tuple
 
-#include "undefined.h"
-import Agda.Utils.Impossible
-
 ------------------------------------------------------------------------
 -- Calls
 
@@ -124,7 +107,7 @@ toList = Graph.edges . theCallGraph
 -- | Converts a list of calls with associated meta information to a
 --   call graph.
 
-fromList :: Monoid cinfo => [Call cinfo] -> CallGraph cinfo
+fromList :: [Call cinfo] -> CallGraph cinfo
 fromList = CallGraph . Graph.fromListWith CMSet.union
 
 -- | 'null' checks whether the call graph is completely disconnected.
@@ -134,21 +117,19 @@ instance Null (CallGraph cinfo) where
 
 -- | Takes the union of two call graphs.
 
-union :: Monoid cinfo
-      => CallGraph cinfo -> CallGraph cinfo -> CallGraph cinfo
+union :: CallGraph cinfo -> CallGraph cinfo -> CallGraph cinfo
 union (CallGraph cs1) (CallGraph cs2) = CallGraph $
   Graph.unionWith CMSet.union cs1 cs2
 
 -- | 'CallGraph' is a monoid under 'union'.
 
-instance Monoid cinfo => Monoid (CallGraph cinfo) where
+instance Monoid (CallGraph cinfo) where
   mempty  = empty
   mappend = union
 
 -- | Inserts a call into a call graph.
 
-insert :: Monoid cinfo
-       => Node -> Node -> CallMatrix -> cinfo
+insert :: Node -> Node -> CallMatrix -> cinfo
        -> CallGraph cinfo -> CallGraph cinfo
 insert s t cm cinfo = CallGraph . Graph.insertEdgeWith CMSet.union e . theCallGraph
   where e = mkCall s t cm cinfo
@@ -265,7 +246,7 @@ instance Pretty cinfo => Pretty (CallGraph cinfo) where
 
 -- | Generates a call graph.
 
-callGraph :: (Monoid cinfo, Arbitrary cinfo) => Gen (CallGraph cinfo)
+callGraph :: Arbitrary cinfo => Gen (CallGraph cinfo)
 callGraph = do
   indices <- fmap List.nub arbitrary
   n <- natural
