@@ -1,6 +1,6 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP               #-}
 {-# LANGUAGE FlexibleInstances #-}
-{-# LANGUAGE ImplicitParams #-}
+{-# LANGUAGE ImplicitParams    #-}
 
 -- | An Abstract domain of relative sizes, i.e., differences
 --   between size of formal function parameter and function argument
@@ -19,10 +19,8 @@ module Agda.Termination.Order
   , tests
   ) where
 
-import Data.Foldable (Foldable)
 import qualified Data.Foldable as Fold
 import Data.List as List hiding (union, insert)
-import Data.Monoid
 
 import Agda.Termination.CutOff
 import Agda.Termination.SparseMatrix as Matrix hiding (tests)
