@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP                 #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 
 #if __GLASGOW_HASKELL__ >= 710
@@ -11,11 +11,8 @@ import Control.Applicative
 import Control.Monad.Reader (asks)
 
 import Data.List
-import Data.Monoid
 import qualified Data.Map as Map
 
-import Debug.Trace (trace)
-
 import Agda.Syntax.Internal
 import Agda.Syntax.Common
 
@@ -87,6 +84,8 @@ match' ((c, es, patch) : stack) = do
   -- traceSLn "reduce.compiled" 95 "CompiledClause.Match.match'" $ do
   debug $ do
 
+    shared <- sharedFun
+
     case c of
 
       -- impossible case
@@ -111,7 +110,7 @@ match' ((c, es, patch) : stack) = do
           lam x t        = Lam (argInfo x) (Abs (unArg x) t)
 
       -- splitting on the @n@th elimination
-      Case n bs -> do
+      Case (Arg _ n) bs -> do
         case genericSplitAt n es of
           -- if the @n@th elimination is not supplied, no match
           (_, []) -> no (NotBlocked Underapplied) es
@@ -158,7 +157,8 @@ match' ((c, es, patch) : stack) = do
                         vs          = map argFromElim es1
 
             -- Now do the matching on the @n@ths argument:
-            case fmap ignoreSharing <$> eb of
+            traceSLn "reduce.compiled" 100 ("caseing on raw " ++ show eb) $
+             case fmap ignoreSharing <$> eb of
               Blocked x _            -> no (Blocked x) es'
               NotBlocked _ (Apply (Arg info (MetaV x _))) -> no (Blocked x) es'
 
@@ -168,7 +168,8 @@ match' ((c, es, patch) : stack) = do
                 let cFrame stack = case ignoreSharing cv of
                       Con c vs -> conFrame c vs stack
                       _        -> stack
-                match' $ litFrame l $ cFrame $ catchAllFrame stack
+                traceSLn "reduce.compiled" 100 ("constructorForm = " ++ show cv) $
+                 match' $ litFrame l $ cFrame $ catchAllFrame stack
 
               -- In case of a constructor, push the conFrame
               NotBlocked _ (Apply (Arg info (Con c vs))) -> performedSimplification $
