@@ -54,6 +54,7 @@ import Agda.Utils.Monad
 import Agda.Utils.Maybe
 import Agda.Utils.Size
 import Agda.Utils.Tuple
+import Agda.Utils.Lens
 
 #include "undefined.h"
 import Agda.Utils.Impossible
@@ -127,19 +128,20 @@ compareTerm cmp a u v = do
     , nest 2 $ prettyTCM u <+> prettyTCM cmp <+> prettyTCM v
     , nest 2 $ text ":" <+> prettyTCM a
     ]
-  -- Check syntactic equality first. This actually saves us quite a bit of work.
-  ((u, v), equal) <- SynEq.checkSyntacticEquality u v
+  -- Check pointer equality first.
+  let checkPointerEquality def | not $ null $ List.intersect (pointerChain u) (pointerChain v) = do
+        verboseS "profile.sharing" 10 $ tick "pointer equality"
+        return ()
+      checkPointerEquality def = def
+  checkPointerEquality $ do
+    -- Check syntactic equality. This actually saves us quite a bit of work.
+    ((u, v), equal) <- runReduceM $ SynEq.checkSyntacticEquality u v
   -- OLD CODE, traverses the *full* terms u v at each step, even if they
   -- are different somewhere.  Leads to infeasibility in issue 854.
   -- (u, v) <- instantiateFull (u, v)
   -- let equal = u == v
-  if equal then unifyPointers cmp u v $ verboseS "profile.sharing" 20 $ tick "equal terms" else do
-    verboseS "profile.sharing" 20 $ tick "unequal terms"
-    let checkPointerEquality def | not $ null $ List.intersect (pointerChain u) (pointerChain v) = do
-          verboseS "profile.sharing" 10 $ tick "pointer equality"
-          return ()
-        checkPointerEquality def = def
-    checkPointerEquality $ do
+    unifyPointers cmp u v $ if equal then verboseS "profile.sharing" 20 $ tick "equal terms" else do
+      verboseS "profile.sharing" 20 $ tick "unequal terms"
       reportSDoc "tc.conv.term" 15 $ sep
         [ text "compareTerm (not syntactically equal)"
         , nest 2 $ prettyTCM u <+> prettyTCM cmp <+> prettyTCM v
@@ -152,7 +154,7 @@ compareTerm cmp a u v = do
       -- at function types into Size.
       let fallback = compareTerm' cmp a u v
           unlessSubtyping cont =
-            if cmp == CmpEq then cont else do
+              if cmp == CmpEq then cont else do
                 -- Andreas, 2014-04-12 do not short cut if type is blocked.
                 ifBlockedType a (\ _ _ -> fallback) {-else-} $ \ a -> do
                   -- do not short circuit size comparison!
@@ -191,13 +193,21 @@ unifyPointers :: Comparison -> Term -> Term -> TCM () -> TCM ()
 unifyPointers _ _ _ action = action
 -- unifyPointers cmp _ _ action | cmp /= CmpEq = action
 -- unifyPointers _ u v action = do
---   old <- gets stDirty
---   modify $ \s -> s { stDirty = False }
+--   reportSLn "tc.ptr.unify" 50 $ "Maybe unifying pointers\n  u = " ++ show u ++ "\n  v = " ++ show v
+--   old <- use stDirty
+--   stDirty .= False
 --   action
+--   reportSLn "tc.ptr.unify" 50 $ "Finished comparison\n  u = " ++ show u ++ "\n  v = " ++ show v
 --   (u, v) <- instantiate (u, v)
---   dirty <- gets stDirty
---   modify $ \s -> s { stDirty = old }
---   when (not dirty) $ forceEqualTerms u v
+--   reportSLn "tc.ptr.unify" 50 $ "After instantiation\n  u = " ++ show u ++ "\n  v = " ++ show v
+--   dirty <- use stDirty
+--   stDirty .= old
+--   if dirty then verboseS "profile.sharing" 20 (tick "unifyPtr: dirty")
+--            else do
+--             verboseS "profile.sharing" 20 (tick "unifyPtr: clean")
+--             reportSLn "tc.ptr.unify" 80 $ "Unifying\n  u = " ++ show u ++ "\n  v = " ++ show v
+--             forceEqualTerms u v
+--             reportSLn "tc.ptr.unify" 80 $ "After unification\n  u = " ++ show u ++ "\n  v = " ++ show v
 
 -- | Try to assign meta.  If meta is projected, try to eta-expand
 --   and run conversion check again.
@@ -238,6 +248,11 @@ compareTerm' cmp a m n =
     isSize   <- isJust <$> isSizeType a'
     s        <- reduce $ getSort a'
     mlvl     <- mlevel
+    reportSDoc "tc.conv.level" 60 $ nest 2 $ sep
+      [ text $ "a'   = " ++ show a'
+      , text $ "mlvl = " ++ show mlvl
+      , text $ "(Just (ignoreSharing $ unEl a') == mlvl) = " ++ show (Just (ignoreSharing $ unEl a') == mlvl)
+      ]
     case s of
       Prop | proofIrr -> return ()
       _    | isSize   -> compareSizes cmp m n
@@ -510,9 +525,16 @@ compareAtom cmp t m n =
                             (_, Apply arg : _) -> arg
                             _ -> __IMPOSSIBLE__
                   -- Infer its type.
+                  reportSDoc "tc.conv.infer" 30 $
+                    text "inferring type of internal arg: " <+> prettyTCM arg
                   targ <- infer $ unArg arg
+                  reportSDoc "tc.conv.infer" 30 $
+                    text "inferred type: " <+> prettyTCM targ
                   -- getDefType wants the argument type reduced.
-                  fromMaybeM __IMPOSSIBLE__ $ getDefType f =<< reduce targ
+                  -- Andreas, 2016-02-09, Issue 1825: The type of arg might be
+                  -- a meta-variable, e.g. in interactive development.
+                  -- In this case, we postpone.
+                  fromMaybeM patternViolation $ getDefType f =<< reduce targ
                 -- The polarity vector of projection-like functions
                 -- does not include the parameters.
                 pol <- getPolarity' cmp f
@@ -657,7 +679,7 @@ compareElims pols0 a v els01 els02 = catchConstraint (ElimCmp pols0 a v els01 el
                    then (arg1 $>) <$> blockTermOnProblem b (unArg arg1) pid
                    else return arg1
             -- continue, possibly with blocked instantiation
-            compareElims pols (piApply a [arg]) (apply v [arg]) els1 els2
+            compareElims pols (codom `lazyAbsApp` unArg arg) (apply v [arg]) els1 els2
             -- any left over constraints of arg are associatd to the comparison
             stealConstraints pid
 
@@ -716,30 +738,6 @@ compareElims pols0 a v els01 els02 = catchConstraint (ElimCmp pols0 a v els01 el
               ]
             patternViolation
 
-{-
-        res <- getDefType f a -- get type of projection (like) function
-        case res of
-          Just ft -> do
-            let arg = defaultArg v  -- we could get the proper Arg deco from ft
-                c   = ft `piApply` [arg]
-            u <- applyDef f arg     -- correct both for proj.s and non proj.s
-            (cmp, els1, els2) <- return $
-              case fst $ nextPolarity pols0 of
-                Invariant     -> (CmpEq , els1, els2)
-                Covariant     -> (CmpLeq, els1, els2)
-                Contravariant -> (CmpLeq, els2, els1)
-                Nonvariant    -> __IMPOSSIBLE__ -- the polarity should be Invariant
-            pols' <- getPolarity' cmp f
-            compareElims pols' c u els1 els2
-          _ -> do
-            reportSDoc "impossible" 10 $ sep
-              [ text $ "projection " ++ show f
-              , text   "applied to value " <+> prettyTCM v
-              , text   "of unexpected type " <+> prettyTCM a
-              ]
-            patternViolation
-            -- __IMPOSSIBLE__
--}
 
 -- | "Compare" two terms in irrelevant position.  This always succeeds.
 --   However, we can dig for solutions of irrelevant metas in the
@@ -877,9 +875,9 @@ coerce v t1 t2 = blockTerm t2 $ do
   if n <= 0 then fallback else do
     ifBlockedType b2 (\ _ _ -> fallback) $ \ _ -> do
       (args, t1') <- implicitArgs n (NotHidden /=) t1
-      coerceSize (v `apply` args) t1' t2
+      coerceSize leqType (v `apply` args) t1' t2
   where
-    fallback = coerceSize v t1 t2
+    fallback = coerceSize leqType v t1 t2
 
 -- | Account for situations like @k : (Size< j) <= (Size< k + 1)@
 --
@@ -889,8 +887,8 @@ coerce v t1 t2 = blockTerm t2 $ do
 --   TODO.
 --
 --   For now, we do a cheap heuristics.
-coerceSize :: Term -> Type -> Type -> TCM Term
-coerceSize v t1 t2 = workOnTypes $ do
+coerceSize :: (Type -> Type -> TCM ()) -> Term -> Type -> Type -> TCM Term
+coerceSize leqType v t1 t2 = workOnTypes $ do
     let fallback = v <$ leqType t1 t2
         done = caseMaybeM (isSizeType t1) fallback $ \ b1 -> return v
     -- Andreas, 2015-07-22, Issue 1615:
@@ -956,7 +954,7 @@ leqSort s1 s2 = catchConstraint (SortCmp CmpLeq s1 s2) $ do
       (SizeUniv, _       ) -> equalSort s1 s2
       (_       , SizeUniv) -> equalSort s1 s2
 
-      (Type a  , Type b  ) -> unlessM typeInType $ leqLevel a b
+      (Type a  , Type b  ) -> leqLevel a b
 
       (Prop    , Prop    ) -> yes
       (Prop    , Type _  ) -> yes
@@ -966,9 +964,9 @@ leqSort s1 s2 = catchConstraint (SortCmp CmpLeq s1 s2) $ do
       -- (SizeUniv, _       ) -> no
       -- (_       , SizeUniv) -> no
 
-      (Inf     , _       ) -> unlessM typeInType $ equalSort s1 s2
-      (DLub{}  , _       ) -> unlessM typeInType $ postpone
-      (_       , DLub{}  ) -> unlessM typeInType $ postpone
+      (Inf     , _       ) -> equalSort s1 s2
+      (DLub{}  , _       ) -> postpone
+      (_       , DLub{}  ) -> postpone
 
 leqLevel :: Level -> Level -> TCM ()
 leqLevel a b = liftTCM $ do
@@ -976,8 +974,11 @@ leqLevel a b = liftTCM $ do
     text "compareLevel" <+>
       sep [ prettyTCM a <+> text "=<"
           , prettyTCM b ]
-  a <- reduce a
-  b <- reduce b
+  -- Andreas, 2015-12-28 Issue 1757
+  -- We normalize both sides to make the syntactic equality check (==) stronger.
+  -- See case for `same term` below.
+  a <- normalise a
+  b <- normalise b
   catchConstraint (LevelCmp CmpLeq a b) $ leqView a b
   where
     leqView a@(Max as) b@(Max bs) = do
@@ -994,7 +995,7 @@ leqLevel a b = liftTCM $ do
         ([], _) -> ok
 
         -- as ≤ 0
-        (as, [])  -> sequence_ [ equalLevel (Max [a]) (Max []) | a <- as ]
+        (as, [])  -> sequence_ [ equalLevel' (Max [a]) (Max []) | a <- as ]
 
         -- as ≤ [b]
         (as@(_:_:_), [b]) -> sequence_ [ leqView (Max [a]) (Max [b]) | a <- as ]
@@ -1042,7 +1043,7 @@ leqLevel a b = liftTCM $ do
 
         -- [a] ≤ [neutral]
         ([a@(Plus n _)], [b@(Plus m NeutralLevel{})])
-          | m == n -> equalLevel (Max [a]) (Max [b])
+          | m == n -> equalLevel' (Max [a]) (Max [b])
           -- Andreas, 2014-04-07: This call to equalLevel is ok even if we removed
           -- subsumed terms from the lhs.
 
@@ -1050,7 +1051,7 @@ leqLevel a b = liftTCM $ do
         _ -> postpone
       where
         ok       = return ()
-        notok    = typeError $ NotLeqSort (Type a) (Type b)
+        notok    = unlessM typeInType $ typeError $ NotLeqSort (Type a) (Type b)
         postpone = patternViolation
 
         wrap m = catchError m $ \e ->
@@ -1086,6 +1087,11 @@ equalLevel :: Level -> Level -> TCM ()
 equalLevel a b = do
   -- Andreas, 2013-10-31 Use normalization to make syntactic equality stronger
   (a, b) <- normalise (a, b)
+  equalLevel' a b
+
+-- | Precondition: levels are 'normalise'd.
+equalLevel' :: Level -> Level -> TCM ()
+equalLevel' a b = do
   reportSLn "tc.conv.level" 50 $ "equalLevel (" ++ show a ++ ") (" ++ show b ++ ")"
   liftTCM $ catchConstraint (LevelCmp CmpEq a b) $
     check a b
@@ -1139,8 +1145,8 @@ equalLevel a b = do
         (_, [ClosedLevel{}]) | any isNeutral as -> notok
 
         -- 0 == any
-        ([ClosedLevel 0], bs@(_:_:_)) -> sequence_ [ equalLevel (Max []) (Max [b]) | b <- bs ]
-        (as@(_:_:_), [ClosedLevel 0]) -> sequence_ [ equalLevel (Max [a]) (Max []) | a <- as ]
+        ([ClosedLevel 0], bs@(_:_:_)) -> sequence_ [ equalLevel' (Max []) (Max [b]) | b <- bs ]
+        (as@(_:_:_), [ClosedLevel 0]) -> sequence_ [ equalLevel' (Max [a]) (Max []) | a <- as ]
         -- Andreas, 2014-04-07 Why should the following be ok?
         --   X (suc a)  could be different from  X (suc (suc a))
         -- -- Same meta
@@ -1175,13 +1181,14 @@ equalLevel a b = do
         _ -> postpone
 
       where
-        a === b   = do
+        a === b   = unlessM typeInType $ do
             lvl <- levelType
             equalAtom lvl a b
         as =!= bs = levelTm (Max as) === levelTm (Max bs)
 
         ok       = return ()
-        notok    = typeError $ UnequalSorts (Type a) (Type b)
+        notok    = unlessM typeInType notOk
+        notOk    = typeError $ UnequalSorts (Type a) (Type b)
         postpone = do
           reportSLn "tc.conv.level" 30 $ "postponing: " ++ show a ++ " == " ++ show b
           patternViolation
@@ -1204,7 +1211,7 @@ equalLevel a b = do
 
         subtr n (ClosedLevel m)
           | m >= n    = return $ ClosedLevel (m - n)
-          | otherwise = notok
+          | otherwise = ifM typeInType (return $ ClosedLevel 0) $ notOk
         subtr n (Plus m a)
           | m >= n    = return $ Plus (m - n) a
         subtr _ (Plus _ BlockedLevel{}) = postpone
@@ -1244,12 +1251,11 @@ equalLevel a b = do
 -- | Check that the first sort equal to the second.
 equalSort :: Sort -> Sort -> TCM ()
 equalSort s1 s2 = do
-  ifM typeInType (return ()) $
     catchConstraint (SortCmp CmpEq s1 s2) $ do
         (s1,s2) <- reduce (s1,s2)
         let postpone = addConstraint (SortCmp CmpEq s1 s2)
             yes      = return ()
-            no       = typeError $ UnequalSorts s1 s2
+            no       = unlessM typeInType $ typeError $ UnequalSorts s1 s2
 
             -- Test whether a level is infinity.
             isInf ClosedLevel{}   = no
